spool 10-package_centraldistribucio.log

CREATE OR REPLACE PACKAGE ELE.PKG_DISTRIBUCIO AS

--
-- NOM.........:   PKG_DISTRIBUCIO
-- FINALITAT...:   GESTIO DE TOTA LA INFORMACIO REFERENTS A CENTRALS DISTRIBUCIO.
--
--
--
--
-- NOM.........:   PR_ALTA_DISTRIBUCIO
-- FINALITAT...:   DONAR D'ALTA UNA CENTRAL DE DISTRIBUCIO
-- CONTROLS....:   INFORMAR CAMPS OBLIGATORIS
--                 POBLACIO EXISTENT
--              
-- PARAMETRES..:   
--         IN..:   idcentraleta: codi de la central de distribucio (Obligatori)
--                 adreca.....: adrea del client
--                 idPoblacio.: codi de la poblacio (Obligatori), a partir d'aquest codi obtindrem idProvincia i idPais.
--        OUT..:   RSP
--
-- PRECONDICIO.:   CENTRAL DISTRIBUCIO NO EXISTEIX
-- POSTCONDICIO:   ES DONA D'ALTA LA CENTRAL DE DISTRIBUCIO
-- RETURN .....:   OK SI S'HA EFECTUAT L'ALTA
--                 ERROR ALGUN CAMP DEL PARAMETRE NO TE VALOR
--                 ERROR REGISTRE DUPLICAT (CENTRAL DISTRIBUCIO EXISTENT)
--                 ERROR PROVINCIA NO EXISTENT
--
--

  PROCEDURE PR_ALTA_DISTRIBUCIO (inCentraleta     IN DISTRIBUCIO.IDCENTRALETA%TYPE,
                                 inAdreca         IN DISTRIBUCIO.ADRECA%TYPE,
                                 inPoblacio       IN DISTRIBUCIO.IDPOBLACIO%TYPE,
                                 RSP              OUT VARCHAR2);

--
-- NOM.........:   PR_BAIXA_DISTRIBUCIO
-- FINALITAT...:   DONAR DE BAIXA UNA CENTRAL DE DISTRIBUCIO
-- CONTROLS....:   CENTRAL NO DONADA DE BAIXA
--                 CENTRAL EXISTEIX EN EL FITXER
--                 CENTRAL SENSE COMPTADORS
--              
-- PARAMETRES..:   
--         IN..:   idDistribucio: Codi de la central de distribucio (Obligatori)
--        OUT..:   RSP
--
-- PRECONDICIO.:   CENTRAL DISTRIBUCIO EXISTEIX
-- POSTCONDICIO:   ES DONA DE BAIXA LA CENTRAL DISTRIBUCIO
-- RETURN .....:   OK SI S'HA EFECTUAT LA BAIXA
--                 ERROR SI LA CENTRAL DISTRIBUCIO NO EXISTEIX
--                 ERROR SI LA CENTRAL DISTRIBUCIO ESTA DONAT DE BAIXA
--                 ERROR SI LA CENTRAL DISTRIBUCIO TE COMPTADORS.
--
--

  PROCEDURE PR_BAIXA_DISTRIBUCIO (inDistribucio        IN DISTRIBUCIO.IDDISTRIBUCIO%TYPE,
                                  RSP                  OUT VARCHAR2);

--
-- NOM.........:   PR_MODIFICACIO_DISTRIBUCIO
-- FINALITAT...:   PODER MODIFICAR QUALSEVOL CAMP D'UNA CENTRAL DE DISTRIBUCIO
-- CONTROLS....:   CENTRAL DE DISTRIBUCIO EXISTENT EN EL FITXER
--                 INFORMAR CAMPS OBLIGATORIS
--              
-- PARAMETRES..:   
--         IN..:   idDistribucio: codi de la central distribucio (obligatori)
--                 idCentraleta.: Referencia de la central de distribucio
--                 adreca.....: adrea del client
--                 idPoblacio.: codi de la poblacio, a partir d'aquest codi obtindrem idProvincia i idPais.
--                 Actiu......: Canviar status de la central de distribucio
--                 DataAlta...: Canviar la data d'alta de la central de distribucio
--                 DataBaixa..: Canviar la data de baixa de la central de distribucio
--        OUT..:   RSP
--
-- PRECONDICIO.:   CENTRAL DISTRIBUCIO EXISTEIX
-- POSTCONDICIO:   ES REALITZA LA MODIFICACIO DELS CAMPS DE LA CENTRAL DE DISTRIBUCIO
-- RETURN .....:   OK SI S'HA EFECTUAT LA MODIFICACIO
--                 ERROR ALGUN CAMP DEL PARAMETRE NO TE VALOR
--                 ERROR REGISTRE DUPLICAT (CENTRAL DISTRIBUCIO EXISTENT)
--                 ERROR ALTA NO CONTE VALORS 0 - 1
--                 ERROR PROVINCIA NO EXISTENT
--
--

  PROCEDURE PR_MODIFICACIO_DISTRIBUCIO (inDistribucio    IN DISTRIBUCIO.IDDISTRIBUCIO%TYPE,
                                        inReferencia     IN DISTRIBUCIO.IDCENTRALETA%TYPE,
                                        inAdreca         IN DISTRIBUCIO.ADRECA%TYPE,
                                        inPoblacio       IN DISTRIBUCIO.IDPOBLACIO%TYPE,
                                        inDataAlta       IN DISTRIBUCIO.DATAALTA%TYPE,
                                        inDataBaixa      IN DISTRIBUCIO.DATABAIXA%TYPE,
                                        RSP              OUT VARCHAR2);

END PKG_DISTRIBUCIO;

--
--
--
CREATE OR REPLACE PACKAGE BODY ELE.PKG_DISTRIBUCIO AS

  PROCEDURE PR_ALTA_DISTRIBUCIO (inCentraleta     IN DISTRIBUCIO.IDCENTRALETA%TYPE,
                                 inAdreca         IN DISTRIBUCIO.ADRECA%TYPE,
                                 inPoblacio       IN DISTRIBUCIO.IDPOBLACIO%TYPE,
                                 RSP              OUT VARCHAR2)
  IS

   V_CAMP_NUL             EXCEPTION;
   V_CENTRAL_EXISTENT     EXCEPTION;
   V_POBLACIO_EXISTENT    EXCEPTION;
   V_PROVINCIA_EXISTENT   EXCEPTION;
   V_PAIS_EXISTENT        EXCEPTION;

   v_in          VARCHAR2(500);
   hihies        VARCHAR2(1);
   v_Provincia   NUMBER;
   v_Pais        NUMBER;
   err_num       NUMBER;
   err_msg       VARCHAR2(100);


   BEGIN
     v_in := 'Centraleta:'||inCentraleta||', Adreca:'||inAdreca||', Poblacio:'||inPoblacio;
--
-- Primera comprovacio que tots els camps obligatoris tinguin valor
--
     IF (TRIM(inCentraleta)     IS NULL) THEN
         RAISE V_CAMP_NUL;
     END IF;
     IF (TRIM(inPoblacio)     IS NULL) THEN
         RAISE V_CAMP_NUL;
     END IF;
--
-- Segona comprovacio que ei idCentraleta no es trovi amb un altre id
--
     BEGIN
       SELECT 'Y'
         INTO hihies
         FROM DISTRIBUCIO
        WHERE IDCENTRALETA  = inCentraleta
          AND ROWNUM  = 1;
        RAISE V_CENTRAL_EXISTENT;
    EXCEPTION
         WHEN NO_DATA_FOUND THEN NULL;
    END;
--
-- Tercera comprovacio que el IdPoblacio estigui donat d'alta
--
    BEGIN
	  SELECT idProvincia
        INTO v_Provincia
        FROM Poblacio
       WHERE idPoblacio = inPoblacio
         AND ROWNUM = 1;      
   EXCEPTION
        WHEN NO_DATA_FOUND THEN RAISE V_POBLACIO_EXISTENT;
   END;
--
-- Miro el Pais que te la privincia
--	
    BEGIN	
      SELECT idPais
        INTO v_Pais
        FROM Provincia
       WHERE idProvincia = v_Provincia
         AND ROWNUM = 1;      
   EXCEPTION
        WHEN NO_DATA_FOUND THEN RAISE V_PROVINCIA_EXISTENT;		  
   END;
--
-- Miro que el Pais estigui donat d'alta
--	
   BEGIN
     SELECT 'Y'
       INTO hihies
       FROM Pais
      WHERE idPais = v_Pais
        AND ROWNUM = 1;      
  EXCEPTION
       WHEN NO_DATA_FOUND THEN RAISE V_PAIS_EXISTENT;
  END;
--
-- Tots els controls superats.
--
   INSERT INTO DISTRIBUCIO (IDCENTRALETA, ADRECA, IDPAIS, IDPROVINCIA, IDPOBLACIO, ACTIU, DATAALTA)
          VALUES (inCentraleta, inAdreca, v_Pais, v_Provincia, inPoblacio, 1, SYSDATE);
   RSP := 'OK';
   INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
          VALUES (SYSDATE,'PKG_DISTRIBUCIO.PR_ALTA_DISTRIBUCIO', v_in, 'RSP: '||RSP);
   COMMIT;
   EXCEPTION
    WHEN V_CENTRAL_EXISTENT THEN
         RSP := 'ERROR: La central de distribucio ya esta donada d`alta';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
          VALUES (SYSDATE,'PKG_DISTRIBUCIO.PR_ALTA_DISTRIBUCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_POBLACIO_EXISTENT THEN
         RSP := 'ERROR: Poblacio Inexistent';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
          VALUES (SYSDATE,'PKG_DISTRIBUCIO.PR_ALTA_DISTRIBUCIO', v_in, 'RSP: '||RSP);
         COMMIT;	  
    WHEN V_PROVINCIA_EXISTENT THEN
         RSP := 'ERROR: Provincia Inexistent';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
          VALUES (SYSDATE,'PKG_DISTRIBUCIO.PR_ALTA_DISTRIBUCIO', v_in, 'RSP: '||RSP);
          COMMIT;	  
    WHEN V_PAIS_EXISTENT THEN
         RSP := 'ERROR: Pais Inexistent';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
          VALUES (SYSDATE,'PKG_DISTRIBUCIO.PR_ALTA_DISTRIBUCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_CAMP_NUL THEN
         RSP := 'ERROR: No pot haver camps NULL';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
          VALUES (SYSDATE,'PKG_DISTRIBUCIO.PR_ALTA_DISTRIBUCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN OTHERS THEN
         err_num := SQLCODE;
         err_msg := SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
          VALUES (SYSDATE,'PKG_DISTRIBUCIO.PR_ALTA_DISTRIBUCIO', v_in, 'RSP: '||RSP);
         COMMIT;
  END;

  PROCEDURE PR_BAIXA_DISTRIBUCIO (inDistribucio        IN DISTRIBUCIO.IDDISTRIBUCIO%TYPE,
                                  RSP                  OUT VARCHAR2)
  IS

   V_CAMP_NUL            EXCEPTION;
   V_CENTRAL_EXISTENT    EXCEPTION;
   V_CENTRAL_BAIXA       EXCEPTION;
   V_CENTRAL_COMPTADORS  EXCEPTION;
   
   v_in           VARCHAR2(500);
   hihies         VARCHAR2(1);
   v_actiu        NUMBER;
   v_consum       NUMBER;
   v_total        NUMBER;
   v_comptadors   NUMBER;
   OutId          NUMBER;
   hiTeContractes VARCHAR2(1);
   err_num        NUMBER;
   err_msg        VARCHAR2(100);


   BEGIN
     v_in := 'Central Distribucio:'||inDistribucio;
--
-- Primera comprovacio que tots els camps obligatoris tinguin valor
--
     IF (TRIM(inDistribucio)     IS NULL) THEN
         RAISE V_CAMP_NUL;
     END IF;
--
-- Segona comprovacio que el idDistribucio existeix
--
     BEGIN
       SELECT Actiu
         INTO V_actiu
         FROM DISTRIBUCIO
        WHERE IDDISTRIBUCIO  = inDistribucio
          AND ROWNUM  = 1;        
    EXCEPTION
         WHEN NO_DATA_FOUND THEN RAISE V_CENTRAL_EXISTENT;
    END;	 
--
-- Tercera comprovacio que no estigui de baixa
--	 
     BEGIN	 
   	   IF (V_ACTIU = 0) THEN
          RAISE V_CENTRAL_BAIXA;
       END IF;     
     END;
-- 
-- Quarta comprovaaci que no tingui comptadors actius.
--
     BEGIN
	    SELECT 'Y'
		  INTO hihies
		  FROM COMPTADORS
		 WHERE ACTIU = 1
		   AND IDCOMPTADOR IN (SELECT DISTINCT(IDCOMPTADOR) 
                                 FROM COMUNICACIODISTRIBUCIO
                                WHERE ACTIU = 1
                                  AND IDDISTRIBUCIO = inDistribucio)
		   AND ROWNUM = 1;
  	     RAISE V_CENTRAL_COMPTADORS;
 	 EXCEPTION
  	      WHEN NO_DATA_FOUND THEN NULL;
	 END;	 
--
-- Tots els controls superats.
--
      UPDATE DISTRIBUCIO
         SET ACTIU = 0,
             DATABAIXA = SYSDATE
       WHERE IDDISTRIBUCIO = INDISTRIBUCIO;
      RSP := 'OK';
      INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_DISTRIBUCIO.PR_BAIXA_DISTRIBUCIO', v_in, 'RSP: '||RSP);
      COMMIT;
  EXCEPTION
    WHEN V_CENTRAL_COMPTADORS THEN
         RSP := 'ERROR: La central de distribucio te comptadors actius';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_DISTRIBUCIO.PR_BAIXA_DISTRIBUCIO', v_in, 'RSP: '||RSP);
         COMMIT;	
    WHEN V_CENTRAL_EXISTENT THEN
         RSP := 'ERROR: La central de distribucio no esta donada d`alta';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_DISTRIBUCIO.PR_BAIXA_DISTRIBUCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_CENTRAL_BAIXA THEN
         RSP := 'ERROR: La central de distribucio ja esta esborrada';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_DISTRIBUCIO.PR_BAIXA_DISTRIBUCIO', v_in, 'RSP: '||RSP);
         COMMIT;	  
    WHEN V_CAMP_NUL THEN
         RSP := 'ERROR: No pot haver camps NULL';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_DISTRIBUCIO.PR_BAIXA_DISTRIBUCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN OTHERS THEN
         err_num := SQLCODE;
         err_msg := SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_DISTRIBUCIO.PR_BAIXA_DISTRIBUCIO', v_in, 'RSP: '||RSP);
         COMMIT;
  END;

  PROCEDURE PR_MODIFICACIO_DISTRIBUCIO (inDistribucio    IN DISTRIBUCIO.IDDISTRIBUCIO%TYPE,
                                        inReferencia     IN DISTRIBUCIO.IDCENTRALETA%TYPE,
                                        inAdreca         IN DISTRIBUCIO.ADRECA%TYPE,
                                        inPoblacio       IN DISTRIBUCIO.IDPOBLACIO%TYPE,
                                        inDataAlta       IN DISTRIBUCIO.DATAALTA%TYPE,
                                        inDataBaixa      IN DISTRIBUCIO.DATABAIXA%TYPE,
                                        RSP              OUT VARCHAR2)
   IS

   V_CAMP_NUL             EXCEPTION;
   V_CENTRAL_EXISTENT     EXCEPTION;
   V_CENTRAL_REPETIT      EXCEPTION;
   V_POBLACIO_EXISTENT    EXCEPTION;
   V_PROVINCIA_EXISTENT   EXCEPTION;
   V_PAIS_EXISTENT        EXCEPTION;

   v_in          VARCHAR2(500);
   hihies        VARCHAR2(1);
   v_Provincia   NUMBER;
   v_Pais        NUMBER;
   err_num       NUMBER;
   err_msg       VARCHAR2(100);

   BEGIN
     v_in := 'Central:'||inDistribucio||', Referencia:'||inReferencia||', Adreca:'||inAdreca||', Poblacio:'||inPoblacio||
	         ', Data Alta:'||inDataAlta||', Data Baixa:'||inDataBaixa;
--
-- Primera comprovacio que tots els camps obligatoris tinguin valor
--
     IF (TRIM(inDistribucio)     IS NULL) THEN
         RAISE V_CAMP_NUL;
     END IF;
--
-- Segona comprovacio que el idClient existeix
--
     BEGIN
       SELECT idPais,idProvincia
         INTO v_Pais,v_Provincia
         FROM DISTRIBUCIO
        WHERE IDDISTRIBUCIO  = inDistribucio
          AND ROWNUM  = 1;        
    EXCEPTION
         WHEN NO_DATA_FOUND THEN RAISE V_CENTRAL_EXISTENT;
    END;	 	 
--
-- Tercera comprovacio que la referencia no es trovi amb un altre id
--
     BEGIN
       SELECT 'Y'
         INTO hihies
         FROM DISTRIBUCIO
        WHERE IDCENTRALETA  = inReferencia
          AND IDDISTRIBUCIO <> inDistribucio
          AND ROWNUM  = 1;
        RAISE V_CENTRAL_REPETIT;
    EXCEPTION
         WHEN NO_DATA_FOUND THEN NULL;
    END;
--
-- Quarta comprovacio que el IdPoblacio estigui donat d'alta si s'ha canviat
--
    IF (TRIM(inPoblacio)     IS NOT NULL) THEN
       BEGIN
           SELECT idProvincia
             INTO v_Provincia
             FROM Poblacio
            WHERE idPoblacio = inPoblacio
              AND ROWNUM = 1;      
       EXCEPTION
            WHEN NO_DATA_FOUND THEN RAISE V_POBLACIO_EXISTENT;
       END;
       --
       BEGIN	
          SELECT idPais
            INTO v_Pais
            FROM Provincia
           WHERE idProvincia = v_Provincia
             AND ROWNUM = 1;      
       EXCEPTION
            WHEN NO_DATA_FOUND THEN RAISE V_PROVINCIA_EXISTENT;		  
       END;
       --
       BEGIN
      	  SELECT 'Y'
            INTO hihies
            FROM Pais
           WHERE idPais = v_Pais
             AND ROWNUM = 1;      
       EXCEPTION
            WHEN NO_DATA_FOUND THEN RAISE V_PAIS_EXISTENT;
       END;	   	   
    END IF;	
--
-- Tots els controls superats.
--
      UPDATE DISTRIBUCIO SET
	         idCentraleta = NVL(inReferencia, idCentraleta),
    	     Adreca = NVL(inAdreca, Adreca),
    		 idPais = NVL(v_Pais, idPais),
	    	 idProvincia = NVL(v_Provincia, idProvincia), 
		     idPoblacio =  NVL(idPoblacio, idPoblacio),
	     	 DataAlta = NVL(inDataAlta, DataAlta),
		     DataBaixa = NVL(inDataBaixa, DataBaixa)
	    WHERE idDistribucio = inDistribucio;
      RSP := 'OK';
      INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_DISTRIBUCIO.PR_MODIFICACIO_DISTRIBUCIO', v_in, 'RSP: '||RSP);

      COMMIT;
  EXCEPTION
    WHEN V_CENTRAL_REPETIT THEN
         RSP := 'ERROR: La Referencia ya esta assignat a una altra central de distribucio';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_DISTRIBUCIO.PR_MODIFICACIO_DISTRIBUCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_CENTRAL_EXISTENT THEN
         RSP := 'ERROR: La Central de distribucio no esta donada d`alta';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_DISTRIBUCIO.PR_MODIFICACIO_DISTRIBUCIO', v_in, 'RSP: '||RSP);
         COMMIT;	  
    WHEN V_POBLACIO_EXISTENT THEN
         RSP := 'ERROR: Poblacio Inexistent';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_DISTRIBUCIO.PR_MODIFICACIO_DISTRIBUCIO', v_in, 'RSP: '||RSP);
         COMMIT;	  
    WHEN V_PROVINCIA_EXISTENT THEN
         RSP := 'ERROR: Provincia Inexistent';  
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_DISTRIBUCIO.PR_MODIFICACIO_DISTRIBUCIO', v_in, 'RSP: '||RSP);
         COMMIT;	  
    WHEN V_PAIS_EXISTENT THEN
         RSP := 'ERROR: Pais Inexistent';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_DISTRIBUCIO.PR_MODIFICACIO_DISTRIBUCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_CAMP_NUL THEN
         RSP := 'ERROR: No pot haver camps NULL';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_DISTRIBUCIO.PR_MODIFICACIO_DISTRIBUCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN OTHERS THEN
         err_num := SQLCODE;
         err_msg := SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_DISTRIBUCIO.PR_MODIFICACIO_DISTRIBUCIO', v_in, 'RSP: '||RSP);
         COMMIT;
  END;
END PKG_DISTRIBUCIO;
--
-- FI DEL PACKAGE
--
spool off;
