spool 11-package_centralproduccio.log

CREATE OR REPLACE PACKAGE ELE.PKG_PRODUCCIO AS

--
-- NOM.........:   PKG_PRODUCCIO
-- FINALITAT...:   GESTIO DE TOTA LA INFORMACIO REFERENTS A CENTRALS PRODUCCIO
--
--
--
--
-- NOM.........:   PR_ALTA_PRODUCCIO
-- FINALITAT...:   DONAR D'ALTA UNA CENTRAL DE PRODUCCIO
-- CONTROLS....:   INFORMAR CAMPS OBLIGATORIS
--                 CODI CENTRAL PRODUCCIO
--                 POBLACIO EXISTENT
--                 ENERGIA MAXIMA A PRODUIR
--                 CODI TIPUS CENTRAL
--              
-- PARAMETRES..:   
--         IN..:   idCodiCentral: codi de la central de produccio (Obligatori)
--                 adreca.......: adrea del client
--                 idPoblacio...: codi de la poblacio (Obligatori), a partir d'aquest codi obtindrem idProvincia i idPais.
--                 energiamaxima: Energia maxima que es pot produir.
--                 idCentral....: codi del tipus de central
--        OUT..:   RSP
--
-- PRECONDICIO.:   CENTRAL PRODUCCIO NO EXISTEIX
-- POSTCONDICIO:   ES DONA D'ALTA LA CENTRAL DE PRODUCCIO
-- RETURN .....:   OK SI S'HA EFECTUAT L'ALTA
--                 ERROR ALGUN CAMP DEL PARAMETRE NO TE VALOR
--                 ERROR REGISTRE DUPLICAT (CENTRAL PRODUCCIO EXISTENT)
--                 ERROR PROVINCIA NO EXISTENT
--                 ERROR TIPUS CENTRAL NO EXISTENT
--
--

  PROCEDURE PR_ALTA_PRODUCCIO (inCodiCentral     IN PRODUCCIO.IDCODICENTRAL%TYPE,
                               inAdreca          IN PRODUCCIO.ADRECA%TYPE,
                               inPoblacio        IN PRODUCCIO.IDPOBLACIO%TYPE,
                               inMaxima          IN PRODUCCIO.ENERGIAMAXIMA%TYPE,      
                               inCentral         IN PRODUCCIO.IDCENTRAL%TYPE,
                               RSP               OUT VARCHAR2);

--
-- NOM.........:   PR_BAIXA_PRODUCCIO.
-- FINALITAT...:   DONAR DE BAIXA UNA CENTRAL DE PRODUCCIO.
-- CONTROLS....:   CENTRAL NO DONADA DE BAIXA.
--                 CENTRAL EXISTEIX EN EL FITXER.
--              
-- PARAMETRES..:   
--         IN..:   idProduccio Codi de la central de produccio (Obligatori)
--        OUT..:   RSP
--
-- PRECONDICIO.:   CENTRAL PRODUCCIO EXISTEIX
-- POSTCONDICIO:   ES DONA DE BAIXA LA CENTRAL PRODUCCIO
-- RETURN .....:   OK SI S'HA EFECTUAT LA BAIXA
--                 ERROR SI LA CENTRAL PRODUCCIO NO EXISTEIX
--                 ERROR SI LA CENTRAL PRODUCCIO ESTA DONAT DE BAIXA
--
--

  PROCEDURE PR_BAIXA_PRODUCCIO (inProduccio      IN PRODUCCIO.IDPRODUCCIO%TYPE,
                                RSP              OUT VARCHAR2);

--
-- NOM.........:   PR_MODIFICACIO_PRODUCCIO
-- FINALITAT...:   PODER MODIFICAR QUALSEVOL CAMP D'UNA CENTRAL DE PRODUCCIO
-- CONTROLS....:   CENTRAL DE PRODUCCCIO EXISTENT EN EL FITXER
--                 INFORMAR CAMPS OBLIGATORIS
--              
-- PARAMETRES..:   
--         IN..:   idProduccio..: codi de la central produccio (obligatori)
--                 idCodiCentral: Referencia de la central de produccio
--                 adreca.......: adrea del client
--                 idPoblacio...: codi de la poblacio, a partir d'aquest codi obtindrem idProvincia i idPais.
--                 energiaMaxima: maxima energia posible a fabricar.
--                 idcentral....: codi del tipus de central.
--                 Actiu........: Canviar status de la central de distribucio
--                 DataAlta.....: Canviar la data d'alta de la central de distribucio
--                 DataBaixa....: Canviar la data de baixa de la central de distribucio
--        OUT..:   RSP
--
-- PRECONDICIO.:   CENTRAL PRODUCCIO EXISTEIX
-- POSTCONDICIO:   ES REALITZA LA MODIFICACIO DELS CAMPS DE LA CENTRAL DE PRODUCCIO
-- RETURN .....:   OK SI S'HA EFECTUAT LA MODIFICACIO
--                 ERROR ALGUN CAMP DEL PARAMETRE NO TE VALOR
--                 ERROR REGISTRE DUPLICAT (CENTRAL PRODUCCIO EXISTENT)
--                 ERROR ALTA NO CONTE VALORS 0 - 1
--                 ERROR PROVINCIA NO EXISTENT
--                 ERROR TIPUS DE CENTRAL NO EXISTENT
--
--

  PROCEDURE PR_MODIFICACIO_PRODUCCIO (inProduccio    IN PRODUCCIO.IDPRODUCCIO%TYPE,
                                       inCodiCentral  IN PRODUCCIO.IDCODICENTRAL%TYPE,
                                       inAdreca       IN PRODUCCIO.ADRECA%TYPE,
                                       inPoblacio     IN PRODUCCIO.IDPOBLACIO%TYPE,
                                       inMaxima       IN PRODUCCIO.ENERGIAMAXIMA%TYPE,
                                       inCentral      IN PRODUCCIO.IDCENTRAL%TYPE,
                                       inDataAlta     IN PRODUCCIO.DATAALTA%TYPE,
                                       inDataBaixa    IN PRODUCCIO.DATABAIXA%TYPE,
                                       RSP            OUT VARCHAR2);

END PKG_PRODUCCIO;

--
--
--
CREATE OR REPLACE PACKAGE BODY ELE.PKG_PRODUCCIO AS

  PROCEDURE PR_ALTA_PRODUCCIO (inCodiCentral     IN PRODUCCIO.IDCODICENTRAL%TYPE,
                               inAdreca          IN PRODUCCIO.ADRECA%TYPE,
                               inPoblacio        IN PRODUCCIO.IDPOBLACIO%TYPE,
                               inMaxima          IN PRODUCCIO.ENERGIAMAXIMA%TYPE,      
                               inCentral         IN PRODUCCIO.IDCENTRAL%TYPE,
                               RSP               OUT VARCHAR2)
  IS

   V_CAMP_NUL             EXCEPTION;
   V_CENTRAL_EXISTENT     EXCEPTION;
   V_POBLACIO_EXISTENT    EXCEPTION;
   V_PROVINCIA_EXISTENT   EXCEPTION;
   V_PAIS_EXISTENT        EXCEPTION;
   V_TIPUS_EXISTENT       EXCEPTION;

   v_in          VARCHAR2(500);
   hihies        VARCHAR2(1);
   v_Provincia   NUMBER;
   v_Pais        NUMBER;
   err_num       NUMBER;
   err_msg       VARCHAR2(100);

   BEGIN
     v_in := 'Codi central:'||inCodiCentral||', Adreca:'||inAdreca||', Poblacio:'||inPoblacio||', Produccio maxima:'||inMaxima||
             'codi tipus central:'||inCentral;
--
-- Primera comprovacio que tots els camps obligatoris tinguin valor
--
     IF (TRIM(inCodiCentral)     IS NULL) THEN
         RAISE V_CAMP_NUL;
     END IF;
     IF (TRIM(inPoblacio)     IS NULL) THEN
         RAISE V_CAMP_NUL;
     END IF;
     IF (TRIM(inMaxima)     IS NULL) THEN
         RAISE V_CAMP_NUL;
     END IF;
     IF (TRIM(inCentral)     IS NULL) THEN
         RAISE V_CAMP_NUL;
     END IF;
--
-- Segona comprovacio que el idCodiCentral no es trovi amb un altre id
--
     BEGIN

       SELECT 'Y'
         INTO hihies
         FROM PRODUCCIO
        WHERE IDCODICENTRAL  = inCodiCentral
          AND ROWNUM  = 1;
        RAISE V_CENTRAL_EXISTENT;
    EXCEPTION
         WHEN NO_DATA_FOUND THEN NULL;
    END;
--
-- Tercera comprovacio que el IdPoblacio estigui donat d'alta
--
    BEGIN
   	  SELECT idProvincia
        INTO v_Provincia
        FROM Poblacio
       WHERE idPoblacio = inPoblacio
         AND ROWNUM = 1;      
   EXCEPTION
        WHEN NO_DATA_FOUND THEN RAISE V_POBLACIO_EXISTENT;
   END;
--
-- Miro el Pais que te la privincia
--	
    BEGIN	
	  SELECT idPais
        INTO v_Pais
        FROM Provincia
       WHERE idProvincia = v_Provincia
         AND ROWNUM = 1;      
   EXCEPTION
        WHEN NO_DATA_FOUND THEN RAISE V_PROVINCIA_EXISTENT;		  
   END;
--
-- Miro que el Pais estigui donat d'alta
--	
   BEGIN
      SELECT 'Y'
        INTO hihies
        FROM Pais
       WHERE idPais = v_Pais
         AND ROWNUM = 1;      
   EXCEPTION
        WHEN NO_DATA_FOUND THEN RAISE V_PAIS_EXISTENT;
   END;
--
-- Quarta comprovacio que el tipus de central estigui donat d'alta
--
   BEGIN
     SELECT 'Y'
       INTO hihies
       FROM CENTRAL
      WHERE idCentral = inCentral
        AND ROWNUM = 1;      
  EXCEPTION
       WHEN NO_DATA_FOUND THEN RAISE V_TIPUS_EXISTENT;
  END;
--
-- Tots els controls superats.
--
   INSERT INTO PRODUCCIO (IDCODICENTRAL, ADRECA, IDPAIS, IDPROVINCIA, IDPOBLACIO, ENERGIAMAXIMA, DATADARRERAINSPE, CONSUMACTUAL, IDCENTRAL, CONSUMSCOMPTADOR,ACTIU, DATAALTA)
          VALUES (inCodiCentral, inAdreca, v_Pais, v_Provincia, inPoblacio, inMaxima, SYSDATE, 0, inCentral, 0, 1, SYSDATE);
   RSP := 'OK';
   INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
          VALUES (SYSDATE,'PKG_PRODUCCIO.PR_ALTA_PRODUCCIO', v_in, 'RSP: '||RSP);
   COMMIT;
  EXCEPTION
    WHEN V_CENTRAL_EXISTENT THEN
         RSP := 'ERROR: La central de produccio ya esta donada d`alta';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
          VALUES (SYSDATE,'PKG_PRODUCCIO.PR_ALTA_PRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_POBLACIO_EXISTENT THEN
         RSP := 'ERROR: Poblacio Inexistent';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
          VALUES (SYSDATE,'PKG_PRODUCCIO.PR_ALTA_PRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;	  
    WHEN V_PROVINCIA_EXISTENT THEN
         RSP := 'ERROR: Provincia Inexistent';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
          VALUES (SYSDATE,'PKG_PRODUCCIO.PR_ALTA_PRODUCCIO', v_in, 'RSP: '||RSP);
          COMMIT;	  
    WHEN V_PAIS_EXISTENT THEN
         RSP := 'ERROR: Pais Inexistent';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
          VALUES (SYSDATE,'PKG_PRODUCCIO.PR_ALTA_PRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_TIPUS_EXISTENT THEN
         RSP := 'ERROR: Tipus de central Inexistent';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
          VALUES (SYSDATE,'PKG_PRODUCCIO.PR_ALTA_PRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_CAMP_NUL THEN
         RSP := 'ERROR: No pot haver camps NULL';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
          VALUES (SYSDATE,'PKG_PRODUCCIO.PR_ALTA_PRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN OTHERS THEN
         err_num := SQLCODE;
         err_msg := SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
          VALUES (SYSDATE,'PKG_PRODUCCIO.PR_ALTA_PRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;
  END;

  PROCEDURE PR_BAIXA_PRODUCCIO (inProduccio      IN PRODUCCIO.IDPRODUCCIO%TYPE,
                                RSP              OUT VARCHAR2)
  IS

   V_CAMP_NUL            EXCEPTION;
   V_CENTRAL_EXISTENT    EXCEPTION;
   V_CENTRAL_BAIXA       EXCEPTION;

   v_in           VARCHAR2(500);
   v_actiu        NUMBER;
   v_total        NUMBER;
   v_produccio    NUMBER;
   hiTeContractes VARCHAR2(1);
   err_num        NUMBER;
   err_msg        VARCHAR2(100);

   BEGIN
     v_in := 'Central Produccio:'||inProduccio;
--
-- Primera comprovacio que tots els camps obligatoris tinguin valor
--
     IF (TRIM(inProduccio)     IS NULL) THEN
         RAISE V_CAMP_NUL;
     END IF;
--
-- Segona comprovacio que el idProduccio existeix
--
     BEGIN
       SELECT Actiu,ConsumActual
         INTO V_actiu,v_produccio
         FROM PRODUCCIO
        WHERE IDPRODUCCIO  = inProduccio
          AND ROWNUM  = 1;        
    EXCEPTION
         WHEN NO_DATA_FOUND THEN RAISE V_CENTRAL_EXISTENT;
    END;	 
--
-- Tercera comprovacio que no estigui de baixa
--	 
     BEGIN	 
   	   IF (V_ACTIU = 0) THEN
          RAISE V_CENTRAL_BAIXA;
       END IF;     
     END;
--	 
-- Actualitzto temes estadistics.
--
   IF (V_PRODUCCIO > 0) THEN
      BEGIN
         SELECT Produccio
           INTO v_total
           FROM HISTORICTOTAL
          WHERE ROWNUM = 1;	  
    	 -- HIHES L'ANY UPDATE.
	     UPDATE HISTORICTOTAL 
    	    SET PRODUCCIO = PRODUCCIO - v_produccio;		
    	 v_total := v_total - v_Produccio;
    	 RSP := 'OK UPDATE HISTORIC TOTAL';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
              VALUES (SYSDATE,'PKG_PRODUCCIO.PR_BAIXA_PRODUCCIO', v_in, 'RSP: '||RSP);	  
      EXCEPTION
        -- ALTA.
        WHEN NO_DATA_FOUND THEN 	 
     	 INSERT INTO HISTORICTOTAL (CONSUM, PRODUCCIO) VALUES (0, v_Produccio);	 
    	 v_total := v_Produccio;	 
     	 RSP := 'OK INSERT HISTORIC TOTAL'; 
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
              VALUES (SYSDATE,'PKG_PRODUCCIO.PR_BAIXA_PRODUCCIO', v_in, 'RSP: '||RSP);		   
      END;
      -- Anual.
	  UPDATE HISTORICANY
	     SET PRODUCCIO = PRODUCCIO - v_produccio
  	   WHERE BD = 1
	     AND IDCLAU = inProduccio;
	   -- linia comunicacio
	   UPDATE LINIACOMUNICACIO
	      SET CAPACITATPRODUIDA = CAPACITATPRODUIDA - v_produccio
		WHERE IDLINIACOMUNICACIO IN (SELECT IDLINIACOMUNICACIO FROM COMUNICACIOPRODUCCIO WHERE ACTIU = 1 AND IDPRODUCCIO = inProduccio);  	  
   END IF; 
--
-- Tots els controls superats.
--
      UPDATE PRODUCCIO
         SET ACTIU = 0,
             DATABAIXA = SYSDATE
       WHERE IDPRODUCCIO = INPRODUCCIO;
      RSP := 'OK';
      INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_PRODUCCIO.PR_BAIXA_PRODUCCIO', v_in, 'RSP: '||RSP);
      COMMIT;
  EXCEPTION
    WHEN V_CENTRAL_EXISTENT THEN
         RSP := 'ERROR: La central de produccio no esta donada d`alta';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_PRODUCCIO.PR_BAIXA_PRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_CENTRAL_BAIXA THEN
         RSP := 'ERROR: La central de produccio ja esta esborrada';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_PRODUCCIO.PR_BAIXA_PRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;	  
    WHEN V_CAMP_NUL THEN
         RSP := 'ERROR: No pot haver camps NULL';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_PRODUCCIO.PR_BAIXA_PRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN OTHERS THEN
         err_num := SQLCODE;
         err_msg := SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_PRODUCCIO.PR_BAIXA_PRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;
  END;
  
  PROCEDURE PR_MODIFICACIO_PRODUCCIO (inProduccio    IN PRODUCCIO.IDPRODUCCIO%TYPE,
                                       inCodiCentral  IN PRODUCCIO.IDCODICENTRAL%TYPE,
                                       inAdreca       IN PRODUCCIO.ADRECA%TYPE,
                                       inPoblacio     IN PRODUCCIO.IDPOBLACIO%TYPE,
                                       inMaxima       IN PRODUCCIO.ENERGIAMAXIMA%TYPE,
                                       inCentral      IN PRODUCCIO.IDCENTRAL%TYPE,
                                       inDataAlta     IN PRODUCCIO.DATAALTA%TYPE,
                                       inDataBaixa    IN PRODUCCIO.DATABAIXA%TYPE,
                                       RSP            OUT VARCHAR2)
   IS

   V_CAMP_NUL             EXCEPTION;
   V_CENTRAL_EXISTENT     EXCEPTION;
   V_CENTRAL_REPETIT      EXCEPTION;
   V_POBLACIO_EXISTENT    EXCEPTION;
   V_PROVINCIA_EXISTENT   EXCEPTION;
   V_PAIS_EXISTENT        EXCEPTION;
   V_TIPUS_EXISTENT       EXCEPTION;

   v_in          VARCHAR2(500);
   hihies        VARCHAR2(1);
   v_Provincia   NUMBER;
   v_Pais        NUMBER;
   err_num       NUMBER;
   err_msg       VARCHAR2(100);

   BEGIN
     v_in := 'Central:'||inProduccio||', Referencia:'||inCodiCentral||', Adreca:'||inAdreca||', Poblacio:'||inPoblacio||
             ', Energia maxima :'||inMaxima||', Tipus central:'||inCentral||
             ', Data Alta:'||inDataAlta||', Data Baixa:'||inDataBaixa;
--
-- Primera comprovacio que tots els camps obligatoris tinguin valor
--
     IF (TRIM(inProduccio)     IS NULL) THEN
         RAISE V_CAMP_NUL;
     END IF;
--
-- Segona comprovacio que el idProduccio existeix
--
     BEGIN
       SELECT idPais,idProvincia
         INTO v_Pais,v_Provincia
         FROM PRODUCCIO
        WHERE IDPRODUCCIO  = inProduccio
          AND ROWNUM  = 1;        
    EXCEPTION
         WHEN NO_DATA_FOUND THEN RAISE V_CENTRAL_EXISTENT;
    END;	 	 
--
-- Tercera comprovacio que la referencia no es trovi amb un altre id
--
     BEGIN
       SELECT 'Y'
         INTO hihies
         FROM PRODUCCIO
        WHERE IDCODICENTRAL  = inCodiCentral
          AND IDPRODUCCIO <> inProduccio
          AND ROWNUM  = 1;
        RAISE V_CENTRAL_REPETIT;
     EXCEPTION
          WHEN NO_DATA_FOUND THEN NULL;
     END;
--
-- Quarta comprovacio que el IdPoblacio estigui donat d'alta si s'ha canviat
--
    IF (TRIM(inPoblacio)     IS NOT NULL) THEN
       BEGIN
           SELECT idProvincia
             INTO v_Provincia
             FROM Poblacio
            WHERE idPoblacio = inPoblacio
              AND ROWNUM = 1;      
       EXCEPTION
            WHEN NO_DATA_FOUND THEN RAISE V_POBLACIO_EXISTENT;
       END;
       --
       BEGIN	
          SELECT idPais
            INTO v_Pais
            FROM Provincia
           WHERE idProvincia = v_Provincia
             AND ROWNUM = 1;      
       EXCEPTION
            WHEN NO_DATA_FOUND THEN RAISE V_PROVINCIA_EXISTENT;		  
       END;
       --
       BEGIN
      	  SELECT 'Y'
            INTO hihies
            FROM Pais
           WHERE idPais = v_Pais
             AND ROWNUM = 1;      
       EXCEPTION
            WHEN NO_DATA_FOUND THEN RAISE V_PAIS_EXISTENT;
       END;	   	   
    END IF;	
--
-- Cinquena comprovacio que el tipus de central estigui donat d'alta si s'ha canviat
--
    IF (TRIM(inCentral)     IS NOT NULL) THEN
       BEGIN
      	  SELECT 'Y'
            INTO hihies
            FROM CENTRAL
           WHERE idCentral = inCentral
             AND ROWNUM = 1;      
       EXCEPTION
            WHEN NO_DATA_FOUND THEN RAISE V_TIPUS_EXISTENT;
       END;	   	   
    END IF;
--
-- Tots els controls superats.
--
      UPDATE PRODUCCIO SET
	         idCodiCentral = NVL(inCodiCentral, idCodiCentral),
             Adreca = NVL(inAdreca, Adreca),
    		 idPais = NVL(v_Pais, idPais),
	    	 idProvincia = NVL(v_Provincia, idProvincia), 
     		 idPoblacio =  NVL(idPoblacio, idPoblacio),
             EnergiaMaxima = NVL(inMaxima, EnergiaMaxima),
             idCentral = NVL(inCentral, idCentral),
	     	 DataAlta = NVL(inDataAlta, DataAlta),
             DataBaixa = NVL(inDataBaixa, DataBaixa)
	    WHERE idProduccio = inProduccio;
      RSP := 'OK';
      INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_PRODUCCIO.PR_MODIFICACIO_PRODUCCIO', v_in, 'RSP: '||RSP);
      COMMIT;
  EXCEPTION
    WHEN V_CENTRAL_REPETIT THEN
         RSP := 'ERROR: La Referencia ya esta assignat a una altra central de produccio';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_PRODUCCIO.PR_MODIFICACIO_PRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_CENTRAL_EXISTENT THEN
         RSP := 'ERROR: La Central de produccio no esta donada d`alta';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_PRODUCCIO.PR_MODIFICACIO_PRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;	  
    WHEN V_POBLACIO_EXISTENT THEN
         RSP := 'ERROR: Poblacio Inexistent';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_PRODUCCIO.PR_MODIFICACIO_PRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;	  
    WHEN V_PROVINCIA_EXISTENT THEN
         RSP := 'ERROR: Provincia Inexistent';  
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_PRODUCCIO.PR_MODIFICACIO_PRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;	  
    WHEN V_PAIS_EXISTENT THEN
         RSP := 'ERROR: Pais Inexistent';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_PRODUCCIO.PR_MODIFICACIO_PRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_TIPUS_EXISTENT THEN
         RSP := 'ERROR: Tipus de central Inexistent';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_PRODUCCIO.PR_MODIFICACIO_PRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_CAMP_NUL THEN
         RSP := 'ERROR: No pot haver camps NULL';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_PRODUCCIO.PR_MODIFICACIO_PRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN OTHERS THEN
         err_num := SQLCODE;
         err_msg := SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_PRODUCCIO.PR_MODIFICACIO_PRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;
  END;								   
END PKG_PRODUCCIO;
--
-- FI DEL PACKAGE
--
spool off;
