spool 12-package_comunicacioproduccio.log

CREATE OR REPLACE PACKAGE ELE.PKG_COMUNICACIOPRODUCCIO AS

--
-- NOM.........:   PKG_COMUNICACIOPRODUCCIO
-- FINALITAT...:   GESTIO DE TOTA LA INFORMACIO REFERENTS AL SUMINISTRAMENT D'ENERIGIA
--
--
--
--
-- NOM.........:   PR_ALTA_COMUNICACIOPRODUCCIO.
-- FINALITAT...:   DONAR D'ALTA UN NOVA LINIA DE SUMINISTRAMENT.
-- CONTROLS....:   INFORMAR CAMPS OBLIGATORIS.
--                 QUE LA CENTRAL DE PRODUCCIO ESTIGUI DONADA D'ALTA.
--                 QUE LA CENTRAL DE PRODUCCIO ESTIGUI ACTIVA.
--                 QUE LA CENTRAL DE PRODUCCIO NO TINGUI UN ALTRA LINIA DE PRODUCCIO. 
--                 QUE LA LINIA DE PRODUCCIO ESTIGUI DONDA D'ALTA.
--                 QUE LA LINIA DE PRODUCCIO ESTIGUI ACTIVA.
--                 QUE LA CENTRAL DE DISTRIBUCIO ESTIGUI DONDA D'ALTA.
--                 QUE LA CENTRAL DE DISTRIBUCIO ESTIGUI ACTIVA.

--              
-- PARAMETRES..:   
--         IN..:   idDistribucio.....: Codi de la central de distribucio (obligatori)
--	    	       idliniacomunicacio: codi de la linia de comunicacio (Obliagtoir)
--                 idproduccio ......: codi de la central de produccio (Obligatori)
--        OUT..:   RSP
--
-- PRECONDICIO.:   LINIA DISTRIBUCIO EXISTEIX
--                 CENTRAL DISTRIBUCIO EXISTEIX
--                 CENTRAL COMUNICACIO EXISTEIX
-- POSTCONDICIO:   ES DONA D'ALTA UNA NOVA LINIA DE DISTRIBUCIO
-- RETURN .....:   OK SI S'HA EFECTUAT L'ALTA
--                 ERROR ALGUN CAMP DEL PARAMETRE NO TE VALOR
--                 ERROR SI LINIA NO EXISTEIX
--                 ERROR SI LINIA INACTIVA
--                 ERORR SI CENTRAL DISTRIBUCIO NO EXISTEIX
--                 ERORR SI CENTRAL DISTRIBUCIO INACTIVA
--                 ERROR SI CENTRAL PRODUCCIO NO EXISTEIX
--                 ERROR SI CENTRAL PRODUCCIO INACTIVA
--
--

PROCEDURE PR_ALTA_COMUNICACIOPRODUCCIO (inDistribucio     IN COMUNICACIOPRODUCCIO.IDDISTRIBUCIO%TYPE,
                                        inLinia           IN COMUNICACIOPRODUCCIO.IDLINIACOMUNICACIO%TYPE,
                                        inProduccio       IN COMUNICACIOPRODUCCIO.IDPRODUCCIO%TYPE,
                                        RSP               OUT VARCHAR2);

--
-- NOM.........:   PR_BAIXA_COMUNICACIOPRODUCCIO
-- FINALITAT...:   DONAR DE BAIXA UNA COMUNICACIO DE PRODUCCIO.
-- CONTROLS....:   INFORMAR CAMPS OBLIGATORIS
--                 QUE EL ID ESTIGUI EN LA BASE DE DADES
--                 QUE EL ID NO ESTIGUI JA ESBORRAT
--                 QUE LA CENTRAL DE PRODUCCIO ESTIGUI INACTIVA
--                 QUE LA LINIA DE PRODUCCIO ESTIGUI INACTIVA
--                 QUE LA CENTRAL DE DISTRIBUCIO ESTIGUI INACTIVA

--              
-- PARAMETRES..:   
--         IN..:   id....: Codi de la Comunicacio produccio.
--        OUT..:   RSP
--
-- PRECONDICIO.:   QUE LA COMUNICACIO PRODUCCIO EXISTEIX.
--                 CENTRAL DISTRIBUCIO EN BAIXA.
--                 CENTRAL PRODUCCIO EN BAIXA.
--                 LINIA DE COMUNICACIO EN BAIXA.
-- POSTCONDICIO:   ES DONA DE BAIXA UNA COMUNICACIO PRODUCCIO.
-- RETURN .....:   OK SI S'HA EFECTUAT LA BAIXA.
--                 ERROR ALGUN CAMP DEL PARAMETRE NO TE VALOR.
--                 ERROR SI COMUNICACIO PRODUCCIO NO EXISTEIX.
--                 ERROR SI COMUNICACIO PRODUCCIO INACTIVA.
--                 ERORR SI CENTRAL DISTRIBUCIO ACTIVA.
--                 ERROR SI CENTRAL PRODUCCIO ACTIVA.
--                 ERROR SI LA LINIA DE COMUNICACIO ACTIVA.
--
--

PROCEDURE PR_BAIXA_COMUNICACIOPRODUCCIO (inID     IN COMUNICACIOPRODUCCIO.ID%TYPE,
                                        RSP      OUT VARCHAR2);
	
--
-- NOM.........:   PR_MODIFICACIO_COMUNICACIOPRODUCCIO.
-- FINALITAT...:   CAMVIAR DE SITUACIO UNA LINIA DE PRODUCCIO, CENTRAL DISTRIBUCIO O LINIA COMUNICACIO O CENTRAL PRODUCCUIO. 
-- CONTROLS....:   INFORMAR CAMPS OBLIGATORIS
--                 QUE EL ID ESTIGUI EN LA BASE DE DADES
--                 QUE EL ID NO ESTIGUI JA ESBORRAT
--                 QUE LA NOVA LINIA NO ESTIGUI JA DONDA.

--              
-- PARAMETRES..:   
--         IN..:   id....: Codi de la Comunicacio produccio.
--        OUT..:   RSP
--
-- PRECONDICIO.:   QUE LA COMUNICACIO EXISTEIX.
--                 CENTRAL DISTRIBUCIO EN ALTA.
--                 CENTRAL PRODUCCIO EN ALTA.
--                 LINIA DE COMUNICACIO EN ALTA.
-- POSTCONDICIO:   ES DONA MODIFICA UNA LINIA DE COMUNICACIO.
-- RETURN .....:   OK SI S'HA EFECTUA LA MODIFICACIO.
--                 ERROR ALGUN CAMP DEL PARAMETRE NO TE VALOR.
--                 ERROR SI COMUNICACIO COMUNICACIO NO EXISTEIX.
--                 ERROR SI COMUNICACIO COMUNICACIO INACTIVA.
--                 ERROR SI LA NOVA LINIA JA EXISTEIX.
--
--
PROCEDURE PR_MODI_COMUNICACIOPRODUCCIO (inID              IN COMUNICACIOPRODUCCIO.ID%TYPE,
		  							   	inDistribucio     IN COMUNICACIOPRODUCCIO.IDDISTRIBUCIO%TYPE,
                                        inLinia           IN COMUNICACIOPRODUCCIO.IDLINIACOMUNICACIO%TYPE,
                                        inProduccio       IN COMUNICACIOPRODUCCIO.IDPRODUCCIO%TYPE,
										RSP               OUT VARCHAR2);										
END PKG_COMUNICACIOPRODUCCIO;

--
--
--
CREATE OR REPLACE PACKAGE BODY ELE.PKG_COMUNICACIOPRODUCCIO AS

PROCEDURE PR_ALTA_COMUNICACIOPRODUCCIO (inDistribucio     IN COMUNICACIOPRODUCCIO.IDDISTRIBUCIO%TYPE,
                                        inLinia           IN COMUNICACIOPRODUCCIO.IDLINIACOMUNICACIO%TYPE,
                                        inProduccio       IN COMUNICACIOPRODUCCIO.IDPRODUCCIO%TYPE,
                                        RSP               OUT VARCHAR2)
  IS

   V_CAMP_NUL                  EXCEPTION;
   V_DISTRIBUCIO_EXISTENT      EXCEPTION;
   V_DISTRIBUCIO_BAIXA         EXCEPTION;
   V_PRODUCCIO_EXISTENT        EXCEPTION;
   V_PRODUCCIO_BAIXA           EXCEPTION;
   V_PRODUCCIO_LINIA           EXCEPTION;
   V_LINIA_EXISTENT            EXCEPTION;
   V_LINIA_BAIXA               EXCEPTION;
   V_COMUNICACIO_EXISTENT      EXCEPTION;

   v_in          VARCHAR2(500);
   hihies        VARCHAR(1);
   v_actiu       NUMBER;
   v_id          NUMBER;
   err_num       NUMBER;
   err_msg       VARCHAR2(100);



   BEGIN
     v_in := 'Central distribucio:'||inDistribucio||', Linia produccio:'||inLinia||', Central produccio:'||inProduccio;
--
-- Primera comprovacio que tots els camps obligatoris tinguin valor
--
     IF (TRIM(inDistribucio)     IS NULL) THEN
         RAISE V_CAMP_NUL;
     END IF;
     IF (TRIM(inLinia)     IS NULL) THEN
         RAISE V_CAMP_NUL;
     END IF;
     IF (TRIM(inProduccio)     IS NULL) THEN
         RAISE V_CAMP_NUL;
     END IF;
--
-- QUE LA COMUNICACIO NO ESTIGUI DONADA D'ALTA
--
   BEGIN
      SELECT 'Y'
        INTO hihies
        FROM COMUNICACIOPRODUCCIO
       WHERE IDDISTRIBUCIO = inDistribucio
         AND IDLINIACOMUNICACIO = inLinia
	     AND IDPRODUCCIO = inProduccio
         AND ROWNUM = 1;
	   RAISE v_COMUNICACIO_EXISTENT;
   EXCEPTION
        WHEN NO_DATA_FOUND THEN NULL; 
   END;
--
-- Segona comprovacio que la central de distribucio alta
--
     BEGIN
       SELECT Actiu
         INTO v_ACTIU
         FROM DISTRIBUCIO
        WHERE IDDISTRIBUCIO  = inDistribucio
          AND ROWNUM  = 1;
    EXCEPTION
         WHEN NO_DATA_FOUND THEN RAISE V_DISTRIBUCIO_EXISTENT;
     END;
--
-- Tercera comprovacio que no estigui de baixa
--	 
     BEGIN
	    IF (V_ACTIU = 0) THEN
           RAISE V_DISTRIBUCIO_BAIXA;
        END IF;     
     END;
--
-- Quarta comprovacio que la linia de produccio alta
--	 
     BEGIN
       SELECT Actiu
         INTO V_ACTIU
         FROM LINIACOMUNICACIO
        WHERE IDLINIACOMUNICACIO  = inLinia
          AND ROWNUM  = 1;
    EXCEPTION
         WHEN NO_DATA_FOUND THEN RAISE V_LINIA_EXISTENT;
    END;
--
-- Cinquena comprovacio que no estigui de baixa
--	 
     BEGIN	 
     	IF (V_ACTIU = 0) THEN
           RAISE V_LINIA_BAIXA;
        END IF;     
     END;
--
-- Sisena comprovacio que la central de produccio alta
--
     BEGIN
       SELECT Actiu
         INTO V_ACTIU
         FROM PRODUCCIO
        WHERE IDPRODUCCIO  = inProduccio
          AND ROWNUM  = 1;
    EXCEPTION
         WHEN NO_DATA_FOUND THEN RAISE V_PRODUCCIO_EXISTENT;
    END;
--
-- Setena comprovacio que no estigui de baixa
--	 
     BEGIN	 
    	IF (V_ACTIU = 0) THEN
           RAISE V_PRODUCCIO_BAIXA;
        END IF;     
     END;
--
-- Vuitena que la central de produccio ja tingui un altra linia
--
     BEGIN
       SELECT IDPRODUCCIO
         INTO V_ID
         FROM COMUNICACIOPRODUCCIO
        WHERE IDLINIACOMUNICACIO = inLinia
          AND ROWNUM  = 1;		  
		IF (V_ID <> inProduccio) THEN
   		   RAISE V_PRODUCCIO_LINIA;
		END IF;  
     EXCEPTION 
         WHEN NO_DATA_FOUND THEN NULL;  
     END;
--
-- Tots els controls superats.
--
      INSERT INTO COMUNICACIOPRODUCCIO (IDDISTRIBUCIO, IDLINIACOMUNICACIO, IDPRODUCCIO, ACTIU, DATAALTA)
                  VALUES (inDistribucio, inLinia, InProduccio, 1, SYSDATE);
      RSP := 'OK';
      INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_COMUNICACIOPRODUCCUIO.PR_ALTA_COMUNICACIOPRODUCCIO', v_in, 'RSP: '||RSP);
      COMMIT;
  EXCEPTION
    WHEN V_DISTRIBUCIO_EXISTENT THEN
         RSP := 'ERROR: Central de distribucio no existent en la base de dades';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_COMUNICACIOPRODUCCUIO.PR_ALTA_COMUNICACIOPRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_DISTRIBUCIO_BAIXA THEN
         RSP := 'ERROR: Central de distribucio donada de baixa';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_COMUNICACIOPRODUCCUIO.PR_ALTA_COMUNICACIOPRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_PRODUCCIO_EXISTENT THEN
         RSP := 'ERROR: Central de produccio no existent en la base de dades';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_COMUNICACIOPRODUCCUIO.PR_ALTA_COMUNICACIOPRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_PRODUCCIO_BAIXA THEN
         RSP := 'ERROR: Central de produccio donada de baixa';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_COMUNICACIOPRODUCCUIO.PR_ALTA_COMUNICACIOPRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_PRODUCCIO_LINIA THEN
         RSP := 'ERROR: La linia de comunicacio ja te central de produccio';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_COMUNICACIOPRODUCCUIO.PR_ALTA_COMUNICACIOPRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_LINIA_EXISTENT THEN
         RSP := 'ERROR: Linia de produccio no existent en la base de dades';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_COMUNICACIOPRODUCCUIO.PR_ALTA_COMUNICACIOPRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_LINIA_BAIXA THEN
         RSP := 'ERROR: Linia de produccio donada de baixa';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_COMUNICACIOPRODUCCUIO.PR_ALTA_COMUNICACIOPRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_COMUNICACIO_EXISTENT THEN
         RSP := 'ERROR: Comunicacio dondad d`alta';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_COMUNICACIOPRODUCCUIO.PR_ALTA_COMUNICACIOPRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_CAMP_NUL THEN
         RSP := 'ERROR: No pot haver camps NULL';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_COMUNICACIOPRODUCCUIO.PR_ALTA_COMUNICACIOPRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN OTHERS THEN
         err_num := SQLCODE;
         err_msg := SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_COMUNICACIOPRODUCCUIO.PR_ALTA_COMUNICACIOPRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;
  END;

PROCEDURE PR_BAIXA_COMUNICACIOPRODUCCIO (inID     IN COMUNICACIOPRODUCCIO.ID%TYPE,
                                        RSP      OUT VARCHAR2)

  IS

   V_CAMP_NUL                  EXCEPTION;
   V_COMUNICACIO_EXISTENT      EXCEPTION;
   V_COMUNICACIO_BAIXA         EXCEPTION;
   V_PRODUCCIO_ACTIVA          EXCEPTION;
   V_DISTRIBUCIO_ACTIVA        EXCEPTION;
   V_LINIA_ACTIVA              EXCEPTION;
   V_PRODUCCIO_EXISTENT        EXCEPTION;
   V_DISTRIBUCIO_EXISTENT      EXCEPTION;
   V_LINIA_EXISTENT            EXCEPTION;

   v_in          VARCHAR2(500);
   v_actiu       NUMBER;
   v_distribucio NUMBER;
   v_linia       NUMBER;
   v_produccio   NUMBER;
   v_id          NUMBER;
   err_num       NUMBER;
   err_msg       VARCHAR2(100);

   BEGIN
     v_in := 'Comunicacio produccio:'||inId;
--
-- Primera comprovacio que tots els camps obligatoris tinguin valor
--
     IF (TRIM(inId)     IS NULL) THEN
         RAISE V_CAMP_NUL;
     END IF;
--
-- Segona comprovacio que el idComptador existeix
--
     BEGIN
       SELECT Actiu,IdDistribucio,IdLiniaComunicacio,IdProduccio
         INTO v_actiu,v_distribucio,v_linia,v_produccio
         FROM COMUNICACIOPRODUCCIO
        WHERE ID = inId
          AND ROWNUM  = 1;
     EXCEPTION
          WHEN NO_DATA_FOUND THEN RAISE V_COMUNICACIO_EXISTENT;
     END;	 
--
-- Tercera comprovacio que no estigui de baixa
--	 
     BEGIN 
        IF (V_ACTIU = 0) THEN
          RAISE V_COMUNICACIO_BAIXA;
        END IF;     
     END;
--
-- QUE LA CENTRAL DE PRODUCCIO ESTIGUI INIACTIVA.
--
   BEGIN
      SELECT Actiu
        INTO v_actiu
        FROM PRODUCCIO
       WHERE IDPRODUCCIO = v_produccio
         AND ROWNUM = 1;
   EXCEPTION
        WHEN NO_DATA_FOUND THEN RAISE V_PRODUCCIO_EXISTENT; 
   END;
   BEGIN 
      IF (V_ACTIU = 1) THEN
         RAISE V_PRODUCCIO_ACTIVA;
      END IF;     
   END;
--
-- QUE LA LINIA DE COMUNICACIO ESTIGUI INIACTIVA.
--
   BEGIN
      SELECT Actiu
        INTO v_actiu
        FROM LINIACOMUNICACIO
       WHERE IDLINIACOMUNICACIO = v_linia
         AND ROWNUM = 1;
   EXCEPTION
        WHEN NO_DATA_FOUND THEN RAISE V_LINIA_EXISTENT; 
   END;
   BEGIN 
      IF (V_ACTIU = 1) THEN
         RAISE V_LINIA_ACTIVA;
      END IF;     
   END;
--
-- QUE LA CENTRAL DE DISTRIBUCIO ESTIGUI INIACTIVA.
--
   BEGIN
      SELECT Actiu
        INTO v_actiu
        FROM DISTRIBUCIO
       WHERE IDDISTRIBUCIO = v_distribucio
         AND ROWNUM = 1;
   EXCEPTION
        WHEN NO_DATA_FOUND THEN RAISE V_DISTRIBUCIO_EXISTENT; 
   END;
   BEGIN 
      IF (V_ACTIU = 1) THEN
         RAISE V_DISTRIBUCIO_ACTIVA;
      END IF;     
   END;
--        
-- Tots els controls superats.
--
    UPDATE COMUNICACIOPRODUCCIO
       SET ACTIU = 0,
           DATABAIXA = SYSDATE
     WHERE ID = INID;

    RSP := 'OK';
    INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
         VALUES (SYSDATE,'PKG_COMUNICACIOPRODUCCIO.PR_BAIXA_COMUNICACIOPRODUCCIO', v_in, 'RSP: '||RSP);
    COMMIT;
 EXCEPTION
    WHEN V_COMUNICACIO_EXISTENT THEN
         RSP := 'ERROR: Linia de comunicacio no existent en la base de dades';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
            VALUES (SYSDATE,'PKG_COMUNICACIOPRODUCCIO.PR_BAIXA_COMUNICACIOPRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_COMUNICACIO_BAIXA THEN
         RSP := 'ERROR: Linia de comunicacio donada de baixa';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
            VALUES (SYSDATE,'PKG_COMUNICACIOPRODUCCIO.PR_BAIXA_COMUNICACIOPRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;		 
    WHEN V_PRODUCCIO_ACTIVA THEN
         RSP := 'ERROR: Central de produccio ACTIVA';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
            VALUES (SYSDATE,'PKG_COMUNICACIOPRODUCCIO.PR_BAIXA_COMUNICACIOPRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_PRODUCCIO_EXISTENT THEN
         RSP := 'ERROR: Central de produccio no donada d`alta';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
            VALUES (SYSDATE,'PKG_COMUNICACIOPRODUCCIO.PR_BAIXA_COMUNICACIOPRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_DISTRIBUCIO_ACTIVA THEN
         RSP := 'ERROR: Central de distribucio ACTIVA';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
            VALUES (SYSDATE,'PKG_COMUNICACIOPRODUCCIO.PR_BAIXA_COMUNICACIOPRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_DISTRIBUCIO_EXISTENT THEN
         RSP := 'ERROR: Central de distribucio no donada d`alta';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
            VALUES (SYSDATE,'PKG_COMUNICACIOPRODUCCIO.PR_BAIXA_COMUNICACIOPRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_LINIA_ACTIVA THEN
         RSP := 'ERROR: Linia de comunicacio ACTIVA';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
            VALUES (SYSDATE,'PKG_COMUNICACIOPRODUCCIO.PR_BAIXA_COMUNICACIOPRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_LINIA_EXISTENT THEN
         RSP := 'ERROR: Linia de comunicacio no donada d`alta';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
            VALUES (SYSDATE,'PKG_COMUNICACIOPRODUCCIO.PR_BAIXA_COMUNICACIOPRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_CAMP_NUL THEN
         RSP := 'ERROR: No pot haver camps NULL';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
            VALUES (SYSDATE,'PKG_COMUNICACIOPRODUCCIO.PR_BAIXA_COMUNICACIOPRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN OTHERS THEN
         err_num := SQLCODE;
         err_msg := SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
            VALUES (SYSDATE,'PKG_COMUNICACIOPRODUCCIO.PR_BAIXA_COMUNICACIOPRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;
  END;

PROCEDURE PR_MODI_COMUNICACIOPRODUCCIO (inID              IN COMUNICACIOPRODUCCIO.ID%TYPE,
		  							   	inDistribucio     IN COMUNICACIOPRODUCCIO.IDDISTRIBUCIO%TYPE,
                                        inLinia           IN COMUNICACIOPRODUCCIO.IDLINIACOMUNICACIO%TYPE,
                                        inProduccio       IN COMUNICACIOPRODUCCIO.IDPRODUCCIO%TYPE,
										RSP               OUT VARCHAR2)										

  IS

   V_CAMP_NUL                  EXCEPTION;
   V_COMUNICACIO_EXISTENT      EXCEPTION;
   V_COMUNICACIO_BAIXA         EXCEPTION;
   V_COMUNICACIO_REPE          EXCEPTION;
   V_PRODUCCIO_LINIA           EXCEPTION;

   v_in             VARCHAR2(500);
   hihies           VARCHAR2(1);
   v_actiu          NUMBER;
   v_distribucio    NUMBER;
   v_linia          NUMBER;
   v_produccio      NUMBER;
   v_consum_vell    NUMBER;
   v_consum_nou     NUMBER;
   v_comptador_vell NUMBER;
   v_comptador_nou  NUMBER;
   v_energia        NUMBER;
   OutId            NUMBER;
   v_id             NUMBER;
   resta            NUMBER;
   err_num          NUMBER;
   err_msg          VARCHAR2(100);

   BEGIN
     v_in := 'Comunicacio :'||inId||', Central distribucio:'||inDistribucio||', Linia comunicacio:'||inLinia||', Central produccio:'||inProduccio;
--
-- Primera comprovacio que tots els camps obligatoris tinguin valor
--
     IF (TRIM(inId)     IS NULL) THEN
         RAISE V_CAMP_NUL;
     END IF;
     IF (TRIM(inDistribucio)     IS NULL) THEN
         RAISE V_CAMP_NUL;
     END IF;
     IF (TRIM(inLinia)     IS NULL) THEN
         RAISE V_CAMP_NUL;
     END IF;
     IF (TRIM(inProduccio)     IS NULL) THEN
         RAISE V_CAMP_NUL;
     END IF;
--
-- Segona comprovacio que el idComptador existeix
--
     BEGIN
       SELECT Actiu,IdDistribucio,IdLiniaComunicacio,IdProduccio
         INTO v_actiu,v_distribucio,v_Linia,v_Produccio
         FROM COMUNICACIOPRODUCCIO
        WHERE ID = inId
          AND ROWNUM  = 1;
     EXCEPTION
          WHEN NO_DATA_FOUND THEN RAISE V_COMUNICACIO_EXISTENT;
     END;	 
--
-- Tercera comprovacio que no estigui de baixa
--	 
     BEGIN 
        IF (V_ACTIU = 0) THEN
          RAISE V_COMUNICACIO_BAIXA;
        END IF;     
     END;
--
-- Quarta que la nova linia no estigui ja donada.
--	 
    IF ((v_distribucio <> inDistribucio) or (v_Linia <> inLinia) or (v_Produccio <> inProduccio)) THEN
	   BEGIN
	      SELECT 'Y'
		    INTO hihies
            FROM COMUNICACIOPRODUCCIO
		   WHERE IdLiniaComunicacio = inLinia
		     AND IdDistribucio = inDistribucio
		     AND IdProduccio = inProduccio;
		   RAISE V_COMUNICACIO_REPE;
       EXCEPTION
	  	    WHEN NO_DATA_FOUND THEN NULL;
       END;
	END IF;
--
-- Vuitena que la central de produccio ja tingui un altra linia
--
     BEGIN
       SELECT IDPRODUCCIO
         INTO V_ID
         FROM COMUNICACIOPRODUCCIO
        WHERE IDLINIACOMUNICACIO = inLinia
          AND ROWNUM  = 1;		  
		IF (V_ID <> inProduccio) THEN
   		   RAISE V_PRODUCCIO_LINIA;
		END IF;  
     EXCEPTION 
         WHEN NO_DATA_FOUND THEN NULL;  
     END;
	
--        
-- Tots els controls superats.
--
--  ACTUALITZEM ESTADISTIQUES.
--
-- Primer recalcul de la situacio anterior.
-- Canvi per dalt.
   IF (v_distribucio <> inDistribucio) THEN 
      SELECT Sum(ConsumActual)
        INTO v_consum_vell
        FROM COMPTADORS
       WHERE IDCOMPTADOR  In (select idcomptador from comunicaciodistribucio where iddistribucio = v_distribucio)
         AND ROWNUM  = 1;
	  v_comptador_vell := 0;    
      IF (V_CONSUM_VELL > 0) THEN
         -- Resto el consum de la central de produccio.
         UPDATE PRODUCCIO
            SET CONSUMSCOMPTADOR = CONSUMSCOMPTADOR - V_CONSUM_VELL
          WHERE ACTIU = 1
            AND IDPRODUCCIO = v_produccio;
         -- Resto el consum de l'historic any.						 
         -- 
         -- Faig l'update TEMES ESTADISTICS LINIACOMUNICACIO
         UPDATE LINIACOMUNICACIO
            SET CONSUM = CONSUM - V_CONSUM_VELL
          WHERE ACTIU = 1
            AND IDLINIACOMUNICACIO = v_linia;
         -- Contem Comptadors meny la central de distribucio que trec.
	     SELECT COUNT(DISTINCT(IDCOMPTADOR))
		   INTO v_comptador_vell
           FROM COMPTADORS
          WHERE ACTIU = 1
            AND IDCOMPTADOR IN (SELECT IDCOMPTADOR
                                  FROM COMUNICACIODISTRIBUCIO
		                         WHERE ACTIU = 1
								   AND IDDISTRIBUCIO <> v_distribucio
					               AND IDDISTRIBUCIO IN (SELECT IDDISTRIBUCIO
						                                   FROM COMUNICACIOPRODUCCIO
								                          WHERE ACTIU = 1
													        AND IDLINIACOMUNICACIO = v_linia));  
   	     -- Actualitzo.
		 resta := v_comptador_vell;
   	     IF (resta = 0) THEN 
			resta := 1;
     	 END IF;
         UPDATE HISTORICANY
            SET CONSUM = CONSUM - V_CONSUM_VELL,
                CONSUMAVG = round((CONSUM-V_CONSUM_VELL) / resta,2)
          WHERE BD = 0
            AND IDANY = TO_CHAR(SysDate,'YYYY')
            AND IDCLAU = v_linia;
         RSP := 'OK UPDATE HISTORIC ANUAL '||TO_CHAR(SysDate,'YYYY'); 
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
              VALUES (SYSDATE,'PKG_COMUNICACIOPRODUCCIO.PR_MODIFICACIO_COMUNICACIOPRODUCCIO', v_in, 'RSP: '||RSP);
      END IF;   
      -- Segon recalcul de la situacio nova.
      SELECT Sum(ConsumActual)
        INTO v_consum_nou
        FROM COMPTADORS
       WHERE IDCOMPTADOR  In (select idcomptador from comunicaciodistribucio where iddistribucio = inDistribucio)
         AND ROWNUM  = 1;    
      IF (V_CONSUM_NOU > 0) THEN
         -- Sumo el consum de la central de produccio.
         UPDATE PRODUCCIO
            SET CONSUMSCOMPTADOR = CONSUMSCOMPTADOR + V_CONSUM_NOU
          WHERE ACTIU = 1
            AND IDPRODUCCIO = inProduccio; 
         -- Sumo el consum de l'historic any.						 
         -- 
         UPDATE LINIACOMUNICACIO
            SET CONSUM = CONSUM + V_CONSUM_NOU
          WHERE ACTIU = 1
            AND IDLINIACOMUNICACIO = inLinia;
         -- Contem els comptadors de la central de distribucio nova.
         SELECT COUNT(DISTINCT(IDCOMPTADOR))
	       INTO v_comptador_nou
           FROM COMPTADORS
          WHERE ACTIU = 1
            AND IDCOMPTADOR IN (SELECT IDCOMPTADOR
                                  FROM COMUNICACIODISTRIBUCIO
   		                         WHERE ACTIU = 1
					               AND IDDISTRIBUCIO = inDistribucio);
         -- Actualitzo.
		 v_comptador_nou := v_comptador_nou + v_comptador_vell; 
         UPDATE HISTORICANY
            SET CONSUM = CONSUM + V_CONSUM_NOU,
             CONSUMAVG = round((CONSUM-V_CONSUM_NOU) / v_comptador_nou,2)
         WHERE BD = 0
           AND IDANY = TO_CHAR(SysDate,'YYYY')
           AND IDCLAU = inLinia;
  	     RSP := 'OK UPDATE HISTORIC ANUAL '||TO_CHAR(SysDate,'YYYY'); 
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
             VALUES (SYSDATE,'PKG_COMUNICACIOPRODUCCIO.PR_MODIFICACIO_COMUNICACIOPRODUCCIO', v_in, 'RSP: '||RSP);
      END IF;
   END IF;
   -- Canvi per baix.
   IF ((v_produccio <> inProduccio) or (v_linia <> inLinia)) THEN
      -- Resto tota la produccio de la vella.
	  SELECT CONSUMACTUAL
	    INTO V_ENERGIA
	   FROM PRODUCCIO
	  WHERE IDPRODUCCIO = v_produccio;
	  IF (V_ENERGIA > 0) THEN
	     -- Resto cap al vell.
	     UPDATE LINIACOMUNICACIO
		    SET CAPACITATPRODUIDA = CAPACITATPRODUIDA - V_ENERGIA
		  WHERE IDLINIACOMUNICACIO = v_linia;
   	     -- Sumo cap al nou.
	     UPDATE LINIACOMUNICACIO
		    SET CAPACITATPRODUIDA = CAPACITATPRODUIDA + V_ENERGIA
		  WHERE IDLINIACOMUNICACIO = inlinia;
	  END IF;   
   END IF; 
--
-- Al fi canvio la lina de distribucio.
--  
    UPDATE COMUNICACIOPRODUCCIO
       SET IDLINIACOMUNICACIO = inLinia,
	       IDDISTRIBUCIO = inDistribucio,
		   IDPRODUCCIO = inProduccio
     WHERE ID = INID;
    RSP := 'OK';
    INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
         VALUES (SYSDATE,'PKG_COMUNICACIOPRODUCCIO.PR_MODIFICACIO_COMUNICACIOPRODUCCIO', v_in, 'RSP: '||RSP);
    COMMIT;
 EXCEPTION
    WHEN V_PRODUCCIO_LINIA THEN
         RSP := 'ERROR: La linia de comunicacio ja te central de produccio';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
             VALUES (SYSDATE,'PKG_COMUNICACIOPRODUCCIO.PR_MODIFICACIO_COMUNICACIOPRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_COMUNICACIO_REPE THEN
         RSP := 'ERROR: Linia de comunicacio ja donada d`alta';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
             VALUES (SYSDATE,'PKG_COMUNICACIOPRODUCCIO.PR_MODIFICACIO_COMUNICACIOPRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_COMUNICACIO_EXISTENT THEN
         RSP := 'ERROR: Linia de comunicacio no existent en la base de dades';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
             VALUES (SYSDATE,'PKG_COMUNICACIOPRODUCCIO.PR_MODIFICACIO_COMUNICACIOPRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_COMUNICACIO_BAIXA THEN
         RSP := 'ERROR: Linia de comunicacio donada de baixa';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
             VALUES (SYSDATE,'PKG_COMUNICACIOPRODUCCIO.PR_MODIFICACIO_COMUNICACIOPRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;		 
    WHEN V_CAMP_NUL THEN
         RSP := 'ERROR: No pot haver camps NULL';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
             VALUES (SYSDATE,'PKG_COMUNICACIOPRODUCCIO.PR_MODIFICACIO_COMUNICACIOPRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN OTHERS THEN
         err_num := SQLCODE;
         err_msg := SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
             VALUES (SYSDATE,'PKG_COMUNICACIOPRODUCCIO.PR_MODIFICACIO_COMUNICACIOPRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;
  END;												 												       
END PKG_COMUNICACIOPRODUCCIO; 
--
-- FI DEL PACKAGE
--
spool off;
