spool 13-package_ComunicacioDistribucio.log

CREATE OR REPLACE PACKAGE ELE.PKG_COMUNICACIODISTRIBUCIO AS

--
-- NOM.........:   PKG_COMUNICACIODISTRIBUCIO.
-- FINALITAT...:   GESTIO DE TOTA LA INFORMACIO REFERENTS AL SUMINISTRAMENT D'ENERIGIA CAP ALS CONTADORS.
--
--
--
--
-- NOM.........:   PR_ALTA_COMUNICACIODISTRIBUCIO.
-- FINALITAT...:   DONAR D'ALTA UN NOVA LINIA DE DISTRIBUCIO.
-- CONTROLS....:   INFORMAR CAMPS OBLIGATORIS.
--                 QUE LA CENTRAL DE DISTRIBUCIO ESTIGUI DONDA D'ALTA.
--                 QUE LA CENTRAL DE DISTRIBUCIO ESTIGUI ACTIVA.
--                 QUE LA CENTRAL DE DISTRIBUCIO TINGUI SUMINISTRAMENT ELETRIC ACTIU.
--                 QUE EL COMPTADOR ESTIGUI DONADA D'ALTA.
--                 QUE EL COMPTADOR ESTIGUI ACTIU.
--                 QUE LA COMUNICACIO NO ESTIGUI DONADA D'ALTA.

--              
-- PARAMETRES..:   
--         IN..:   idDistribucio: Codi de la central de distribucio (obligatori).
--	    	       idComptador..: codi del comptador (Obliagtoir).
--        OUT..:   RSP.
--
-- PRECONDICIO.:   CENTRAL DISTRIBUCIO EXISTEIX.
--                 COMPTADOR EXISTEIX.
-- POSTCONDICIO:   ES DONA D'ALTA UNA NOVA LINIA DE DISTRIBUCIO.
-- RETURN .....:   OK SI S'HA EFECTUAT L'ALTA.
--                 ERROR ALGUN CAMP DEL PARAMETRE NO TE VALOR.
--                 ERORR SI CENTRAL DISTRIBUCIO NO EXISTEIX.
--                 ERORR SI CENTRAL DISTRIBUCIO INACTIVA.
--                 ERROR SI CENTRAL DISTRIBUCIO NO TE CENTRAL DE PRODUCCIO ACTIVA.
--                 ERROR SI COMPTADOR NO EXISTEIX.
--                 ERROR SI COMPTADOR INACTIU.
--                 ERORR SI COMUNICACIO DE DISTRIBUCIO JA DONAT D'ALTA.
--
--

PROCEDURE PR_ALTA_COMUNICACIODISTRIBUCIO (inDistribucio     IN COMUNICACIODISTRIBUCIO.IDDISTRIBUCIO%TYPE,
                                          inComptador       IN COMUNICACIODISTRIBUCIO.IDCOMPTADOR%TYPE,
                                          RSP               OUT VARCHAR2);

--
-- NOM.........:   PR_BAIXA_COMUNICACIODISTRIBUCIO.
-- FINALITAT...:   DONAR DE BAIXA UNA COMUNICACIO DE DISTRIBUCIO.
-- CONTROLS....:   INFORMAR CAMPS OBLIGATORIS.
--                 QUE EL ID ESTIGUI EN LA BASE DE DADES.
--                 QUE EL ID NO ESTIGUI JA ESBORRAT.
--                 QUE LA CENTRAL DE DISTRIBUIO ESTIGUI INACTIVA.
--                 QUE EL COMPTADOR ESTIGUI INACTIU.

--              
-- PARAMETRES..:   
--         IN..:   id....: Codi de la Comunicacio distribucio.
--        OUT..:   RSP
--
-- PRECONDICIO.:   QUE LA COMUNICACIO DISTRIBUCIO EXISTEIX.
--                 CENTRAL DISTRIBUCIO EN BAIXA.
--                 COMPTADOR EN BAIXA.
-- POSTCONDICIO:   ES DONA DE BAIXA UNA COMUNICACIO PRODUCCIO.
-- RETURN .....:   OK SI S'HA EFECTUAT LA BAIXA.
--                 ERROR ALGUN CAMP DEL PARAMETRE NO TE VALOR.
--                 ERROR SI COMUNICACIO DISTRIBUCIO NO EXISTEIX.
--                 ERROR SI COMUNICACIO DISTRIBUCIO INACTIVA.
--                 ERORR SI CENTRAL DISTRIBUCIO ACTIVA.
--                 ERROR SI COMPTADORACTIU.
--
--
PROCEDURE PR_BAIX_COMUNICACIODISTRIBUCIO (inID     IN COMUNICACIODISTRIBUCIO.ID%TYPE,
                                          RSP      OUT VARCHAR2);
										  

--
-- NOM.........:   PR_MODIFICACIO_COMUNICACIODISTRIBUCIO.
-- FINALITAT...:   CAMVIAR DE SITUACIO UNA LINIA DE DISTRIBUCIO, COMPTADOR O CENTRAL.
-- CONTROLS....:   INFORMAR CAMPS OBLIGATORIS
--                 QUE EL ID ESTIGUI EN LA BASE DE DADES
--                 QUE EL ID NO ESTIGUI JA ESBORRAT
--                 QUE LA NOVA LINIA NO ESTIGUI JA DONDA.

--              
-- PARAMETRES..:   
--         IN..:   id....: Codi de la Comunicacio produccio.
--        OUT..:   RSP
--
-- PRECONDICIO.:   QUE LA COMUNICACIO DISTRIBUCIO EXISTEIX.
--                 CENTRAL DISTRIBUCIO EN ALTA.
--                 COMPTADOR EN ALTA.
-- POSTCONDICIO:   ES DONA MODIFICA UNA LINIA DE DISTRIBUCIO.
-- RETURN .....:   OK SI S'HA EFECTUA LA MODIFICACIO.
--                 ERROR ALGUN CAMP DEL PARAMETRE NO TE VALOR.
--                 ERROR SI COMUNICACIO DISTRIBUCIO NO EXISTEIX.
--                 ERROR SI COMUNICACIO DISTRIBUCIO INACTIVA.
--                 ERROR SI LA NOVA LINIA JA EXISTEIX.
--
--
PROCEDURE PR_MODI_COMUNICACIODISTRIBUCIO (inID              IN COMUNICACIODISTRIBUCIO.ID%TYPE,
                                          inDistribucio     IN COMUNICACIODISTRIBUCIO.IDDISTRIBUCIO%TYPE,
                                          inComptador       IN COMUNICACIODISTRIBUCIO.IDCOMPTADOR%TYPE,
                                          RSP               OUT VARCHAR2);
										   
END PKG_COMUNICACIODISTRIBUCIO;

--
--
--
CREATE OR REPLACE PACKAGE BODY ELE.PKG_COMUNICACIODISTRIBUCIO AS

PROCEDURE PR_ALTA_COMUNICACIODISTRIBUCIO (inDistribucio     IN COMUNICACIODISTRIBUCIO.IDDISTRIBUCIO%TYPE,
                                          inComptador       IN COMUNICACIODISTRIBUCIO.IDCOMPTADOR%TYPE,
                                          RSP               OUT VARCHAR2)
  IS

   V_CAMP_NUL                  EXCEPTION;
   V_DISTRIBUCIO_EXISTENT      EXCEPTION;
   V_DISTRIBUCIO_BAIXA         EXCEPTION;
   V_DISTRIBUCIO_PRODUCCIO     EXCEPTION;
   V_COMPTADOR_EXISTENT        EXCEPTION;
   V_COMPTADOR_BAIXA           EXCEPTION;
   V_COMUNICACIO_EXISTENT      EXCEPTION;

   v_in          VARCHAR2(500);
   hihies        VARCHAR(1);
   v_actiu       NUMBER;
   err_num       NUMBER;
   err_msg       VARCHAR2(100);

   BEGIN
     v_in := 'Central distribucio:'||inDistribucio||', Comptador:'||inComptador;
--
-- Primera comprovacio que tots els camps obligatoris tinguin valor
--
     IF (TRIM(inDistribucio)     IS NULL) THEN
         RAISE V_CAMP_NUL;
     END IF;
     IF (TRIM(inComptador)     IS NULL) THEN
         RAISE V_CAMP_NUL;
     END IF;
--
-- QUE LA COMUNICACIO NO ESTIGUI DONADA D'ALTA
--
   BEGIN
      SELECT 'Y'
        INTO hihies
       FROM COMUNICACIODISTRIBUCIO
     WHERE IDDISTRIBUCIO = inDistribucio
           AND IDCOMPTADOR = inComptador
       AND ROWNUM = 1;
	   RAISE V_COMUNICACIO_EXISTENT;
    EXCEPTION
         WHEN NO_DATA_FOUND THEN NULL; 
   END;
--
-- Segona comprovacio que la central de distribucio alta
--
     BEGIN
       SELECT Actiu
         INTO v_ACTIU
         FROM DISTRIBUCIO
        WHERE IDDISTRIBUCIO  = inDistribucio
          AND ROWNUM  = 1;
    EXCEPTION
         WHEN NO_DATA_FOUND THEN RAISE V_DISTRIBUCIO_EXISTENT;
     END;
--
-- Tercera comprovacio que no estigui de baixa
--	 
     BEGIN
	    IF (V_ACTIU = 0) THEN
           RAISE V_DISTRIBUCIO_BAIXA;
        END IF;     
     END;
--
-- Quarta comprovacio que el comptador alta
--	 
     BEGIN
       SELECT Actiu
         INTO V_ACTIU
         FROM COMPTADORS
        WHERE IDCOMPTADOR  = inComptador
          AND ROWNUM  = 1;
    EXCEPTION
         WHEN NO_DATA_FOUND THEN RAISE V_COMPTADOR_EXISTENT;
    END;
--
-- Cinquena comprovacio que no estigui de baixa
--	 
     BEGIN	 
     	IF (V_ACTIU = 0) THEN
           RAISE V_COMPTADOR_BAIXA;
        END IF;     
     END;
--
-- Sisena que la central de distribucio tingui central de produccio activa.
--
     BEGIN
       SELECT 'Y'
         INTO HIHIES
         FROM COMUNICACIOPRODUCCIO
        WHERE IDDISTRIBUCIO  = inDistribucio
          AND Actiu = 1
          AND ROWNUM  = 1;
    EXCEPTION
         WHEN NO_DATA_FOUND THEN RAISE V_DISTRIBUCIO_PRODUCCIO;
     END;
-- Tots els controls superats.
      INSERT INTO COMUNICACIODISTRIBUCIO (IDDISTRIBUCIO, IDCOMPTADOR, ACTIU, DATAALTA)
                  VALUES (inDistribucio, inComptador, 1, SYSDATE);
      RSP := 'OK';
      INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_COMUNICACIODISTRIBUCIO.PR_ALTA_COMUNICACIODISTRIBUCIO', v_in, 'RSP: '||RSP);
      COMMIT;
  EXCEPTION
    WHEN V_DISTRIBUCIO_EXISTENT THEN
         RSP := 'ERROR: Central de distribucio no existent en la base de dades';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_COMUNICACIODISTRIBUCIO.PR_ALTA_COMUNICACIODISTRIBUCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_DISTRIBUCIO_BAIXA THEN
         RSP := 'ERROR: Central de distribucio donada de baixa';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_COMUNICACIODISTRIBUCIO.PR_ALTA_COMUNICACIODISTRIBUCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_DISTRIBUCIO_PRODUCCIO THEN
         RSP := 'ERROR: Central de distribucio sense suministrament';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_COMUNICACIODISTRIBUCIO.PR_ALTA_COMUNICACIODISTRIBUCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_COMPTADOR_EXISTENT THEN
         RSP := 'ERROR: Comtpador no existent en la base de dades';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_COMUNICACIODISTRIBUCIO.PR_ALTA_COMUNICACIODISTRIBUCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_COMPTADOR_BAIXA THEN
         RSP := 'ERROR: Central de produccio donada de baixa';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_COMUNICACIODISTRIBUCIO.PR_ALTA_COMUNICACIODISTRIBUCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_COMUNICACIO_EXISTENT THEN
         RSP := 'ERROR: Comunicacio donda d`alta';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_COMUNICACIODISTRIBUCIO.PR_ALTA_COMUNICACIODISTRIBUCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_CAMP_NUL THEN
         RSP := 'ERROR: No pot haver camps NULL';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_COMUNICACIODISTRIBUCIO.PR_ALTA_COMUNICACIODISTRIBUCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN OTHERS THEN
         err_num := SQLCODE;
         err_msg := SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_COMUNICACIODISTRIBUCIO.PR_ALTA_COMUNICACIODISTRIBUCIO', v_in, 'RSP: '||RSP);
         COMMIT;
  END;
  
PROCEDURE PR_BAIX_COMUNICACIODISTRIBUCIO (inID     IN COMUNICACIODISTRIBUCIO.ID%TYPE,
                                           RSP      OUT VARCHAR2)
										   
  IS

   V_CAMP_NUL                  EXCEPTION;
   V_COMUNICACIO_EXISTENT      EXCEPTION;
   V_COMUNICACIO_BAIXA         EXCEPTION;
   V_DISTRIBUCIO_ACTIVA        EXCEPTION;
   V_COMPTADOR_ACTIVA          EXCEPTION;
   V_DISTRIBUCIO_EXISTENT      EXCEPTION;
   V_COMPTADOR_EXISTENT        EXCEPTION;

   v_in          VARCHAR2(500);
   v_actiu       NUMBER;
   v_distribucio NUMBER;
   v_comptador   NUMBER;
   v_id          NUMBER;
   err_num       NUMBER;
   err_msg       VARCHAR2(100);

   BEGIN
     v_in := 'Comunicacio distribucio:'||inId;
--
-- Primera comprovacio que tots els camps obligatoris tinguin valor
--
     IF (TRIM(inId)     IS NULL) THEN
         RAISE V_CAMP_NUL;
     END IF;
--
-- Segona comprovacio que el idComptador existeix
--
     BEGIN
       SELECT Actiu,IdDistribucio,IdComptador
         INTO v_actiu,v_distribucio,v_Comptador
         FROM COMUNICACIODISTRIBUCIO
        WHERE ID = inId
          AND ROWNUM  = 1;
     EXCEPTION
          WHEN NO_DATA_FOUND THEN RAISE V_COMUNICACIO_EXISTENT;
     END;	 
--
-- Tercera comprovacio que no estigui de baixa
--	 
     BEGIN 
        IF (V_ACTIU = 0) THEN
          RAISE V_COMUNICACIO_BAIXA;
        END IF;     
     END;
--
-- QUE LA CENTRAL DE DISTRIBUCIO ESTIGUI INIACTIVA.
--
   BEGIN
      SELECT Actiu
        INTO v_actiu
        FROM DISTRIBUCIO
       WHERE IDDISTRIBUCIO = v_distribucio
         AND ROWNUM = 1;
   EXCEPTION
        WHEN NO_DATA_FOUND THEN RAISE V_DISTRIBUCIO_EXISTENT; 
   END;
   BEGIN 
      IF (V_ACTIU = 1) THEN
         RAISE V_DISTRIBUCIO_ACTIVA;
      END IF;     
   END; 
--
-- QUE EL COMPTADOR ESTIGUI INIACTIU.
--
   BEGIN
      SELECT Actiu
        INTO v_actiu
        FROM COMPTADORS
       WHERE IDCOMPTADOR = v_comptador
         AND ROWNUM = 1;
   EXCEPTION
        WHEN NO_DATA_FOUND THEN RAISE V_COMPTADOR_EXISTENT; 
   END;
   BEGIN 
      IF (V_ACTIU = 1) THEN
         RAISE V_COMPTADOR_ACTIVA;
      END IF;     
   END;
--        
-- Tots els controls superats.
--
    UPDATE COMUNICACIODISTRIBUCIO
       SET ACTIU = 0,
           DATABAIXA = SYSDATE
     WHERE ID = INID;

    RSP := 'OK';
    INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
         VALUES (SYSDATE,'PKG_COMUNICACIODISTRIBUCIO.PR_BAIXA_COMUNICACIOPRODUCCIO', v_in, 'RSP: '||RSP);
    COMMIT;
 EXCEPTION
    WHEN V_COMUNICACIO_EXISTENT THEN
         RSP := 'ERROR: Linia de comunicacio no existent en la base de dades';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
            VALUES (SYSDATE,'PKG_COMUNICACIODISTRIBUCIO.PR_BAIXA_COMUNICACIOPRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_COMUNICACIO_BAIXA THEN
         RSP := 'ERROR: Linia de comunicacio donada de baixa';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
            VALUES (SYSDATE,'PKG_COMUNICACIODISTRIBUCIO.PR_BAIXA_COMUNICACIOPRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;		 
    WHEN V_DISTRIBUCIO_ACTIVA THEN
         RSP := 'ERROR: Central de distribucio ACTIVA';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
            VALUES (SYSDATE,'PKG_COMUNICACIODISTRIBUCIO.PR_BAIXA_COMUNICACIOPRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_DISTRIBUCIO_EXISTENT THEN
         RSP := 'ERROR: Central de distribucio no donada d`alta';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
            VALUES (SYSDATE,'PKG_COMUNICACIODISTRIBUCIO.PR_BAIXA_COMUNICACIOPRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_COMPTADOR_ACTIVA THEN
         RSP := 'ERROR: Comptador ACTIU';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
            VALUES (SYSDATE,'PKG_COMUNICACIODISTRIBUCIO.PR_BAIXA_COMUNICACIOPRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_COMPTADOR_EXISTENT THEN
         RSP := 'ERROR: Comptador no donat d`alta';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
            VALUES (SYSDATE,'PKG_COMUNICACIODISTRIBUCIO.PR_BAIXA_COMUNICACIOPRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_CAMP_NUL THEN
         RSP := 'ERROR: No pot haver camps NULL';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
            VALUES (SYSDATE,'PKG_COMUNICACIODISTRIBUCIO.PR_BAIXA_COMUNICACIOPRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN OTHERS THEN
         err_num := SQLCODE;
         err_msg := SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
            VALUES (SYSDATE,'PKG_COMUNICACIODISTRIBUCIO.PR_BAIXA_COMUNICACIOPRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;
  END;
  
PROCEDURE PR_MODI_COMUNICACIODISTRIBUCIO (inID              IN COMUNICACIODISTRIBUCIO.ID%TYPE,
                                          inDistribucio     IN COMUNICACIODISTRIBUCIO.IDDISTRIBUCIO%TYPE,
                                          inComptador       IN COMUNICACIODISTRIBUCIO.IDCOMPTADOR%TYPE,
                                          RSP               OUT VARCHAR2)
										   
  IS

   V_CAMP_NUL                  EXCEPTION;
   V_COMUNICACIO_EXISTENT      EXCEPTION;
   V_COMUNICACIO_BAIXA         EXCEPTION;
   V_COMUNICACIO_REPE          EXCEPTION;

   v_in             VARCHAR2(500);
   hihies           VARCHAR2(1);
   v_actiu          NUMBER;
   v_distribucio    NUMBER;
   v_comptador      NUMBER;
   v_consum_vell    NUMBER;
   v_consum_nou     NUMBER;
   v_comptador_vell NUMBER;
   v_comptador_nou  NUMBER;
   OutId            NUMBER;
   v_id             NUMBER;
   err_num          NUMBER;
   err_msg          VARCHAR2(100);

   -- Per saber les linies de comunicacio del comptador vell.
   cursor LiniasVell is (SELECT DISTINCT(IDLINIACOMUNICACIO) 
   	                       FROM COMUNICACIOPRODUCCIO
	        	          WHERE ACTIU = 1
					        AND IDDISTRIBUCIO IN (SELECT DISTINCT(IDDISTRIBUCIO) 
					                                FROM COMUNICACIODISTRIBUCIO
						                           WHERE IDCOMPTADOR = v_Comptador
        									         AND ACTIU = 1));	   
   -- Per saber les linies de comunicacio del comptador nou.
   cursor LiniasNou is (SELECT DISTINCT(IDLINIACOMUNICACIO) 
   	                      FROM COMUNICACIOPRODUCCIO
	        	         WHERE ACTIU = 1
					       AND IDDISTRIBUCIO = inDistribucio);   
   BEGIN
     v_in := 'Comunicacio distribucio:'||inId||', Comptador:'||inComptador||', Central distribucio:'||inDistribucio;
--
-- Primera comprovacio que tots els camps obligatoris tinguin valor
--
     IF (TRIM(inId)     IS NULL) THEN
         RAISE V_CAMP_NUL;
     END IF;
     IF (TRIM(inDistribucio)     IS NULL) THEN
         RAISE V_CAMP_NUL;
     END IF;
     IF (TRIM(inComptador)     IS NULL) THEN
         RAISE V_CAMP_NUL;
     END IF;
--
-- Segona comprovacio que el idComptador existeix
--
     BEGIN
       SELECT Actiu,IdDistribucio,IdComptador
         INTO v_actiu,v_distribucio,v_Comptador
         FROM COMUNICACIODISTRIBUCIO
        WHERE ID = inId
          AND ROWNUM  = 1;
     EXCEPTION
          WHEN NO_DATA_FOUND THEN RAISE V_COMUNICACIO_EXISTENT;
     END;	 
--
-- Tercera comprovacio que no estigui de baixa
--	 
     BEGIN 
        IF (V_ACTIU = 0) THEN
          RAISE V_COMUNICACIO_BAIXA;
        END IF;     
     END;
--
-- Quarta que la nova linia no estigui ja donada.
--	 
    IF ((v_distribucio <> inDistribucio) or (v_comptador <> inComptador)) THEN
	   BEGIN
	      SELECT 'Y'
		    INTO hihies
		   FROM COMUNICACIODISTRIBUCIO
		  WHERE IdComptador = inComptador
		    AND IdDistribucio = inDistribucio;
		  RAISE V_COMUNICACIO_REPE;
      EXCEPTION
		   WHEN NO_DATA_FOUND THEN NULL;
      END;
	END IF;
--        
-- Tots els controls superats.
--
--  ACTUALITZEM ESTADISTIQUES.
--
-- Primer recalcul de la situacio anterior.
     SELECT ConsumActual
       INTO v_consum_vell
       FROM COMPTADORS
      WHERE IDCOMPTADOR  = v_Comptador
        AND ROWNUM  = 1;    
   IF (V_CONSUM_VELL > 0) THEN
      -- Resto el consum de la central de produccio.
      UPDATE PRODUCCIO
         SET CONSUMSCOMPTADOR = CONSUMSCOMPTADOR - V_CONSUM_VELL
       WHERE ACTIU = 1
         AND IDPRODUCCIO IN (SELECT DISTINCT(IDPRODUCCIO) FROM COMUNICACIOPRODUCCIO WHERE ACTIU = 1 AND IDDISTRIBUCIO 
	                     IN (SELECT DISTINCT(IDDISTRIBUCIO) FROM COMUNICACIODISTRIBUCIO WHERE IDCOMPTADOR = v_Comptador AND ACTIU = 1));
      -- Resto el consum de l'historic any.						 
      -- 
      OPEN LiniasVell;
      FETCH LiniasVell INTO outId;
      WHILE LiniasVell%FOUND LOOP
            -- Faig l'update TEMES ESTADISTICS LINIACOMUNICACIO
            UPDATE LINIACOMUNICACIO
               SET CONSUM = CONSUM - V_CONSUM_VELL
             WHERE ACTIU = 1
               AND IDLINIACOMUNICACIO = outId;
            -- Contem Comptadors
	     	SELECT COUNT(DISTINCT(IDCOMPTADOR))
		      INTO v_comptador_vell
              FROM COMPTADORS
             WHERE ACTIU = 1
               AND IDCOMPTADOR IN (SELECT IDCOMPTADOR
                                     FROM COMUNICACIODISTRIBUCIO
				    	            WHERE ACTIU = 1
					                  AND IDDISTRIBUCIO IN (SELECT IDDISTRIBUCIO
						                                      FROM COMUNICACIOPRODUCCIO
								        	         	     WHERE ACTIU = 1
													           AND IDLINIACOMUNICACIO = outId));  
   		    -- Actualitzo 
    		v_comptador_vell := v_comptador_vell - 1;
	    	IF (V_COMPTADOR_vell = 0) THEN 
			   v_comptador_vell := 1;
			END IF;
            UPDATE HISTORICANY
               SET CONSUM = CONSUM - V_CONSUM_VELL,
                   CONSUMAVG = round((CONSUM-V_CONSUM_VELL) / v_comptador_vell,2)
             WHERE BD = 0
               AND IDANY = TO_CHAR(SysDate,'YYYY')
               AND IDCLAU = outId;
      	   RSP := 'OK UPDATE HISTORIC ANUAL '||TO_CHAR(SysDate,'YYYY'); 
           INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
             VALUES (SYSDATE,'PKG_COMUNICACIODISTRIBUCIO.PR_MODIFICACIO_COMUNICACIOPRODUCCIO', v_in, 'RSP: '||RSP);
           FETCH LiniasVell INTO outId;	  
     END LOOP;
     CLOSE LiniasVell;
   END IF;
-- Segon recalcul de la situacio nova.
     SELECT ConsumActual
       INTO v_consum_nou
       FROM COMPTADORS
      WHERE IDCOMPTADOR  = inComptador
        AND ROWNUM  = 1;    
   IF (V_CONSUM_NOU > 0) THEN
      -- Sumo el consum de la central de produccio.
      UPDATE PRODUCCIO
         SET CONSUMSCOMPTADOR = CONSUMSCOMPTADOR + V_CONSUM_NOU
       WHERE ACTIU = 1
         AND IDPRODUCCIO IN (SELECT DISTINCT(IDPRODUCCIO) FROM COMUNICACIOPRODUCCIO WHERE ACTIU = 1 AND IDDISTRIBUCIO = inDistribucio); 
      -- Sumo el consum de l'historic any.						 
      -- 
      OPEN LiniasNou;
      FETCH LiniasNou INTO outId;
      WHILE LiniasNou%FOUND LOOP
            -- Faig l'update TEMES ESTADISTICS LINIACOMUNICACIO
            UPDATE LINIACOMUNICACIO
               SET CONSUM = CONSUM + V_CONSUM_NOU
             WHERE ACTIU = 1
               AND IDLINIACOMUNICACIO = outId;
            -- Contem Comptadors
	     	SELECT COUNT(DISTINCT(IDCOMPTADOR))
		      INTO v_comptador_nou
              FROM COMPTADORS
             WHERE ACTIU = 1
               AND IDCOMPTADOR IN (SELECT IDCOMPTADOR
                                     FROM COMUNICACIODISTRIBUCIO
				    	            WHERE ACTIU = 1
					                  AND IDDISTRIBUCIO IN (SELECT IDDISTRIBUCIO
						                                      FROM COMUNICACIOPRODUCCIO
								        	         	     WHERE ACTIU = 1
													           AND IDLINIACOMUNICACIO = outId));  
   		    -- Actualitzo el nou comptador 
    		v_comptador_nou := v_comptador_nou + 1;
            UPDATE HISTORICANY
               SET CONSUM = CONSUM + V_CONSUM_NOU,
                   CONSUMAVG = round((CONSUM-V_CONSUM_NOU) / v_comptador_nou,2)
             WHERE BD = 0
               AND IDANY = TO_CHAR(SysDate,'YYYY')
               AND IDCLAU = outId;
      	   RSP := 'OK UPDATE HISTORIC ANUAL '||TO_CHAR(SysDate,'YYYY'); 
           INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
             VALUES (SYSDATE,'PKG_COMUNICACIODISTRIBUCIO.PR_MODIFICACIO_COMUNICACIOPRODUCCIO', v_in, 'RSP: '||RSP);
           FETCH LiniasNou INTO outId;	  
     END LOOP;
     CLOSE LiniasNou;
   END IF;
--
-- Al fi canvio la lina de distribucio.
--  
    UPDATE COMUNICACIODISTRIBUCIO
       SET IDCOMPTADOR = inComptador,
	       IDDISTRIBUCIO = inDistribucio
     WHERE ID = INID;
    RSP := 'OK';
    INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
         VALUES (SYSDATE,'PKG_COMUNICACIODISTRIBUCIO.PR_MODIFICACIO_COMUNICACIODISTRIBUCIO', v_in, 'RSP: '||RSP);
    COMMIT;
 EXCEPTION
    WHEN V_COMUNICACIO_REPE THEN
         RSP := 'ERROR: Linia de comunicacio ja donada d`alta';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
             VALUES (SYSDATE,'PKG_COMUNICACIODISTRIBUCIO.PR_MODIFICACIO_COMUNICACIODISTRIBUCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_COMUNICACIO_EXISTENT THEN
         RSP := 'ERROR: Linia de comunicacio no existent en la base de dades';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
             VALUES (SYSDATE,'PKG_COMUNICACIODISTRIBUCIO.PR_MODIFICACIO_COMUNICACIODISTRIBUCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_COMUNICACIO_BAIXA THEN
         RSP := 'ERROR: Linia de comunicacio donada de baixa';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
             VALUES (SYSDATE,'PKG_COMUNICACIODISTRIBUCIO.PR_MODIFICACIO_COMUNICACIODISTRIBUCIO', v_in, 'RSP: '||RSP);
         COMMIT;		 
    WHEN V_CAMP_NUL THEN
         RSP := 'ERROR: No pot haver camps NULL';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
             VALUES (SYSDATE,'PKG_COMUNICACIODISTRIBUCIO.PR_MODIFICACIO_COMUNICACIODISTRIBUCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN OTHERS THEN
         err_num := SQLCODE;
         err_msg := SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
             VALUES (SYSDATE,'PKG_COMUNICACIODISTRIBUCIO.PR_MODIFICACIO_COMUNICACIODISTRIBUCIO', v_in, 'RSP: '||RSP);
         COMMIT;
  END;												 												     
END PKG_COMUNICACIODISTRIBUCIO; 
--
-- FI DEL PACKAGE
--
spool off;
