spool 14-package_historiclectures.log

CREATE OR REPLACE PACKAGE ELE.PKG_HISTORICLECTURES AS

--
-- NOM.........:   PKG_HISTORICOLECTURES
-- FINALITAT...:   GESTIO DE TOTA LA INFORMACIO REFERENTS AL CONSUM DE COMPTADORS
--
--
--
--
-- NOM.........:   PR_ALTA_HISTORICOLECTURES
-- FINALITAT...:   DONAR D'ALTA UN NOU CONSUM
-- CONTROLS....:   INFORMAR CAMPS OBLIGATORIS
--                 COMPTADOR EXISTENT
--                 DATA+COMPTADOR NO ESTA EN LA BASE DE DADES.
--                 DATA MAJOR QUE L'ANTERIOR LECTURA
--                 CONSUM MAJOR QUE L'ANTERIOR LECTURA
--              
-- PARAMETRES..:   
--         IN..:   idComptador: Codi del comptador a llegir (obligatori)
--	    	   datalectura: Data de la lectura efectuada (Obliagtoir)
--                 Consum ....: Consum efectuat (Obligatori)
--                 Tipus......: Tipus de lectura 0-> Telematica 1-> Presencial
--        OUT..:   RSP
--
-- PRECONDICIO.:   COMPTADOR EXISTEIX
-- POSTCONDICIO:   ES DONA D'ALTA UNA NOVA LECTURA DEL COMPTADOR
-- RETURN .....:   OK SI S'HA EFECTUAT L'ALTA
--                 ERROR ALGUN CAMP DEL PARAMETRE NO TE VALOR
--                 ERROR SI LA DATA ES MENOR
--                 ERROR SI LA LECTURA ES MENOR
--
--

PROCEDURE PR_ALTA_HISTORICLECTURES (inComptador     IN HISTORICLECTURES.IDCOMPTADOR%TYPE,
                                    inData          IN HISTORICLECTURES.DATALECTURA%TYPE,
                                    inConsum        IN HISTORICLECTURES.CONSUMACTUAL%TYPE,
                                    inTipus         IN HISTORICLECTURES.TIPUSLECTURA%TYPE,
                                    RSP             OUT VARCHAR2);

END PKG_HISTORICLECTURES;

--
--
--
CREATE OR REPLACE PACKAGE BODY ELE.PKG_HISTORICLECTURES AS

PROCEDURE PR_ALTA_HISTORICLECTURES (inComptador     IN HISTORICLECTURES.IDCOMPTADOR%TYPE,
                                    inData          IN HISTORICLECTURES.DATALECTURA%TYPE,
                                    inConsum        IN HISTORICLECTURES.CONSUMACTUAL%TYPE,
                                    inTipus         IN HISTORICLECTURES.TIPUSLECTURA%TYPE,
                                    RSP             OUT VARCHAR2)
  IS

   V_CAMP_NUL                  EXCEPTION;
   V_COMPTADOR_EXISTENT        EXCEPTION;
   V_COMPTADOR_BAIXA           EXCEPTION;
   V_COMPTADOR_LINIA           EXCEPTION;
   V_LECTURA_EXISTENT          EXCEPTION;
   V_PRODUCCIO_LINIA           EXCEPTION;
   V_TIPUS_ERROR               EXCEPTION;
   V_DATA_ERROR                EXCEPTION;
   V_CONSUM_ERROR              EXCEPTION;
   V_PRODUCCIO_CENTRAL         EXCEPTION;   

   v_in          VARCHAR2(500);
   v_actiu       NUMBER;
   v_consum      NUMBER;
   v_total       NUMBER;
   v_comptadors  NUMBER;
   v_mitja       NUMBER;
   v_data        DATE;
   hihies        VARCHAR(1);
   outId         NUMBER;

   err_num       NUMBER;
   err_msg       VARCHAR2(100);
   
   cursor LiniasComptador is (SELECT DISTINCT(IDLINIACOMUNICACIO) 
   	                            FROM COMUNICACIOPRODUCCIO
	        	               WHERE ACTIU = 1
					             AND IDDISTRIBUCIO IN (SELECT DISTINCT(IDDISTRIBUCIO) 
					                                     FROM COMUNICACIODISTRIBUCIO
											            WHERE IDCOMPTADOR = inComptador
        										          AND ACTIU = 1));			  
												   
   BEGIN

     v_in := 'Comptador:'||inComptador||', Data Lectura:'||inData||', Consum Lectura:'||inConsum||', Tipus lectura:'||inTipus;

--
-- Primera comprovacio que tots els camps obligatoris tinguin valor
--
     IF (TRIM(inComptador)     IS NULL) THEN
         RAISE V_CAMP_NUL;
     END IF;
     IF (TRIM(inData)     IS NULL) THEN
         RAISE V_CAMP_NUL;
     END IF;
     IF (TRIM(inConsum)     IS NULL) THEN
         RAISE V_CAMP_NUL;
     END IF;
     IF (TRIM(inTipus)     IS NULL) THEN
         RAISE V_CAMP_NUL;
     END IF;
--
-- QUE LA LECTURA NO ESTIGUI DONADA D'ALTA
--
   BEGIN

      SELECT 'Y'
        INTO hihies
       FROM HISTORICLECTURES
     WHERE DATALECTURA = inData
	   AND IDCOMPTADOR = inComptador
       AND ROWNUM = 1;

	   RAISE V_LECTURA_EXISTENT;
    EXCEPTION
         WHEN NO_DATA_FOUND THEN NULL; 
   END;

	 
--
-- Segona comprovacio que el comptador es trovi en alta. Carrega valors anteriors
--
     BEGIN

       SELECT Actiu,ConsumActual,DataLectura
         INTO v_ACTIU,V_CONSUM,V_DATA
         FROM COMPTADORS
        WHERE IDCOMPTADOR  = inComptador
          AND ROWNUM  = 1;

     EXCEPTION
         WHEN NO_DATA_FOUND THEN RAISE V_COMPTADOR_EXISTENT;

     END;
--
-- Tercera comprovacio que no estigui de baixa
--	 
     BEGIN
	 
	 	IF (V_ACTIU = 0) THEN
           RAISE V_COMPTADOR_BAIXA;
        END IF;     
     END;
--
-- Quarta comprovacio que els valors de la lectura es corresponguin
--	 
     BEGIN
       
        IF ((InTipus < 0) OR (InTipus > 1)) THEN 
           RAISE V_TIPUS_ERROR;
        END IF;
     END;
--
-- Comprovacio de les dades anteriors
--
   BEGIN
       
      IF (V_DATA > INDATA) THEN 
         RAISE V_DATA_ERROR;
      END IF;
      IF (V_CONSUM > INCONSUM) THEN 
         RAISE V_DATA_ERROR;
      END IF;
   END;
--
-- QUE EL COMPTADOR TINGUI LINIA DE SUMINISTRAMENT.
--
   BEGIN

      SELECT 'Y'
        INTO hihies
        FROM COMUNICACIODISTRIBUCIO
       WHERE IDCOMPTADOR = inComptador
	     AND ACTIU = 1
         AND ROWNUM = 1;

    EXCEPTION
         WHEN NO_DATA_FOUND THEN RAISE V_COMPTADOR_LINIA;
   END;
--
-- 	QUE AL COMPTADOR LI ARRIBI ENERGIA.
--
   BEGIN
      
	  SELECT 'Y'
	    INTO hihies
   	    FROM COMUNICACIOPRODUCCIO
  	   WHERE ACTIU = 1
	     AND IDDISTRIBUCIO IN (SELECT IDDISTRIBUCIO FROM COMUNICACIODISTRIBUCIO WHERE IDCOMPTADOR = inComptador AND ACTIU = 1)
		 AND ROWNUM = 1;
		 
	  EXCEPTION
	      WHEN NO_DATA_FOUND THEN RAISE V_PRODUCCIO_LINIA;  
   END;
--
-- FINALMENT QUE LA CENTRAL FUNCIONI.
--
  BEGIN
  
      SELECT 'Y'
	    INTO hihies
        FROM PRODUCCIO
       WHERE ACTIU = 1
         AND IDPRODUCCIO IN (SELECT IDPRODUCCIO 
       	                       FROM COMUNICACIOPRODUCCIO
 		                      WHERE ACTIU = 1
		                         AND IDDISTRIBUCIO IN (SELECT IDDISTRIBUCIO
		                                                 FROM COMUNICACIODISTRIBUCIO
                                                        WHERE ACTIU = 1
                                                          AND IDCOMPTADOR = INCOMPTADOR)) 
		 AND ROWNUM = 1;
  EXCEPTION
     WHEN NO_DATA_FOUND THEN RAISE V_PRODUCCIO_CENTRAL;	
  END;  
--   
-- Tots els controls superats.
      v_Consum := INCONSUM - V_CONSUM;
	  
-- Actulitzo taula historic any TEMES ESTADISTICS.
-- Primer miro si ha canviat l'any crear historic de l'any
  BEGIN
     SELECT 'Y'
       INTO hihies
       FROM HISTORICANY
      WHERE BD = 0
	    AND IDANY = TO_CHAR(inData,'YYYY')
	    AND ROWNUM = 1;
	  
  EXCEPTION
     -- ALTA.
     WHEN NO_DATA_FOUND THEN 
	 
	 INSERT INTO HISTORICANY (SELECT 0, IDLINIACOMUNICACIO,TO_CHAR(SYSDATE,'YYYY'), 0, 0, 0, 0, 0,0  FROM LINIACOMUNICACIO);
	 
	 RSP := 'OK INSERT HISTORIC TOTAL '||TO_CHAR(inData,'YYYY'); 
     INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_HISTORICLECTURES.PR_ALTA_HISTORICLECTURES', v_in, 'RSP: '||RSP);
		   
  END;
  
																	    
-- Actualitzo taula historic total consum TEMES ESTADISTICS.

  BEGIN
     SELECT Consum
        INTO v_total
       FROM HISTORICTOTAL
      WHERE ROWNUM = 1;
	  
	 -- HIHES L'ANY UPDATE.
	 UPDATE HISTORICTOTAL 
	    SET CONSUM = CONSUM + v_Consum;
		
	 v_total := v_total + v_Consum;

	 RSP := 'OK UPDATE HISTORIC TOTAL';
     INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_HISTORICLECTURES.PR_ALTA_HISTORICLECTURES', v_in, 'RSP: '||RSP);
	  
  EXCEPTION
     -- ALTA.
     WHEN NO_DATA_FOUND THEN 
	 
	 INSERT INTO HISTORICTOTAL (CONSUM, PRODUCCIO) VALUES (v_Consum, 0);
	 
	 v_total := v_consum;
	 
	 RSP := 'OK INSERT HISTORIC TOTAL'; 
     INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_HISTORICLECTURES.PR_ALTA_HISTORICLECTURES', v_in, 'RSP: '||RSP);
		   
  END;

-- CONTO ELS COMPTADORS DE CADA LINIA I ACTULITZO MITJA I CONSUM
  
  OPEN LiniasComptador;
  FETCH LiniasComptador INTO outId;

  WHILE LiniasComptador%FOUND LOOP

        -- Faig l'update TEMES ESTADISTICS LINIACOMUNICACIO
        UPDATE LINIACOMUNICACIO
           SET CONSUM = CONSUM + V_CONSUM
         WHERE ACTIU = 1
           AND IDLINIACOMUNICACIO = outId;

        -- Contem Comptadors
		SELECT COUNT(DISTINCT(IDCOMPTADOR))
		  INTO v_comptadors
          FROM COMPTADORS
         WHERE ACTIU = 1
           AND IDCOMPTADOR IN (SELECT IDCOMPTADOR
                                 FROM COMUNICACIODISTRIBUCIO
					            WHERE ACTIU = 1
						           AND IDDISTRIBUCIO IN (SELECT IDDISTRIBUCIO
						                                   FROM COMUNICACIOPRODUCCIO
										         	      WHERE ACTIU = 1
													        AND IDLINIACOMUNICACIO = outId));
  
		-- Actualitzo 
        UPDATE HISTORICANY
           SET CONSUM = CONSUM + V_CONSUM,
	           CONSUMAVG = round((CONSUM+V_CONSUM) / v_comptadors,2)
         WHERE BD = 0
           AND IDANY = TO_CHAR(inData,'YYYY')
           AND IDCLAU = outId;

   	    RSP := 'OK UPDATE HISTORIC ANUAL '||TO_CHAR(inData,'YYYY'); 
        INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
             VALUES (SYSDATE,'PKG_HISTORICLECTURES.PR_ALTA_HISTORICLECTURES', v_in, 'RSP: '||RSP);
		
        FETCH LiniasComptador INTO outId;
		  
  END LOOP;
  CLOSE LiniasComptador;
  
-- Faig l'update TEMES ESTADISTICS DEL CONSUM TOTAL
  v_total := v_total * 0.50;

  UPDATE HISTORICANY
     SET CONSUMTOTAL = v_total
   WHERE BD = 0;		  

	RSP := 'OK UPDATE HISTORIC ANUAL '||TO_CHAR(inData,'YYYY'); 
    INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_HISTORICLECTURES.PR_ALTA_HISTORICLECTURES', v_in, 'RSP: '||RSP);

-- Al fi insterto la laectura.  	  

      INSERT INTO HISTORICLECTURES (IDCOMPTADOR, DATALECTURA, CONSUMACTUAL, CONSUM, TIPUSLECTURA)
                  VALUES (inComptador, inData, inConsum, v_Consum, inTipus);

      RSP := 'OK';

      INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_HISTORICLECTURES.PR_ALTA_HISTORICLECTURES', v_in, 'RSP: '||RSP);

      COMMIT;

  EXCEPTION
    WHEN V_PRODUCCIO_CENTRAL THEN
         RSP := 'ERROR: Comptador sense central de producci';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_HISTORICLECTURES.PR_ALTA_HISTORICLECTURES', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_COMPTADOR_EXISTENT THEN
         RSP := 'ERROR: Comptador no existent en la base de dades';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_HISTORICLECTURES.PR_ALTA_HISTORICLECTURES', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_COMPTADOR_BAIXA THEN
         RSP := 'ERROR: Comptador donat de baixa';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_HISTORICLECTURES.PR_ALTA_HISTORICLECTURES', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_COMPTADOR_LINIA THEN
         RSP := 'ERROR: Comptador Sense linia suministrament';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_HISTORICLECTURES.PR_ALTA_HISTORICLECTURES', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_PRODUCCIO_LINIA THEN
         RSP := 'ERROR: Linea de produccio no actives';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_HISTORICLECTURES.PR_ALTA_HISTORICLECTURES', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_LECTURA_EXISTENT THEN
         RSP := 'ERROR: Lectura ja efectuada';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_HISTORICLECTURES.PR_ALTA_HISTORICLECTURES', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_DATA_ERROR THEN
         RSP := 'ERROR: Data lectura inferior a l`anterior';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_HISTORICLECTURES.PR_ALTA_HISTORICLECTURES', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_CONSUM_ERROR THEN
         RSP := 'ERROR: Consum lectura inferior a l`anterior';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_HISTORICLECTURES.PR_ALTA_HISTORICLECTURES', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_TIPUS_ERROR THEN
         RSP := 'ERROR: Tipus de lectura erroni';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_HISTORICLECTURES.PR_ALTA_HISTORICLECTURES', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_CAMP_NUL THEN
         RSP := 'ERROR: No pot haver camps NULL';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_HISTORICLECTURES.PR_ALTA_HISTORICLECTURES', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN OTHERS THEN
         err_num := SQLCODE;
         err_msg := SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_HISTORICLECTURES.PR_ALTA_HISTORICLECTURES', v_in, 'RSP: '||RSP);
         COMMIT;
  END;
 

END PKG_HISTORICLECTURES;

--
-- FI DEL PACKAGE
--
spool off;
