spool 14-package_historicproduccio.log

CREATE OR REPLACE PACKAGE ELE.PKG_HISTORICPRODUCCIO AS

--
-- NOM.........:   PKG_HISTORICOPRODUCCIO
-- FINALITAT...:   GESTIO DE TOTA LA INFORMACIO REFERENTS A LA PRODUCCIO D'UNA CENTRAL.
--
--
--
--
-- NOM.........:   PR_ALTA_HISTORICOPRODUCCIO.
-- FINALITAT...:   DONAR D'ALTA UN NOVA PRODUCCIO
-- CONTROLS....:   INFORMAR CAMPS OBLIGATORIS
--                 CENTRAL EXISTEIX
--                 CENTARL NO ESTA DE BAIXA
--                 PRODUCCIO POT CIRCULAR PER LA LINIA
--                 CENTRAL TE LINIA DE COMUNICACIO
--              
-- PARAMETRES..:   
--         IN..:   idProduccio: Codi de la central de produccio (Obligatori),
--        	     dataProduccio: Data de la produccio efectuada (Obliagtoir)
--                    Kilovats: Produccio efectuat (Obligatori)
--        OUT..:   RSP
--
-- PRECONDICIO.:   CENTRAL EXISTEIX
-- POSTCONDICIO:   ES DONA D'ALTA UNA NOVA PRODUCCIO DE LA CENTRAL
-- RETURN .....:   OK SI S'HA EFECTUAT L'ALTA
--                 ERROR ALGUN CAMP DEL PARAMETRE NO TE VALOR
--                 ERROR SI LA CENTRAL NO EXISTEIEX.
--                 ERROR SI LA PRODUCCIO ES MAJOR A LA PERMESA
--                 ERROR SI LA PRODUCCIO NO POT CIRCULAR.
--
--

PROCEDURE PR_ALTA_HISTORICPRODUCCIO (inProduccio    IN HISTORICPRODUCCIO.IDPRODUCCIO%TYPE,
                                    inData          IN HISTORICPRODUCCIO.DATAPRODUCCIO%TYPE,
                                    inKilo          IN HISTORICPRODUCCIO.KILOWATS%TYPE,
                                    RSP             OUT VARCHAR2);

END PKG_HISTORICPRODUCCIO;

--
--
--
CREATE OR REPLACE PACKAGE BODY ELE.PKG_HISTORICPRODUCCIO AS

PROCEDURE PR_ALTA_HISTORICPRODUCCIO (inProduccio    IN HISTORICPRODUCCIO.IDPRODUCCIO%TYPE,
                                    inData          IN HISTORICPRODUCCIO.DATAPRODUCCIO%TYPE,
                                    inKilo          IN HISTORICPRODUCCIO.KILOWATS%TYPE,
                                    RSP             OUT VARCHAR2)
  IS

   V_CAMP_NUL                  EXCEPTION;
   V_PRODUCCIO_EXISTENT        EXCEPTION;
   V_PRODUCCIO_BAIXA           EXCEPTION;
   V_LINIA_EXISTENT            EXCEPTION;
   V_LINIA_BAIXA               EXCEPTION;
   V_LINIA_SUPERIOR            EXCEPTION;
   V_LECTURA_EXISTENT          EXCEPTION;

   v_in          VARCHAR2(500);
   v_actiu       NUMBER;
   v_data        DATE;
   v_maxima      NUMBER;
   v_total       NUMBER;
   v_linia       NUMBER;
   hihies        VARCHAR(1);
   err_num       NUMBER;
   err_msg       VARCHAR2(100);
   
   BEGIN
     v_in := 'Produccio:'||inProduccio||', Data Produccio:'||inData||', Produccio efectuada:'||inKilo;
--
-- Primera comprovacio que tots els camps obligatoris tinguin valor
--
     IF (TRIM(inProduccio)     IS NULL) THEN
         RAISE V_CAMP_NUL;
     END IF;
     IF (TRIM(inData)     IS NULL) THEN
         RAISE V_CAMP_NUL;
     END IF;
     IF (TRIM(inKilo)     IS NULL) THEN
         RAISE V_CAMP_NUL;
     END IF;
--
-- QUE LA LECTURA NO ESTIGUI DONADA D'ALTA
--
   BEGIN
      SELECT 'Y'
        INTO hihies
       FROM HISTORICPRODUCCIO
     WHERE DATAPRODUCCIO = inData
	   AND IDPRODUCCIO = inProduccio
       AND ROWNUM = 1;
	   RAISE V_LECTURA_EXISTENT;
    EXCEPTION
         WHEN NO_DATA_FOUND THEN NULL; 
   END; 
--
-- Segona comprovacio que la central es trovi en alta.
--
     BEGIN
       SELECT Actiu
         INTO V_ACTIU
         FROM PRODUCCIO
        WHERE IDPRODUCCIO  = inProduccio
          AND ROWNUM  = 1;
     EXCEPTION
         WHEN NO_DATA_FOUND THEN RAISE V_PRODUCCIO_EXISTENT;
     END;
--
-- Tercera comprovacio que no estigui de baixa.
--	 
     BEGIN	 
	 	IF (V_ACTIU = 0) THEN
           RAISE V_PRODUCCIO_BAIXA;
        END IF;     
     END;
--
-- QUE EL COMPTADOR TINGUI LINIA DE SUMINISTRAMENT.
--
   BEGIN
      SELECT idLiniaComunicacio 
        INTO v_linia
       FROM COMUNICACIOPRODUCCIO
      WHERE IDPRODUCCIO = inProduccio
	    AND ACTIU = 1
        AND ROWNUM = 1;
    EXCEPTION
         WHEN NO_DATA_FOUND THEN RAISE V_LINIA_EXISTENT;
   END;
--
--
--   
   BEGIN
      SELECT CapacitatMaxima,Actiu 
        INTO v_maxima, v_actiu
       FROM LINIACOMUNICACIO
      WHERE IDLINIACOMUNICACIO = v_linia
        AND ROWNUM = 1;
    EXCEPTION
         WHEN NO_DATA_FOUND THEN RAISE V_LINIA_EXISTENT;
   END;
--
-- Tercera comprovacio que no estigui de baixa.
--	 
     BEGIN	 
	 	IF (V_ACTIU = 0) THEN
           RAISE V_LINIA_BAIXA;
        END IF;     
     END;
--
--
--

--
-- Que la produccio pugui circular
--	 
     BEGIN	 
	 	IF (INKILO > V_MAXIMA) THEN
           RAISE V_LINIA_SUPERIOR;
        END IF;     
     END;   
--   
-- Tots els controls superats.
	  
-- Actulitzo taula historic any TEMES ESTADISTICS.
  BEGIN
     SELECT 'Y'
       INTO hihies
       FROM HISTORICANY
      WHERE BD = 1
	    AND IDANY = TO_CHAR(inData,'YYYY')
	    AND ROWNUM = 1;	  
  EXCEPTION
     -- ALTA.
     WHEN NO_DATA_FOUND THEN 
	 
	 INSERT INTO HISTORICANY (SELECT 1, IDPRODUCCIO, TO_CHAR(SYSDATE,'YYYY'),0,0,0,0,0, 0 FROM PRODUCCIO);
	 
	 RSP := 'OK INSERT HISTORIC TOTAL '||TO_CHAR(inData,'YYYY'); 
     INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_HISTORICLECTURES.PR_ALTA_HISTORICPRODUCCIO', v_in, 'RSP: '||RSP);
		   
  END;
-- Faig l'update TEMES ESTADISTICS .

  UPDATE LINIACOMUNICACIO
     SET CAPACITATPRODUIDA = INKILO + (SELECT SUM(KILOWATS) FROM HISTORICPRODUCCIO WHERE IDPRODUCCIO = INPRODUCCIO) 
   WHERE IDLINIACOMUNICACIO = V_LINIA;
   
	RSP := 'OK UPDATE LINIACOMUNICACIO'; 
    INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_HISTORICLECTURES.PR_ALTA_HISTORICPRODUCCIO', v_in, 'RSP: '||RSP);
      
-- Faig l'update TEMES ESTADISTICS DE LA PRODUCCIO ANUAL

  UPDATE HISTORICANY
     SET PRODUCCIO = PRODUCCIO + INKILO
   WHERE BD = 1
     AND IDANY = TO_CHAR(inData,'YYYY')
	 AND IDCLAU = INPRODUCCIO;

	RSP := 'OK UPDATE HISTORIC TOTAL '||TO_CHAR(inData,'YYYY'); 
    INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_HISTORICLECTURES.PR_ALTA_HISTORICPRODUCCIO', v_in, 'RSP: '||RSP);
		   

-- Actualitzo taula historic total TEMES ESTADISTICS.
  BEGIN
     SELECT Produccio
        INTO v_total
       FROM HISTORICTOTAL
      WHERE ROWNUM = 1;
	  
	 -- HIHES L'ANY UPDATE.
	 UPDATE HISTORICTOTAL 
	    SET PRODUCCIO = PRODUCCIO + INKILO;
	  
	  v_total := v_total + inkilo;

	 RSP := 'OK UPDATE HISTORIC'||TO_CHAR(inData,'YYYY');
     INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_HISTORICLECTURES.PR_ALTA_HISTORICPRODUCCIO', v_in, 'RSP: '||RSP);
	  
  EXCEPTION
     -- ALTA.
     WHEN NO_DATA_FOUND THEN 
	 
	 INSERT INTO HISTORICTOTAL (PRODUCCIO, CONSUM) VALUES (inKilo, 0);
	 
	 v_total := inKilo;
	 
	 RSP := 'OK INSERT HISTORIC '||TO_CHAR(inData,'YYYY'); 
     INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_HISTORICLECTURES.PR_ALTA_HISTORICPRODUCCIO', v_in, 'RSP: '||RSP);
		   
  END;
		   
-- Faig l'update TEMES ESTADISTICS DEL CONSUM TOTAL
   v_total := v_total * 0.30;


  UPDATE HISTORICANY
     SET PRODUCCIOTOTAL = V_TOTAL
   WHERE BD = 1;

	RSP := 'OK UPDATE HISTORIC TOTAL '||TO_CHAR(inData,'YYYY'); 
    INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_HISTORICLECTURES.PR_ALTA_HISTORICPRODUCCIO', v_in, 'RSP: '||RSP);
  
	  
-- AL FI INSERTO LA PRODUCCIO.


      INSERT INTO HISTORICPRODUCCIO (IDPRODUCCIO, DATAPRODUCCIO, KILOWATS)
                  VALUES (inProduccio, inData, inKilo);

      RSP := 'OK';

      INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_HISTORICLECTURES.PR_ALTA_HISTORICPRODUCCIO', v_in, 'RSP: '||RSP);

      COMMIT;

  EXCEPTION
    WHEN V_PRODUCCIO_EXISTENT THEN
         RSP := 'ERROR: Central de produccio no existent en la base de dades';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_HISTORICLECTURES.PR_ALTA_HISTORICPRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_PRODUCCIO_BAIXA THEN
         RSP := 'ERROR: Central de produccio donada de baixa';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_HISTORICLECTURES.PR_ALTA_HISTORICPRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_LINIA_SUPERIOR THEN
         RSP := 'ERROR: Consum superior al permes';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_HISTORICLECTURES.PR_ALTA_HISTORICPRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_LECTURA_EXISTENT THEN
         RSP := 'ERROR: Lectura ja efectuada';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_HISTORICLECTURES.PR_ALTA_HISTORICPRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_LINIA_EXISTENT THEN
         RSP := 'ERROR: Central sense linia de comunicacio';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_HISTORICLECTURES.PR_ALTA_HISTORICPRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_LINIA_BAIXA THEN
         RSP := 'ERROR: Linia de comunicacio donada de baixa';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_HISTORICLECTURES.PR_ALTA_HISTORICPRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_CAMP_NUL THEN
         RSP := 'ERROR: No pot haver camps NULL';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_HISTORICLECTURES.PR_ALTA_HISTORICPRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN OTHERS THEN
         err_num := SQLCODE;
         err_msg := SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_HISTORICLECTURES.PR_ALTA_HISTORICPRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;
  END;
 

END PKG_HISTORICPRODUCCIO;

--
-- FI DEL PACKAGE
--
spool off;
