spool 3-tauless.log
--
-- LOG
--
--DROP TABLE ELE.LOGS CASCADE CONSTRAINTS;
--
CREATE TABLE ELE.LOGS
(
  IDLOGS         NUMBER         NOT NULL,
  DATA           DATE           NOT NULL,
  PROCEDIMENT    VARCHAR2(200)  NOT NULL,
  PARAMSIN       VARCHAR2(2000) NOT NULL,
  PARAMSOUT      VARCHAR2(2000) NOT NULL
);
--
ALTER TABLE ELE.LOGS
  ADD CONSTRAINT PK_LOGS PRIMARY KEY (IDLOGS)
  USING INDEX
  TABLESPACE ELE_IND;
--
-- PAIS
--
--DROP TABLE ELE.PAIS CASCADE CONSTRAINTS;
--
CREATE TABLE ELE.PAIS
(
  IDPAIS             NUMBER       NOT NULL,
  NOMPAIS            VARCHAR2(30) NOT NULL
)
TABLESPACE ELE_DAT;
--
ALTER TABLE ELE.PAIS
  ADD CONSTRAINT PK_PAIS PRIMARY KEY (IDPAIS)
  USING INDEX
  TABLESPACE ELE_IND;
--
-- PROVINCIA
--
--DROP TABLE ELE.PROVINCIA CASCADE CONSTRAINTS;
--
CREATE TABLE ELE.PROVINCIA
(
  IDPROVINCIA        NUMBER       NOT NULL,
  IDPAIS             NUMBER       NOT NULL,
  NOMPROVINCIA       VARCHAR2(30) NOT NULL
)
TABLESPACE ELE_DAT;
--
ALTER TABLE ELE.PROVINCIA
  ADD CONSTRAINT PK_PROVINCIA PRIMARY KEY (IDPROVINCIA)
  USING INDEX
  TABLESPACE ELE_IND;
ALTER TABLE ELE.PROVINCIA
  ADD CONSTRAINT FK_PAIS FOREIGN KEY (IDPAIS)
  REFERENCES ELE.PAIS (IDPAIS);
-- INDEX
CREATE INDEX ELE.I_PROVINCIA_IDPAIS ON ELE.PROVINCIA(IDPAIS) TABLESPACE  ELE_IND;
--
-- POBLACIO
--
--DROP TABLE ELE.POBLACIO CASCADE CONSTRAINTS;
--
CREATE TABLE ELE.POBLACIO
(
  IDPOBLACIO         NUMBER       NOT NULL,
  IDPROVINCIA        NUMBER       NOT NULL,
  NOMPOBLACIO        VARCHAR2(30) NOT NULL
)
TABLESPACE ELE_DAT;
--
ALTER TABLE ELE.POBLACIO
  ADD CONSTRAINT PK_POBLACIO PRIMARY KEY (IDPOBLACIO)
  USING INDEX
  TABLESPACE ELE_IND;
ALTER TABLE ELE.POBLACIO
  ADD CONSTRAINT FK_PROVINCIA FOREIGN KEY (IDPROVINCIA)
  REFERENCES ELE.PROVINCIA (IDPROVINCIA);
-- INDEX
CREATE INDEX ELE.I_POBLACIO_IDPROVINCIA ON ELE.POBLACIO(IDPROVINCIA) TABLESPACE  ELE_IND;
--
-- MODELS
--
--DROP TABLE ELE.MODEL CASCADE CONSTRAINTS;
--
CREATE TABLE ELE.MODEL
(
  IDMODEL            NUMBER       NOT NULL,
  NOMMODEL           VARCHAR2(30) NOT NULL,
  EMPRESA            VARCHAR2(30) NOT NULL
)
TABLESPACE ELE_DAT;
--
ALTER TABLE ELE.MODEL
  ADD CONSTRAINT PK_MODEL PRIMARY KEY (IDMODEL)
  USING INDEX
  TABLESPACE ELE_IND;
--
-- TIPUS CENTRAL
--
--DROP TABLE ELE.CENTRAL CASCADE CONSTRAINTS;
--
CREATE TABLE ELE.CENTRAL
(
  IDCENTRAL          NUMBER       NOT NULL,
  TIPUS              NUMBER,
  NOMCENTRAL         VARCHAR2(30) NOT NULL,
  ENERGIA            NUMBER,
  REBUIG             NUMBER,
  EMISSIO            NUMBER,
  APARELLS           NUMBER
)
TABLESPACE ELE_DAT;
--
ALTER TABLE ELE.CENTRAL
  ADD CONSTRAINT PK_CENTRAL PRIMARY KEY (IDCENTRAL)
  USING INDEX
  TABLESPACE ELE_IND;
--
-- MESTRES
--
--
-- CLIENTS
--
--DROP TABLE ELE.CLIENTS CASCADE CONSTRAINTS;
--
CREATE TABLE ELE.CLIENTS
(
  IDCLIENT           NUMBER       NOT NULL,
  NIFCLIENT          VARCHAR2(15) NOT NULL,
  NOMCLIENT          VARCHAR2(30) NOT NULL,
  ADRECA             VARCHAR2(30)         ,
  IDPAIS             NUMBER       NOT NULL,
  IDPROVINCIA        NUMBER       NOT NULL,
  IDPOBLACIO         NUMBER       NOT NULL,
  CONSUMTOTAL        NUMBER               ,
  COMPTADORS         NUMBER               ,
  CONSUMMIG          NUMBER               ,
  ACTIU              NUMBER       NOT NULL,
  DATAALTA           DATE                 ,
  DATABAIXA          DATE
)
TABLESPACE ELE_DAT;
--
ALTER TABLE ELE.CLIENTS
  ADD CONSTRAINT PK_CLIENT PRIMARY KEY (IDCLIENT)
  USING INDEX
  TABLESPACE ELE_IND;
ALTER TABLE ELE.CLIENTS
  ADD CONSTRAINT FK_CL_PAIS FOREIGN KEY (IDPAIS)
  REFERENCES ELE.PAIS (IDPAIS);
ALTER TABLE ELE.CLIENTS
  ADD CONSTRAINT FK_CL_PROVINCIA FOREIGN KEY (IDPROVINCIA)
  REFERENCES ELE.PROVINCIA (IDPROVINCIA);
ALTER TABLE ELE.CLIENTS
  ADD CONSTRAINT FK_CL_POBLACIO FOREIGN KEY (IDPOBLACIO)
  REFERENCES ELE.POBLACIO (IDPOBLACIO);
--
ALTER TABLE ELE.CLIENTS
  ADD CONSTRAINT CK_CL_ACTIU CHECK (ACTIU BETWEEN 0 AND 1); 
-- INDEX
CREATE INDEX ELE.I_CLIENTS_IDPAIS ON ELE.CLIENTS(IDPAIS) TABLESPACE  ELE_IND;
CREATE INDEX ELE.I_CLIENTS_IDPROVINCIA ON ELE.CLIENTS(IDPROVINCIA) TABLESPACE  ELE_IND;
CREATE INDEX ELE.I_CLIENTS_IDPOBLACI ON ELE.CLIENTS(IDPOBLACIO) TABLESPACE  ELE_IND;
CREATE INDEX ELE.I_CLIENTS_IDNIF ON ELE.CLIENTS(NIFCLIENT) TABLESPACE ELE_IND;
--
-- COMPTADORS
--
--DROP TABLE ELE.COMPTADORS CASCADE CONSTRAINTS;
--
CREATE TABLE ELE.COMPTADORS
(
  IDCOMPTADOR        NUMBER       NOT NULL,
  IDCONTRACTE        VARCHAR(15)  NOT NULL,
  IDCLIENT           NUMBER       NOT NULL,
  ADRECA             VARCHAR2(30)         ,
  IDPAIS             NUMBER       NOT NULL,
  IDPROVINCIA        NUMBER       NOT NULL,
  IDPOBLACIO         NUMBER       NOT NULL,
  POTENCIACONTRACTE  NUMBER               ,
  CONSUMACTUAL       NUMBER               ,
  DATALECTURA        DATE                 ,
  IDMODEL            NUMBER       NOT NULL,
  ANYFABRICACIO      VARCHAR2(4)          ,
  ACTIU              NUMBER       NOT NULL,
  DATAALTA           DATE                 ,
  DATABAIXA          DATE
)
TABLESPACE ELE_DAT;
--
ALTER TABLE ELE.COMPTADORS
  ADD CONSTRAINT PK_COMPTADOR PRIMARY KEY (IDCOMPTADOR)
  USING INDEX
  TABLESPACE ELE_IND;
ALTER TABLE ELE.COMPTADORS
  ADD CONSTRAINT FK_CO_CLIENT FOREIGN KEY (IDCLIENT)
  REFERENCES ELE.CLIENTS (IDCLIENT);
ALTER TABLE ELE.COMPTADORS
  ADD CONSTRAINT FK_CO_PAIS FOREIGN KEY (IDPAIS)
  REFERENCES ELE.PAIS (IDPAIS);
ALTER TABLE ELE.COMPTADORS
  ADD CONSTRAINT FK_CO_PROVINCIA FOREIGN KEY (IDPROVINCIA)
  REFERENCES ELE.PROVINCIA (IDPROVINCIA);
ALTER TABLE ELE.COMPTADORS
  ADD CONSTRAINT FK_CO_POBLACIO FOREIGN KEY (IDPOBLACIO)
  REFERENCES ELE.POBLACIO (IDPOBLACIO);
ALTER TABLE ELE.COMPTADORS
  ADD CONSTRAINT FK_CO_MODEL FOREIGN KEY (IDMODEL)
  REFERENCES ELE.MODEL (IDMODEL);
--
ALTER TABLE ELE.COMPTADORS
  ADD CONSTRAINT CK_CO_ACTIU CHECK (ACTIU BETWEEN 0 AND 1); 
-- INDEX
CREATE INDEX ELE.I_COMPTADORS_IDCLIENT ON ELE.COMPTADORS(IDCLIENT) TABLESPACE  ELE_IND;
CREATE INDEX ELE.I_COMPTADORS_IDPAIS ON ELE.COMPTADORS(IDPAIS) TABLESPACE  ELE_IND;
CREATE INDEX ELE.I_COMPTADORS_IDPROVINCIA ON ELE.COMPTADORS(IDPROVINCIA) TABLESPACE  ELE_IND;
CREATE INDEX ELE.I_COMPTADORS_IDPOBLACIO ON ELE.COMPTADORS(IDPOBLACIO) TABLESPACE  ELE_IND;
CREATE INDEX ELE.I_COMPTADORS_IDMODEL ON ELE.COMPTADORS(IDMODEL) TABLESPACE  ELE_IND;
CREATE INDEX ELE.I_COMPTADORS_IDCONTRACTE ON ELE.COMPTADORS(IDCONTRACTE) TABLESPACE  ELE_IND;
--
-- LINIES DE COMUNICACIO
--
--DROP TABLE ELE.LINIACOMUNICACIO CASCADE CONSTRAINTS;
--
CREATE TABLE ELE.LINIACOMUNICACIO
(
  IDLINIACOMUNICACIO NUMBER       NOT NULL,
  IDREFERENCIA       VARCHAR2(15) NOT NULL,
  CAPACITATMAXIMA    NUMBER               ,
  CAPACITATPRODUIDA  NUMBER               ,
  CONSUM             NUMBER               ,
  ACTIU              NUMBER       NOT NULL,
  DATAALTA           DATE                 ,
  DATABAIXA          DATE
)
TABLESPACE ELE_DAT;
--
ALTER TABLE ELE.LINIACOMUNICACIO
  ADD CONSTRAINT PK_LINIACOMUNICACIO PRIMARY KEY (IDLINIACOMUNICACIO)
  USING INDEX
  TABLESPACE ELE_IND;
--
ALTER TABLE ELE.LINIACOMUNICACIO
  ADD CONSTRAINT CK_LI_ACTIU CHECK (ACTIU BETWEEN 0 AND 1); 
-- INDEX
CREATE INDEX ELE.I_LINIACOMUNICACIO_ID ON ELE.LINIACOMUNICACIO(IDREFERENCIA) TABLESPACE  ELE_IND;
--
-- CENTRALS PRODUCCIO
--
--DROP TABLE ELE.PRODUCCIO CASCADE CONSTRAINTS;
--
CREATE TABLE ELE.PRODUCCIO
(
  IDPRODUCCIO        NUMBER       NOT NULL,
  IDCODICENTRAL      VARCHAR(15)  NOT NULL,
  ADRECA             VARCHAR2(30)         ,
  IDPAIS             NUMBER       NOT NULL,
  IDPROVINCIA        NUMBER       NOT NULL,
  IDPOBLACIO         NUMBER       NOT NULL,
  ENERGIAMAXIMA      NUMBER               ,
  CONSUMACTUAL       NUMBER               ,
  DATADARRERAINSPE   DATE                 ,
  IDCENTRAL          NUMBER       NOT NULL,
  CONSUMSCOMPTADOR   NUMBER               ,
  ACTIU              NUMBER       NOT NULL,
  DATAALTA           DATE                 ,
  DATABAIXA          DATE
)
TABLESPACE ELE_DAT;
--
ALTER TABLE ELE.PRODUCCIO
  ADD CONSTRAINT PK_PRODUCCIO PRIMARY KEY (IDPRODUCCIO)
  USING INDEX
  TABLESPACE ELE_IND;
ALTER TABLE ELE.PRODUCCIO
  ADD CONSTRAINT FK_PR_PAIS FOREIGN KEY (IDPAIS)
  REFERENCES ELE.PAIS (IDPAIS);
ALTER TABLE ELE.PRODUCCIO
  ADD CONSTRAINT FK_PR_PROVINCIA FOREIGN KEY (IDPROVINCIA)
  REFERENCES ELE.PROVINCIA (IDPROVINCIA);
ALTER TABLE ELE.PRODUCCIO
  ADD CONSTRAINT FK_PR_POBLACIO FOREIGN KEY (IDPOBLACIO)
  REFERENCES ELE.POBLACIO (IDPOBLACIO);
ALTER TABLE ELE.PRODUCCIO
  ADD CONSTRAINT FK_PR_CENTRAL FOREIGN KEY (IDCENTRAL)
  REFERENCES ELE.CENTRAL (IDCENTRAL);
--
ALTER TABLE ELE.PRODUCCIO
  ADD CONSTRAINT CK_PR_ACTIU CHECK (ACTIU BETWEEN 0 AND 1); 
-- INDEX
CREATE INDEX ELE.I_PRODUCCIO_IDPAIS ON ELE.PRODUCCIO(IDPAIS)  TABLESPACE  ELE_IND;
CREATE INDEX ELE.I_PRODUCCIO_IDPROVINCIA ON ELE.PRODUCCIO(IDPROVINCIA) TABLESPACE  ELE_IND;
CREATE INDEX ELE.I_PRODUCCIO_IDPOBLACIO ON ELE.PRODUCCIO(IDPOBLACIO) TABLESPACE  ELE_IND;
CREATE INDEX ELE.I_PRODUCCIO_IDCENTRAL ON ELE.PRODUCCIO(IDCENTRAL) TABLESPACE  ELE_IND;
CREATE INDEX ELE.I_PRODUCCIO_IDCODICENTRAL ON ELE.PRODUCCIO(IDCODICENTRAL) TABLESPACE  ELE_IND;
--
-- CENTRALS DISTRIBUCIO
--
--DROP TABLE ELE.DISTRIBUCIO CASCADE CONSTRAINTS;
--
CREATE TABLE ELE.DISTRIBUCIO
(
  IDDISTRIBUCIO      NUMBER       NOT NULL,
  IDCENTRALETA       VARCHAR(15)  NOT NULL,
  ADRECA             VARCHAR2(30)         ,
  IDPAIS             NUMBER       NOT NULL,
  IDPROVINCIA        NUMBER       NOT NULL,
  IDPOBLACIO         NUMBER       NOT NULL,
  ACTIU              NUMBER       NOT NULL,
  DATAALTA           DATE                 ,
  DATABAIXA          DATE
)
TABLESPACE ELE_DAT;
--
ALTER TABLE ELE.DISTRIBUCIO
  ADD CONSTRAINT PK_DISTRIBUCIO PRIMARY KEY (IDDISTRIBUCIO)
  USING INDEX
  TABLESPACE ELE_IND;
ALTER TABLE ELE.DISTRIBUCIO
  ADD CONSTRAINT FK_DI_PAIS FOREIGN KEY (IDPAIS)
  REFERENCES ELE.PAIS (IDPAIS);
ALTER TABLE ELE.DISTRIBUCIO
  ADD CONSTRAINT FK_DI_PROVINCIA FOREIGN KEY (IDPROVINCIA)
  REFERENCES ELE.PROVINCIA (IDPROVINCIA);
ALTER TABLE ELE.DISTRIBUCIO
  ADD CONSTRAINT FK_DI_POBLACIO FOREIGN KEY (IDPOBLACIO)
  REFERENCES ELE.POBLACIO (IDPOBLACIO);
--
ALTER TABLE ELE.DISTRIBUCIO
  ADD CONSTRAINT CK_DI_ACTIU CHECK (ACTIU BETWEEN 0 AND 1); 
-- INDEX
CREATE INDEX ELE.I_DISTRIBUCIO_IDPAIS ON ELE.DISTRIBUCIO(IDPAIS)  TABLESPACE  ELE_IND;
CREATE INDEX ELE.I_DISTRIBUCIO_IDPROVINCIA ON ELE.DISTRIBUCIO(IDPROVINCIA) TABLESPACE  ELE_IND;
CREATE INDEX ELE.I_DISTRIBUCIO_IDPOBLACIO ON ELE.DISTRIBUCIO(IDPOBLACIO) TABLESPACE  ELE_IND;
CREATE INDEX ELE.I_DISTRIBUCIO_IDCENTRALETA ON ELE.DISTRIBUCIO(IDCENTRALETA) TABLESPACE  ELE_IND;


-- AUXILIARS
--
-- COMUNICACIO DISTRIBUCIO
--
--DROP TABLE ELE.COMUNICACIODISTRIBUCIO CASCADE CONSTRAINTS;
--
CREATE TABLE ELE.COMUNICACIODISTRIBUCIO
(
  IDDISTRIBUCIO      NUMBER       NOT NULL,
  IDCOMPTADOR        NUMBER       NOT NULL,
  ID                 NUMBER       NOT NULL,
  ACTIU              NUMBER       NOT NULL,
  DATAALTA           DATE                 ,
  DATABAIXA          DATE
)
TABLESPACE ELE_DAT;
ALTER TABLE ELE.COMUNICACIODISTRIBUCIO
  ADD CONSTRAINT PK_COMUNICACIODISTRIBUCIO PRIMARY KEY (IDDISTRIBUCIO,IDCOMPTADOR)
  USING INDEX
  TABLESPACE ELE_IND;
--
-- COMUNICACIO PRODUCCIO
--
--DROP TABLE ELE.COMUNICACIOPRODUCCIO CASCADE CONSTRAINTS;
--
CREATE TABLE ELE.COMUNICACIOPRODUCCIO
(
  IDDISTRIBUCIO      NUMBER       NOT NULL,
  IDLINIACOMUNICACIO NUMBER       NOT NULL,
  IDPRODUCCIO        NUMBER       NOT NULL,
  ID                 NUMBER       NOT NULL,
  ACTIU              NUMBER       NOT NULL,
  DATAALTA           DATE                 ,
  DATABAIXA          DATE
)
TABLESPACE ELE_DAT;
ALTER TABLE ELE.COMUNICACIOPRODUCCIO
  ADD CONSTRAINT PK_COMUNICACIOPRODUCCIO PRIMARY KEY (IDLINIACOMUNICACIO,IDPRODUCCIO, IDDISTRIBUCIO)
  USING INDEX
  TABLESPACE ELE_IND;
-- INDEX
CREATE INDEX ELE.I_COMUNICACIOPRODUCCIO_IDCLAU ON ELE.COMUNICACIOPRODUCCIO(IDPRODUCCIO)  TABLESPACE  ELE_IND;
---
-- HISTORIC LECTURES.
--
--DROP TABLE ELE.HISTORICLECTURES CASCADE CONSTRAINTS;
--
CREATE TABLE ELE.HISTORICLECTURES
(
  IDCOMPTADOR        NUMBER       NOT NULL,
  DATALECTURA        DATE         NOT NULL,
  CONSUMACTUAL       NUMBER       NOT NULL,
  CONSUM             NUMBER       NOT NULL,
  TIPUSLECTURA       NUMBER       NOT NULL,
  ID                 NUMBER       NOT NULL
)
TABLESPACE ELE_DAT;
ALTER TABLE ELE.HISTORICLECTURES
  ADD CONSTRAINT PK_HISTORICLECTURES PRIMARY KEY (ID)
  USING INDEX
  TABLESPACE ELE_IND;
-- INDEX
CREATE INDEX ELE.I_HISTORICLECTURES_IDSEARCH ON ELE.HISTORICLECTURES(IDCOMPTADOR,DATALECTURA)  TABLESPACE  ELE_IND;
---
-- HISTORIC PRODUCCIO.
--
--DROP TABLE ELE.HISTORICPRODUCCIO CASCADE CONSTRAINTS;
--
CREATE TABLE ELE.HISTORICPRODUCCIO
(
  IDPRODUCCIO        NUMBER       NOT NULL,
  DATAPRODUCCIO      DATE         NOT NULL,
  KILOWATS           NUMBER       NOT NULL,
  ID                 NUMBER       NOT NULL
)
TABLESPACE ELE_DAT;
ALTER TABLE ELE.HISTORICPRODUCCIO
  ADD CONSTRAINT PK_HISTORICPRODUCCIO PRIMARY KEY (ID)
  USING INDEX
  TABLESPACE ELE_IND;
-- INDEX
CREATE INDEX ELE.I_HISTORICPRODUCCIO_IDSEARCH ON ELE.HISTORICPRODUCCIO(IDPRODUCCIO,DATAPRODUCCIO)  TABLESPACE  ELE_IND;
---
-- HISTORIC ANY
--
--DROP TABLE ELE.HISTORICANY CASCADE CONSTRAINTS;
--
CREATE TABLE ELE.HISTORICANY
(
  BD                 NUMBER       NOT NULL,
  IDCLAU             NUMBER       NOT NULL,
  IDANY              NUMBER       NOT NULL,
  PRODUCCIO          NUMBER       NOT NULL,
  CONSUM             NUMBER       NOT NULL,
  CONSUMAVG          NUMBER       NOT NULL,
  PRODUCCIOTOTAL     NUMBER       NOT NULL,
  CONSUMTOTAL        NUMBER       NOT NULL,
  ID                 NUMBER       NOT NULL
)
TABLESPACE ELE_DAT;
ALTER TABLE ELE.HISTORICANY
  ADD CONSTRAINT PK_HISTORICANY PRIMARY KEY (ID)
  USING INDEX
  TABLESPACE ELE_IND;
-- INDEX
CREATE UNIQUE INDEX ELE.I_HISTORICANY_IDSEARCH ON ELE.HISTORICANY(BD, IDCLAU, IDANY)  TABLESPACE  ELE_IND;
---
-- HISTORIC TOTAL.
--
--DROP TABLE ELE.HISTORICTOTAL CASCADE CONSTRAINTS;
--
CREATE TABLE ELE.HISTORICTOTAL
(
  PRODUCCIO          NUMBER       NOT NULL,
  CONSUM             NUMBER       NOT NULL,
  ID                 NUMBER       NOT NULL
)
TABLESPACE ELE_DAT;
ALTER TABLE ELE.HISTORICTOTAL
  ADD CONSTRAINT PK_HISTORICTOTAL PRIMARY KEY (ID)
  USING INDEX
  TABLESPACE ELE_IND;

spool off;
