CREATE OR REPLACE PROCEDURE CONSUMPRODUCCIO(inProduccio IN INTEGER, inDataIni IN DATE, inDataFin IN DATE)
IS 	
  -- Variables de treball				  
  V_CAMP_NUL    EXCEPTION;
  V_NODADES     EXCEPTION;
  outProduccio  NUMBER;
  outConsum     NUMBER;
  v_in          VARCHAR2(500);
  rsp           VARCHAR2(100);
  err_num       NUMBER;
  err_msg       VARCHAR2(100);
  t1            NUMBER;
  t2            NUMBER;  
BEGIN
   v_in := 'Central de produccio:'||inProduccio||', Data inicial:'||InDataIni||', Data final:'||inDataFin;
   --   
   -- Comprovar que parametre d'entrada
   --
   IF (TRIM(inProduccio)     IS NULL) THEN
      RAISE V_CAMP_NUL;
   END IF;
   IF (TRIM(inDataIni)     IS NULL) THEN
      RAISE V_CAMP_NUL;
   END IF;
   IF (TRIM(inDataFin)     IS NULL) THEN
      RAISE V_CAMP_NUL;
   END IF;
   -- 
   -- Sument tota la produccio en l'interval de temps donat.
   BEGIN
      SELECT SUM(KILOWATS) 
        INTO outProduccio
        FROM HISTORICPRODUCCIO
       WHERE IDPRODUCCIO = inProduccio
         AND DATAPRODUCCIO >= inDataIni
         AND DATAPRODUCCIO <= inDataFin;
  EXCEPTION
    WHEN NO_DATA_FOUND THEN outProduccio := 0; 
  END;
  --
  -- Sument tot el consum en l'interval de temps donat.
  --
  BEGIN
    SELECT SUM(CONSUM)
      INTO outConsum
      FROM HISTORICLECTURES
     WHERE DATALECTURA >= inDataIni
       AND DATALECTURA <= inDataFin
       AND IDCOMPTADOR IN (SELECT IDCOMPTADOR
                             FROM COMUNICACIODISTRIBUCIO
                            WHERE ACTIU = 1
                              AND IDDISTRIBUCIO IN (SELECT IDDISTRIBUCIO
                                                      FROM COMUNICACIOPRODUCCIO
                                                     WHERE IDPRODUCCIO = inProduccio
                                      			       AND ACTIU = 1));  
  EXCEPTION
    WHEN NO_DATA_FOUND THEN outConsum := 0;
  END;
  --
  -- No temin dades
  --	
  IF ((outProduccio = 0) AND (outConsum = 0)) THEN
     RAISE V_NODADES;
  END IF;
  --
  -- Mostrem les dades
  --
  rsp := 'Central Produccio: '||inProduccio;
  DBMS_OUTPUT.put_line(rsp);
  rsp := 'Consum comptadors: '||to_char(outConsum,'99,999,999.99');
  DBMS_OUTPUT.put_line(rsp);
  rsp := 'Energia produida.: '||to_char(outProduccio,'99,999,999.99');
  DBMS_OUTPUT.put_line(rsp);
  --
  -- Excepcions
  --  
  EXCEPTION
    WHEN V_NODADES THEN
         RSP := 'ERROR: NO hi ha Produccio i/o consum';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
                VALUES (SYSDATE,'PROCEDURE.CONSUMPRODUCCIO', v_in, 'RSP: '||RSP);
         DBMS_OUTPUT.put_line(rsp);
         COMMIT;
    WHEN V_CAMP_NUL THEN
         RSP := 'ERROR: No pot haver camps NULL';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
                VALUES (SYSDATE,'PROCEDURE.CONSUMPRODUCCIO', v_in, 'RSP: '||RSP);
         DBMS_OUTPUT.put_line(rsp);
         COMMIT;
    WHEN OTHERS THEN
         err_num := SQLCODE;
         err_msg := SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
                VALUES (SYSDATE,'PROCEDURE.CONSUMPRODUCCIO', v_in, 'RSP: '||RSP);
         COMMIT;		 		     
END;
