CREATE OR REPLACE PROCEDURE PERCENTATGELECTURES(inDataIni IN Date, inDataFin IN Date)
IS 	
  -- VAriables de treball				  
  V_CAMP_NUL    EXCEPTION;
  V_NODADES     EXCEPTION;
  outPresencial NUMBER;
  outTelematica NUMBER;
  v_in          VARCHAR2(500);
  rsp           VARCHAR2(100);
  err_num       NUMBER;
  err_msg       VARCHAR2(100);
  t1            NUMBER;
  t2            NUMBER;  
BEGIN
  v_in := 'Data inicial:'||InDataIni||', Data final:'||inDataFin;
  --   
  -- Comprovar que parametre d'entrada
  --
  IF (TRIM(inDataIni)     IS NULL) THEN
     RAISE V_CAMP_NUL;
  END IF;
  IF (TRIM(inDataFin)     IS NULL) THEN
     RAISE V_CAMP_NUL;
  END IF;
  --
  -- Contem totes les lectures presencials
  --
  BEGIN
    SELECT COUNT(*)  
      INTO outPresencial
      FROM HISTORICLECTURES
    WHERE TIPUSLECTURA = 1
      AND DATALECTURA >= inDataIni
   	  AND DATALECTURA <= inDataFin;
  EXCEPTION
    WHEN NO_DATA_FOUND THEN outPresencial := 0;
  END;
  --
  -- Contem totes les lectures telematiques
  -- 
  BEGIN	
     SELECT COUNT(*)  
       INTO outTelematica
       FROM HISTORICLECTURES
      WHERE TIPUSLECTURA = 0
        AND DATALECTURA >= inDataIni
     	AND DATALECTURA <= inDataFin;
  EXCEPTION
    WHEN NO_DATA_FOUND THEN outTelematica := 0;
  END;
  --
  -- NO temim dades
  --	
  IF ((outPresencial = 0) AND (outTelematica = 0)) THEN
     RAISE V_NODADES;
  END IF;
  --
  -- Calculem els percentatges
  --
  t1 := outPresencial + outTelematica;
  t2 := round((outPresencial / t1) * 100,0);
  t1 := 100 - t2;
  --
  -- Mostrem els percentatges
  --
  rsp := 'LECTURES PRESENCIALS .: '||t2||'%';
  DBMS_OUTPUT.put_line(rsp);
  rsp := 'LECTURES TELEMATIQUES : '||t1||'%';
  DBMS_OUTPUT.put_line(rsp);
  --
  -- Excepcions
  --  
  EXCEPTION
    WHEN V_NODADES THEN
         RSP := 'ERROR: NO hi ha lectures';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
                VALUES (SYSDATE,'PROCEDURE.PERCENTATGELECTURES', v_in, 'RSP: '||RSP);
         DBMS_OUTPUT.put_line(rsp);
         COMMIT;
    WHEN V_CAMP_NUL THEN
         RSP := 'ERROR: No pot haver camps NULL';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
                VALUES (SYSDATE,'PROCEDURE.PERCENTATGELECTURES', v_in, 'RSP: '||RSP);
         DBMS_OUTPUT.put_line(rsp);
         COMMIT;
    WHEN OTHERS THEN
         err_num := SQLCODE;
         err_msg := SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
                VALUES (SYSDATE,'PROCEDURE.PERCENTATGELECTURES', v_in, 'RSP: '||RSP);
         COMMIT;		 		     
END;
