spool 7-package_clients.log

CREATE OR REPLACE PACKAGE ELE.PKG_CLIENTS AS

--
-- NOM.........:   PKG_CLIENTS
-- FINALITAT...:   GESTIO DE TOTA LA INFORMACIO REFERENTS A CLIENTS.
--
--
--
--
-- NOM.........:   PR_ALTA_CLIENTS
-- FINALITAT...:   DONAR D'ALTA UN NOU CLIENT
-- CONTROLS....:   INFORMAR CAMPS OBLIGATORIS
--                 NIF NO REPETIT
--              
-- PARAMETRES..:   
--         IN..:   nifClient..: nif del client (Obligatori)
--	             nomClient..: nom del client (Obliagtoir)
--                 adreca.....: adrea del client
--                 idPoblacio.: codi de la poblacio (Obligatori), a partir d'aquest codi obtindrem idProvincia i idPais.
--        OUT..:   RSP
--
-- PRECONDICIO.:   CLIENT NO EXISTEIX
-- POSTCONDICIO:   ES DONA D'ALTA EL CLIENT
-- RETURN .....:   OK SI S'HA EFECTUAT L'ALTA
--                 ERROR ALGUN CAMP DEL PARAMETRE NO TE VALOR
--                 ERROR REGISTRE DUPLICAT (CLIENT EXISTENT)
--                 ERROR PROVINCIA NO EXISTENT
--
--

  PROCEDURE PR_ALTA_CLIENT (inNif            IN CLIENTS.NIFCLIENT%TYPE,
                            inNom            IN CLIENTS.NOMCLIENT%TYPE,
                            inAdreca         IN CLIENTS.ADRECA%TYPE,
                            inPoblacio       IN CLIENTS.IDPOBLACIO%TYPE,
                            RSP              OUT VARCHAR2);

--
-- NOM.........:   PR_BAIXA_CLIENTS
-- FINALITAT...:   DONAR DE BAIXA UN CLIENT
-- CONTROLS....:   CLIENT NO DONAT DE BAIXA
--                 CLIENT EXISTEIX EN EL FITXER
--                 CLIENT SENSE CONTRACTES
--              
-- PARAMETRES..:   
--         IN..:   idClient: Codi del client (Obligatori)
--        OUT..:   RSP
--
-- PRECONDICIO.:   CLIENT EXISTEIX
-- POSTCONDICIO:   ES DONA DE BAIXA EL CLIENT
-- RETURN .....:   OK SI S'HA EFECTUAT LA BAIXA
--                 ERROR SI EL CLIENT NO EXISTEIX
--                 ERROR SI EL CLIENT ESTA DONAT DE BAIXA
--                 ERROR SI EL CLIENT TE CONTRACTES.
--
--

  PROCEDURE PR_BAIXA_CLIENT (inClient        IN CLIENTS.IDCLIENT%TYPE,
                             RSP             OUT VARCHAR2);

--
-- NOM.........:   PR_MODIFICACIO_CLIENTS
-- FINALITAT...:   PODER MODIFICAR QUALSEVOL CAMP D'UN CLIENT
-- CONTROLS....:   CLIENT EXISTENT EN EL FITXER
--                 INFORMAR CAMPS OBLIGATORIS
--                 NIF NO REPETIT
--              
-- PARAMETRES..:   
--         IN..:   idclient ..: codi del client (obligatori)
--                 nifClient..: nif del client
--		           nomClient..: nom del client
--                 adreca.....: adrea del client
--                 idPoblacio.: codi de la poblacio, a partir d'aquest codi obtindrem idProvincia i idPais.
--                 ConsumMig..: Consum del client
--                 Actiu......: Canviar status del client
--                 DataAlta...: Canviar la data d'alta del client
--                 DataBaixa..: Canviar la data de baixa del client
--        OUT..:   RSP
--
-- PRECONDICIO.:   CLIENT EXISTEIX
-- POSTCONDICIO:   ES REALITZA LA MODIFICACIO DELS CAMPS DEL CLIENT
-- RETURN .....:   OK SI S'HA EFECTUAT LA MODIFICACIO
--                 ERROR ALGUN CAMP DEL PARAMETRE NO TE VALOR
--                 ERROR REGISTRE DUPLICAT (CLIENT EXISTENT)
--                 ERROR ALTA NO CONTE VALORS 0 - 1
--                 ERROR PROVINCIA NO EXISTENT
--
--

  PROCEDURE PR_MODIFICACIO_CLIENT (inClient         IN CLIENTS.IDCLIENT%TYPE,
                                   inNif            IN CLIENTS.NIFCLIENT%TYPE,
                                   inNom            IN CLIENTS.NOMCLIENT%TYPE,
                                   inAdreca         IN CLIENTS.ADRECA%TYPE,
                                   inPoblacio       IN CLIENTS.IDPOBLACIO%TYPE,
                                   inConsum         IN CLIENTS.CONSUMMIG%TYPE,
                                   inDataAlta       IN CLIENTS.DATAALTA%TYPE,
                                   inDataBaixa      IN CLIENTS.DATABAIXA%TYPE,
                                   RSP              OUT VARCHAR2);

END PKG_CLIENTS;

--
--
--
CREATE OR REPLACE PACKAGE BODY ELE.PKG_CLIENTS AS

  PROCEDURE PR_ALTA_CLIENT (inNif            IN CLIENTS.NIFCLIENT%TYPE,
                            inNom            IN CLIENTS.NOMCLIENT%TYPE,
                            inAdreca         IN CLIENTS.ADRECA%TYPE,
                            inPoblacio       IN CLIENTS.IDPOBLACIO%TYPE,
                            RSP              OUT VARCHAR2)
  IS

   V_CAMP_NUL             EXCEPTION;
   V_CLIENT_EXISTENT      EXCEPTION;
   V_POBLACIO_EXISTENT    EXCEPTION;
   V_PROVINCIA_EXISTENT   EXCEPTION;
   V_PAIS_EXISTENT        EXCEPTION;

   v_in          VARCHAR2(500);
   hihies        VARCHAR2(1);
   v_Provincia   NUMBER;
   v_Pais        NUMBER;
   err_num       NUMBER;
   err_msg       VARCHAR2(100);


   BEGIN
     v_in := 'Nif:'||inNif||', Nom:'||inNom||', Adreca:'||inAdreca||', Poblacio:'||inPoblacio;
--
-- Primera comprovacio que tots els camps obligatoris tinguin valor
--
     IF (TRIM(inNif)     IS NULL) THEN
         RAISE V_CAMP_NUL;
     END IF;
     IF (TRIM(inNom)     IS NULL) THEN
         RAISE V_CAMP_NUL;
     END IF;
     IF (TRIM(inPoblacio)     IS NULL) THEN
         RAISE V_CAMP_NUL;
     END IF;
--
-- Segona comprovacio que el NIf no es trovi amb un altre id
--
     BEGIN
       SELECT 'Y'
         INTO hihies
         FROM CLIENTS
        WHERE NIFCLIENT  = inNif
          AND ROWNUM  = 1;
        RAISE V_CLIENT_EXISTENT;
    EXCEPTION
         WHEN NO_DATA_FOUND THEN NULL;
    END;
--
-- Tercera comprovacio que el IdPoblacio estigui donat d'alta
--
    BEGIN
      SELECT idProvincia
        INTO v_Provincia
        FROM Poblacio
       WHERE idPoblacio = inPoblacio
         AND ROWNUM = 1;      
   EXCEPTION
        WHEN NO_DATA_FOUND THEN RAISE V_POBLACIO_EXISTENT;
   END;
--
-- Miro el Pais que te la privincia
--	
    BEGIN	
  	  SELECT idPais
        INTO v_Pais
        FROM Provincia
       WHERE idProvincia = v_Provincia
         AND ROWNUM = 1;      
   EXCEPTION
        WHEN NO_DATA_FOUND THEN RAISE V_PROVINCIA_EXISTENT;
   END;
--
-- Miro que el Pais estigui donat d'alta
--	
   BEGIN
     SELECT 'Y'
       INTO hihies
       FROM Pais
      WHERE idPais = v_Pais
        AND ROWNUM = 1;      
  EXCEPTION
       WHEN NO_DATA_FOUND THEN RAISE V_PAIS_EXISTENT;
  END;
--
-- Tots els controls superats.
--
  INSERT INTO CLIENTS (NIFCLIENT, NOMCLIENT, ADRECA, IDPAIS, IDPROVINCIA, IDPOBLACIO, CONSUMTOTAL, COMPTADORS,CONSUMMIG, ACTIU, DATAALTA)
         VALUES (inNif, inNom, inAdreca, v_Pais, v_Provincia, inPoblacio, 0, 0, 0, 1, SYSDATE);

  RSP := 'OK';
  INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
         VALUES (SYSDATE,'PKG_CLIENTS.PR_ALTA_CLIENTS', v_in, 'RSP: '||RSP);
  COMMIT;
  EXCEPTION
    WHEN V_CLIENT_EXISTENT THEN
         RSP := 'ERROR: El Client ja esta donat d`alta';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
                VALUES (SYSDATE,'PKG_CLIENTS.PR_ALTA_CLIENTS', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_POBLACIO_EXISTENT THEN
         RSP := 'ERROR: Poblacio Inexistent';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
                VALUES (SYSDATE,'PKG_CLIENTS.PR_ALTA_CLIENTS', v_in, 'RSP: '||RSP);
         COMMIT;	  
    WHEN V_PROVINCIA_EXISTENT THEN
         RSP := 'ERROR: Provincia Inexistent';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
                VALUES (SYSDATE,'PKG_CLIENTS.PR_ALTA_CLIENTS', v_in, 'RSP: '||RSP);
          COMMIT;	  
    WHEN V_PAIS_EXISTENT THEN
         RSP := 'ERROR: Pais Inexistent';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
                VALUES (SYSDATE,'PKG_CLIENTS.PR_ALTA_CLIENTS', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_CAMP_NUL THEN
         RSP := 'ERROR: No pot haver camps NULL';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
                VALUES (SYSDATE,'PKG_CLIENTS.PR_ALTA_CLIENTS', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN OTHERS THEN
         err_num := SQLCODE;
         err_msg := SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
                VALUES (SYSDATE,'PKG_CLIENTS.PR_ALTA_CLIENTS', v_in, 'RSP: '||RSP);
         COMMIT;
  END;

PROCEDURE PR_BAIXA_CLIENT (inClient        IN CLIENTS.IDCLIENT%TYPE,
                           RSP              OUT VARCHAR2)
  IS

   V_CAMP_NUL             EXCEPTION;
   V_CLIENT_EXISTENT      EXCEPTION;
   V_CLIENT_BAIXA         EXCEPTION;
   V_CONTRACTES_EXISTENT  EXCEPTION;

   v_in           VARCHAR2(500);
   v_actiu        NUMBER;
   hiTeContractes VARCHAR2(1);
   err_num        NUMBER;
   err_msg        VARCHAR2(100);

   BEGIN
     v_in := 'Client:'||inClient;
--
-- Primera comprovacio que tots els camps obligatoris tinguin valor
--
     IF (TRIM(inClient)     IS NULL) THEN
         RAISE V_CAMP_NUL;
     END IF;
--
-- Segona comprovacio que el idClient existeix
--
     BEGIN
       SELECT Actiu
         INTO V_actiu
         FROM CLIENTS
        WHERE IDCLIENT  = inClient
          AND ROWNUM  = 1;
    EXCEPTION
         WHEN NO_DATA_FOUND THEN RAISE V_CLIENT_EXISTENT;
    END;	 
--
-- Tercera comprovacio que no estigui de baixa
--	 
    BEGIN	 
   	  IF (V_ACTIU = 0) THEN
         RAISE V_CLIENT_BAIXA;
      END IF;     
   END;
--
-- Quarta comprovacio que el Client no tingui cap contracte
--
   BEGIN
   	 SELECT 'Y'
       INTO hiTeContractes
       FROM COMPTADORS
      WHERE idClient = inClient
        AND Actiu = 1
        AND ROWNUM = 1;
		      
     RAISE V_CONTRACTES_EXISTENT;		
  EXCEPTION
       WHEN NO_DATA_FOUND THEN NULL; 
  END;
--
-- Tots els controls superats.
--
     UPDATE CLIENTS
        SET ACTIU = 0,
            DATABAIXA = SYSDATE
      WHERE IDCLIENT = INCLIENT;
      RSP := 'OK';
      INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_CLIENTS.PR_BAIXA_CLIENTS', v_in, 'RSP: '||RSP);
      COMMIT;
  EXCEPTION
    WHEN V_CLIENT_EXISTENT THEN
         RSP := 'ERROR: El Client no esta donat d`alta';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
              VALUES (SYSDATE,'PKG_CLIENTS.PR_BAIXA_CLIENTS', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_CLIENT_BAIXA THEN
         RSP := 'ERROR: El Client ja esta esborrat';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
              VALUES (SYSDATE,'PKG_CLIENTS.PR_BAIXA_CLIENTS', v_in, 'RSP: '||RSP);
         COMMIT;	  
    WHEN V_CONTRACTES_EXISTENT THEN
         RSP := 'ERROR: El Client te contractes actius';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
              VALUES (SYSDATE,'PKG_CLIENTS.PR_BAIXA_CLIENTS', v_in, 'RSP: '||RSP);
         COMMIT;	  
    WHEN V_CAMP_NUL THEN
         RSP := 'ERROR: No pot haver camps NULL';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
              VALUES (SYSDATE,'PKG_CLIENTS.PR_BAIXA_CLIENTS', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN OTHERS THEN
         err_num := SQLCODE;
         err_msg := SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
              VALUES (SYSDATE,'PKG_CLIENTS.PR_BAIXA_CLIENTS', v_in, 'RSP: '||RSP);
         COMMIT;
  END;

  PROCEDURE PR_MODIFICACIO_CLIENT (inClient         IN CLIENTS.IDCLIENT%TYPE,
                                   inNif            IN CLIENTS.NIFCLIENT%TYPE,
                                   inNom            IN CLIENTS.NOMCLIENT%TYPE,
                                   inAdreca         IN CLIENTS.ADRECA%TYPE,
                                   inPoblacio       IN CLIENTS.IDPOBLACIO%TYPE,
                                   inConsum         IN CLIENTS.CONSUMMIG%TYPE,
                                   inDataAlta       IN CLIENTS.DATAALTA%TYPE,
                                   inDataBaixa      IN CLIENTS.DATABAIXA%TYPE,
                                   RSP              OUT VARCHAR2) 
   IS

   V_CAMP_NUL             EXCEPTION;
   V_CLIENT_EXISTENT      EXCEPTION;
   V_CLIENT_REPETIT       EXCEPTION;
   V_POBLACIO_EXISTENT    EXCEPTION;
   V_PROVINCIA_EXISTENT   EXCEPTION;
   V_PAIS_EXISTENT        EXCEPTION;

   v_in          VARCHAR2(500);
   hihies        VARCHAR2(1);
   v_Provincia   NUMBER;
   v_Pais        NUMBER;
   err_num       NUMBER;
   err_msg       VARCHAR2(100);


   BEGIN
     v_in := 'Client:'||inClient||', Nif:'||inNif||', Nom:'||inNom||', Adreca:'||inAdreca||', Poblacio:'||inPoblacio||
	         ', Consum:'||inConsum||', Data Alta:'||inDataAlta||', Data Baixa:'||inDataBaixa;
--
-- Primera comprovacio que tots els camps obligatoris tinguin valor
--
     IF (TRIM(inClient)     IS NULL) THEN
         RAISE V_CAMP_NUL;
     END IF;
--
-- Segona comprovacio que el idClient existeix
--
     BEGIN
       SELECT idPais,idProvincia
         INTO v_Pais,v_Provincia
         FROM CLIENTS
        WHERE IDCLIENT  = inClient
          AND ROWNUM  = 1;
        RAISE V_CLIENT_EXISTENT;
    EXCEPTION
         WHEN NO_DATA_FOUND THEN NULL;
    END;	 	 
--
-- Tercera comprovacio que el NIf no es trovi amb un altre id
--
     BEGIN
       SELECT 'Y'
         INTO hihies
         FROM CLIENTS
        WHERE NIFCLIENT  = inNif
          AND IDCLIENT <> inClient
          AND ROWNUM  = 1;
        RAISE V_CLIENT_REPETIT;
    EXCEPTION
         WHEN NO_DATA_FOUND THEN NULL;
    END;
--
-- Quarta comprovacio que el IdPoblacio estigui donat d'alta si s'ha canviat
--
    IF (TRIM(inPoblacio)     IS NOT NULL) THEN
       BEGIN
           SELECT idProvincia
             INTO v_Provincia
             FROM Poblacio
            WHERE idPoblacio = inPoblacio
              AND ROWNUM = 1;      
       EXCEPTION
            WHEN NO_DATA_FOUND THEN RAISE V_POBLACIO_EXISTENT;
       END;
       --
       BEGIN	
          SELECT idPais
            INTO v_Pais
            FROM Provincia
           WHERE idProvincia = v_Provincia
             AND ROWNUM = 1;      
       EXCEPTION
            WHEN NO_DATA_FOUND THEN RAISE V_PROVINCIA_EXISTENT;		  
       END;
       --
       BEGIN
          SELECT 'Y'
            INTO hihies
            FROM Pais
           WHERE idPais = v_Pais
             AND ROWNUM = 1;      
       EXCEPTION
            WHEN NO_DATA_FOUND THEN RAISE V_PAIS_EXISTENT;
       END;	   	   
    END IF;	
--
-- Tots els controls superats.
--
      UPDATE Clients SET
   	         NifClient = NVL(inNif, NifClient),
		     NomClient = NVL(inNom, NomClient),
    	     Adreca = NVL(inAdreca, Adreca),
		     idPais = NVL(v_Pais, idPais),
		     idProvincia = NVL(v_Provincia, idProvincia), 
		     idPoblacio =  NVL(idPoblacio, idPoblacio),
		     ConsumMig = NVL(inConsum, ConsumMig),
		     DataAlta = NVL(inDataAlta, DataAlta),
     		 DataBaixa = NVL(inDataBaixa, DataBaixa)
	         WHERE idClient = inClient;
      RSP := 'OK';
      INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_CLIENTS.PR_MODIFICACIO_CLIENTS', v_in, 'RSP: '||RSP);
      COMMIT;
  EXCEPTION
    WHEN V_CLIENT_REPETIT THEN
         RSP := 'ERROR: El Dni ya esta assignat a un altre client';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
            VALUES (SYSDATE,'PKG_CLIENTS.PR_MODIFICACIO_CLIENTS', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_CLIENT_EXISTENT THEN
         RSP := 'ERROR: El Client no esta donat d`alta';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_CLIENTS.PR_MODIFICACIO_CLIENTS', v_in, 'RSP: '||RSP);
         COMMIT;	  
    WHEN V_POBLACIO_EXISTENT THEN
         RSP := 'ERROR: Poblacio Inexistent';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_CLIENTS.PR_MODIFICACIO_CLIENTS', v_in, 'RSP: '||RSP);
         COMMIT;	  
    WHEN V_PROVINCIA_EXISTENT THEN
         RSP := 'ERROR: Provincia Inexistent';  
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_CLIENTS.PR_MODIFICACIO_CLIENTS', v_in, 'RSP: '||RSP);
         COMMIT;	  
    WHEN V_PAIS_EXISTENT THEN
         RSP := 'ERROR: Pais Inexistent';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_CLIENTS.PR_MODIFICACIO_CLIENTS', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_CAMP_NUL THEN
         RSP := 'ERROR: No pot haver camps NULL';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_CLIENTS.PR_MODIFICACIO_CLIENTS', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN OTHERS THEN
         err_num := SQLCODE;
         err_msg := SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_CLIENTS.PR_MODIFICACIO_CLIENTS', v_in, 'RSP: '||RSP);
         COMMIT;
  END;
END PKG_CLIENTS;
--
-- FI DEL PACKAGE
--
spool off;
