spool 8-package_comptadors.log

CREATE OR REPLACE PACKAGE ELE.PKG_COMPTADORS AS

--
-- NOM.........:   PKG_COMPTADORS
-- FINALITAT...:   GESTIO DE TOTA LA INFORMACIO REFERENTS A COMPTADORS (CONTRACTES)
--
--
--
--
-- NOM.........:   PR_ALTA_COMPTADORS
-- FINALITAT...:   DONAR D'ALTA UN COMPTADOR
-- CONTROLS....:   INFORMAR CAMPS OBLIGATORIS
--                 CONTRACTE NO REPETIT
--                 CODI DEL CLIENT EXISTENT
--                 MODEL EXISTENT
--              
-- PARAMETRES..:   
--         IN..:   idContracte..: Codi del contracte (Obligatori)
--	             idClient.....: Codi del client (Obliagtoir)
--                 adreca.......: adrea 
--                 idPoblacio...: codi de la poblacio (Obligatori), a partir d'aquest codi obtindrem idProvincia i idPais.
--                 potencia.....: Potencia contractada
--                 idModel......: Codi del model del contracte
--                 AnyFabricacio: Any fabricacio del comptador
--        OUT..:   RSP
--
-- PRECONDICIO.:   COMPTADOR NO EXISTEIX
-- POSTCONDICIO:   ES DONA D'ALTA EL COMPTADOR
-- RETURN .....:   OK SI S'HA EFECTUAT L'ALTA
--                 ERROR ALGUN CAMP DEL PARAMETRE NO TE VALOR
--                 ERROR REGISTRE DUPLICAT (CLIENT EXISTENT)
--                 ERROR PROVINCIA NO EXISTENT
--                 ERROR SI EL CLIENT NO EXISTENT
--                 ERROR SI EL CLIENT ESTA ESBORRAT
--                 ERORR MODEL NO EXISTENT
--
--

  PROCEDURE PR_ALTA_COMPTADORS (inContracte      IN COMPTADORS.IDCONTRACTE%TYPE,
                                inClient         IN COMPTADORS.IDCLIENT%TYPE,
                                inAdreca         IN COMPTADORS.ADRECA%TYPE,
                                inPoblacio       IN COMPTADORS.IDPOBLACIO%TYPE,
                                inPotencia       IN COMPTADORS.POTENCIACONTRACTE%TYPE,
                                inModel          IN COMPTADORS.IDMODEL%TYPE,
                                inAny            IN COMPTADORS.ANYFABRICACIO%TYPE,
                                RSP              OUT VARCHAR2);

--
-- NOM.........:   PR_BAIXA_COMPTADORS
-- FINALITAT...:   DONAR DE BAIXA UN CONTADORS
-- CONTROLS....:   COMPTADOR NO DONAT DE BAIXA
--                 COMPTADOR EXISTEIX EN EL FITXER
--                 CLIENT SENSE CONTRACTES
--              
-- PARAMETRES..:   
--         IN..:   idComptador: Codi del comptador (Obligatori)
--        OUT..:   RSP
--
-- PRECONDICIO.:   COMPTADOR EXISTEIX
-- POSTCONDICIO:   ES DONA DE BAIXA EL COMPTADOR
-- RETURN .....:   OK SI S'HA EFECTUAT LA BAIXA
--                 ERROR SI EL COMPTADOR NO EXISTEIX
--                 ERROR SI EL COMPTADOR ESTA DONAT DE BAIXA              
--
--

  PROCEDURE PR_BAIXA_COMPTADORS (inComptador      IN COMPTADORS.IDCOMPTADOR%TYPE,
                                 RSP              OUT VARCHAR2);

--
-- NOM.........:   PR_MODIFICACIO_COMPTADORS
-- FINALITAT...:   PODER MODIFICAR QUALSEVOL CAMP D'UN COMPTADOR
-- CONTROLS....:   COMPTADOR EXISTENT EN EL FITXER
--                 INFORMAR CAMPS OBLIGATORIS
--                 CONTRACTE NO REPETIT
--                 CODI DEL CLIENT EXISTENT
--                 MODEL EXISTENT
--              
-- PARAMETRES..:   
--         IN..:   idcomptador..: codi del comptador (obligatori)
--                 idcontrate...: codi del contracte 
--	               idclient.....: codi del client
--                 adreca.......: adrea del comptador
--                 idPoblacio...: codi de la poblacio (Obligatori), a partir d'aquest codi obtindrem idProvincia i idPais.
--                 Potencia.....: Potencia contractada
--                 consum.......: Consum del comptador
--                 datalectura..: data de la ultima lectura
--                 idmodel......: codi del model del comptador
--                 anyfabricacio: any que es va fabricar el comptador
--                 DataAlta.....: Canviar la data d'alta del client
--                 DataBaixa....: Canviar la data de baixa del client
--        OUT..:   RSP
--
-- PRECONDICIO.:   COMPTADOR EXISTEIX
-- POSTCONDICIO:   ES REALITZA LA MODIFICACIO DELS CAMPS DEL COMPTADOR
-- RETURN .....:   OK SI S'HA EFECTUAT LA MODIFICACIO
--                 ERROR ALGUN CAMP DEL PARAMETRE NO TE VALOR
--                 ERROR REGISTRE DUPLICAT (COMPTADORT EXISTENT)
--                 ERROR PROVINCIA NO EXISTENT
--                 ERROR SI EL CLIENT NO EXISTENT
--                 ERORR MODEL NO EXISTENT
--

  PROCEDURE PR_MODIFICACIO_COMPTADORS (inComptador      IN COMPTADORS.IDCOMPTADOR%TYPE,
                                       inContracte      IN COMPTADORS.IDCONTRACTE%TYPE,
                                       inClient         IN COMPTADORS.IDCLIENT%TYPE,
                                       inAdreca         IN COMPTADORS.ADRECA%TYPE,
                                       inPoblacio       IN COMPTADORS.IDPOBLACIO%TYPE,
                                       inPotencia       IN COMPTADORS.POTENCIACONTRACTE%TYPE,
                                       inConsum         IN COMPTADORS.CONSUMACTUAL%TYPE,
                                       inDataLectura    IN COMPTADORS.DATALECTURA%TYPE,
                                       inModel          IN COMPTADORS.IDMODEL%TYPE,
                                       inAny            IN COMPTADORS.ANYFABRICACIO%TYPE,
                                       inDataAlta       IN COMPTADORS.DATAALTA%TYPE,
                                       inDataBaixa      IN COMPTADORS.DATABAIXA%TYPE,
                                       RSP              OUT VARCHAR2);

END PKG_COMPTADORS;

--
--
--
CREATE OR REPLACE PACKAGE BODY ELE.PKG_COMPTADORS AS

  PROCEDURE PR_ALTA_COMPTADORS (inContracte      IN COMPTADORS.IDCONTRACTE%TYPE,
                                inClient         IN COMPTADORS.IDCLIENT%TYPE,
                                inAdreca         IN COMPTADORS.ADRECA%TYPE,
                                inPoblacio       IN COMPTADORS.IDPOBLACIO%TYPE,
                                inPotencia       IN COMPTADORS.POTENCIACONTRACTE%TYPE,
                                inModel          IN COMPTADORS.IDMODEL%TYPE,
                                inAny            IN COMPTADORS.ANYFABRICACIO%TYPE,
                                RSP              OUT VARCHAR2)
  IS

   V_CAMP_NUL             EXCEPTION;
   V_CONTRACTE_EXISTENT   EXCEPTION;
   V_CLIENT_EXISTENT      EXCEPTION;
   V_CLIENT_ACTIU         EXCEPTION;
   V_POBLACIO_EXISTENT    EXCEPTION;
   V_PROVINCIA_EXISTENT   EXCEPTION;
   V_PAIS_EXISTENT        EXCEPTION;
   V_MODEL_EXISTENT       EXCEPTION;

   v_in          VARCHAR2(500);
   hihies        VARCHAR2(1);
   ActiuClient   NUMBER;
   v_Provincia   NUMBER;
   v_Pais        NUMBER;
   err_num       NUMBER;
   err_msg       VARCHAR2(100);

   BEGIN
     v_in := 'Contracte:'||inContracte||', Client:'||inClient||', Adreca:'||inAdreca||', Poblacio:'||inPoblacio||
             ', Potencia:'||inPotencia||', Model:'||inModel||', Any Fabricacio:'||inAny;
--
-- Primera comprovacio que tots els camps obligatoris tinguin valor
--
     IF (TRIM(inContracte)     IS NULL) THEN
         RAISE V_CAMP_NUL;
     END IF;
     IF (TRIM(inClient)     IS NULL) THEN
         RAISE V_CAMP_NUL;
     END IF;
     IF (TRIM(inPoblacio)     IS NULL) THEN
         RAISE V_CAMP_NUL;
     END IF;
     IF (TRIM(inModel)     IS NULL) THEN
         RAISE V_CAMP_NUL;
     END IF;
--
-- Segona comprovacio que el Contracte no es trovi amb un altre id
--
     BEGIN
       SELECT 'Y'
         INTO hihies
         FROM COMPTADORS
        WHERE IDCONTRACTE  = inContracte
          AND ROWNUM  = 1;
        RAISE V_CONTRACTE_EXISTENT;
    EXCEPTION
         WHEN NO_DATA_FOUND THEN NULL;
    END;
--
-- Tercera comprovacio que el client estigui donat d'alta
--
     BEGIN
       SELECT Actiu
         INTO ActiuClient
         FROM CLIENTS
        WHERE IDCLIENT  = inClient
          AND ROWNUM  = 1;
    EXCEPTION
         WHEN NO_DATA_FOUND THEN RAISE V_CLIENT_EXISTENT;
    END;
--
-- Quarta comprovacio que el client estigui actiu
--
   BEGIN
      IF (ActiuClient = 0) THEN 
         RAISE V_CLIENT_ACTIU;
      END IF;
   END;
--
-- Cinquena comprovacio que el IdPoblacio estigui donat d'alta
--
    BEGIN
	  SELECT idProvincia
        INTO v_Provincia
        FROM Poblacio
       WHERE idPoblacio = inPoblacio
         AND ROWNUM = 1;      
   EXCEPTION
        WHEN NO_DATA_FOUND THEN RAISE V_POBLACIO_EXISTENT;
   END;
--
-- Miro el Pais que te la privincia
--	
	BEGIN	
	  SELECT idPais
        INTO v_Pais
        FROM Provincia
       WHERE idProvincia = v_Provincia
         AND ROWNUM = 1;      
   EXCEPTION
        WHEN NO_DATA_FOUND THEN RAISE V_PROVINCIA_EXISTENT;	  
   END;
--
-- Miro que el Pais estigui donat d'alta
--	
	BEGIN
	  SELECT 'Y'
        INTO hihies
        FROM Pais
       WHERE idPais = v_Pais
         AND ROWNUM = 1;      
   EXCEPTION
        WHEN NO_DATA_FOUND THEN RAISE V_PAIS_EXISTENT;
   END;
--
-- Sisena que el model estigui donat d'alta
--
      BEGIN
        SELECT 'Y'
          INTO hihies
          FROM MODEL
         WHERE IDMODEL  = inModel
           AND ROWNUM  = 1;
     EXCEPTION
          WHEN NO_DATA_FOUND THEN RAISE V_MODEL_EXISTENT;
     END;
-- Tots els controls superats.
-- Coloquem la data 01/01/2000 per poder saltar el control de data lectura i poderinserir registres, lo normal fos colocar SYSDATE.
      INSERT INTO COMPTADORS (IDCONTRACTE, IDCLIENT, ADRECA, IDPAIS, IDPROVINCIA, IDPOBLACIO, POTENCIACONTRACTE, CONSUMACTUAL, DATALECTURA, IDMODEL, ANYFABRICACIO, ACTIU, DATAALTA)
           VALUES (inContracte, inClient, inAdreca, v_Pais, v_Provincia, inPoblacio, inPotencia, 0, '01/01/2000', inModel, inAny, 1, SYSDATE);
      RSP := 'OK';
      INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_COMPTADORS.PR_ALTA_COMPTADORS', v_in, 'RSP: '||RSP);
      COMMIT;
  EXCEPTION
    WHEN V_CONTRACTE_EXISTENT THEN
         RSP := 'ERROR: El Contracte ya esta donat d`alta';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
              VALUES (SYSDATE,'PKG_COMPTADORS.PR_ALTA_COMPTADORS', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_CLIENT_EXISTENT THEN
         RSP := 'ERROR: El Client no esta donat d`alta';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
              VALUES (SYSDATE,'PKG_COMPTADORS.PR_ALTA_COMPTADORS', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_CLIENT_ACTIU THEN
         RSP := 'ERROR: El Client esta donat de baixa';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
              VALUES (SYSDATE,'PKG_COMPTADORS.PR_ALTA_COMPTADORS', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_MODEL_EXISTENT THEN
         RSP := 'ERROR: El Model no esta donat d`alta';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
              VALUES (SYSDATE,'PKG_COMPTADORS.PR_ALTA_COMPTADORS', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_POBLACIO_EXISTENT THEN
         RSP := 'ERROR: Poblacio Inexistent';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
              VALUES (SYSDATE,'PKG_COMPTADORS.PR_ALTA_COMPTADORS', v_in, 'RSP: '||RSP);
         COMMIT;	  
    WHEN V_PROVINCIA_EXISTENT THEN
         RSP := 'ERROR: Provincia Inexistent';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
              VALUES (SYSDATE,'PKG_COMPTADORS.PR_ALTA_COMPTADORS', v_in, 'RSP: '||RSP);
         COMMIT;	  
    WHEN V_PAIS_EXISTENT THEN
         RSP := 'ERROR: Pais Inexistent';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
              VALUES (SYSDATE,'PKG_COMPTADORS.PR_ALTA_COMPTADORS', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_CAMP_NUL THEN
         RSP := 'ERROR: No pot haver camps NULL';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
              VALUES (SYSDATE,'PKG_COMPTADORS.PR_ALTA_COMPTADORS', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN OTHERS THEN
         err_num := SQLCODE;
         err_msg := SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
              VALUES (SYSDATE,'PKG_COMPTADORS.PR_ALTA_COMPTADORS', v_in, 'RSP: '||RSP);
         COMMIT;
  END;

  PROCEDURE PR_BAIXA_COMPTADORS (inComptador      IN COMPTADORS.IDCOMPTADOR%TYPE,
                                 RSP              OUT VARCHAR2)
  IS

   V_CAMP_NUL             EXCEPTION;
   V_COMPTADOR_EXISTENT   EXCEPTION;
   V_COMPTADOR_BAIXA      EXCEPTION;

   v_in           VARCHAR2(500);
   v_actiu        NUMBER;
   v_consum       NUMBER;
   v_comptadors   NUMBER;
   v_total        NUMBER;
   v_vell         NUMBER;
   v_mig          NUMBER;
   v_contadors    NUMBER;
   OutId          NUMBER;
   err_num        NUMBER;
   err_msg        VARCHAR2(100);

   -- Per saber les linies de comunicacio d'aquest comptador.
   cursor LiniasComptador is (SELECT DISTINCT(IDLINIACOMUNICACIO) 
   	                            FROM COMUNICACIOPRODUCCIO
	        	               WHERE ACTIU = 1
					             AND IDDISTRIBUCIO IN (SELECT DISTINCT(IDDISTRIBUCIO) 
					                                     FROM COMUNICACIODISTRIBUCIO
											            WHERE IDCOMPTADOR = inComptador
        										          AND ACTIU = 1));	   

   BEGIN
     v_in := 'Comptador:'||inComptador;
--
-- Primera comprovacio que tots els camps obligatoris tinguin valor
--
     IF (TRIM(inComptador)     IS NULL) THEN
         RAISE V_CAMP_NUL;
     END IF;
--
-- Segona comprovacio que el idComptador existeix
--
     BEGIN
       SELECT Actiu,ConsumActual,idClient
         INTO v_actiu,v_consum,v_vell
         FROM COMPTADORS
        WHERE IDCOMPTADOR  = inComptador
          AND ROWNUM  = 1;        
     EXCEPTION
          WHEN NO_DATA_FOUND THEN RAISE V_COMPTADOR_EXISTENT;
     END;	 
--
-- Tercera comprovacio que no estigui de baixa
--	 
     BEGIN 
        IF (V_ACTIU = 0) THEN
          RAISE V_COMPTADOR_BAIXA;
        END IF;     
     END;
--	 
-- Actualitzo estadistiques SI TENIM CONSUM,
--
   IF (V_CONSUM > 0) THEN
      -- Resto el consum de la central de produccio.
      UPDATE PRODUCCIO
         SET CONSUMSCOMPTADOR = CONSUMSCOMPTADOR - V_CONSUM
       WHERE ACTIU = 1
         AND IDPRODUCCIO IN (SELECT DISTINCT(IDPRODUCCIO) FROM COMUNICACIOPRODUCCIO WHERE ACTIU = 1 AND IDDISTRIBUCIO 
	                     IN (SELECT DISTINCT(IDDISTRIBUCIO) FROM COMUNICACIODISTRIBUCIO WHERE IDCOMPTADOR = inComptador AND ACTIU = 1));
      -- Resto el consum de l'historic total.						 
      BEGIN
         SELECT Consum
           INTO v_total
           FROM HISTORICTOTAL
          WHERE ROWNUM = 1;	  
    	 -- HIHES L'ANY UPDATE.
	     UPDATE HISTORICTOTAL 
    	    SET CONSUM = CONSUM - v_Consum;		
    	 v_total := v_total - v_Consum;
    	 RSP := 'OK UPDATE HISTORIC TOTAL';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
              VALUES (SYSDATE,'PKG_HISTORICLECTURES.PR_ALTA_HISTORICLECTURES', v_in, 'RSP: '||RSP);	  
      EXCEPTION
        -- ALTA.
        WHEN NO_DATA_FOUND THEN 	 
     	 INSERT INTO HISTORICTOTAL (CONSUM, PRODUCCIO) VALUES (v_Consum, 0);	 
    	 v_total := v_consum;	 
     	 RSP := 'OK INSERT HISTORIC TOTAL'; 
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
              VALUES (SYSDATE,'PKG_HISTORICLECTURES.PR_ALTA_HISTORICLECTURES', v_in, 'RSP: '||RSP);		   
      END;
      -- 
      OPEN LiniasComptador;
      FETCH LiniasComptador INTO outId;
      WHILE LiniasComptador%FOUND LOOP
            -- Faig l'update TEMES ESTADISTICS LINIACOMUNICACIO
            UPDATE LINIACOMUNICACIO
               SET CONSUM = CONSUM - V_CONSUM
             WHERE ACTIU = 1
               AND IDLINIACOMUNICACIO = outId;
            -- Contem Comptadors
	     	SELECT COUNT(DISTINCT(IDCOMPTADOR))
		      INTO v_comptadors
              FROM COMPTADORS
             WHERE ACTIU = 1
               AND IDCOMPTADOR IN (SELECT IDCOMPTADOR
                                     FROM COMUNICACIODISTRIBUCIO
				    	            WHERE ACTIU = 1
					                  AND IDDISTRIBUCIO IN (SELECT IDDISTRIBUCIO
						                                      FROM COMUNICACIOPRODUCCIO
								        	         	     WHERE ACTIU = 1
													           AND IDLINIACOMUNICACIO = outId));  
   		    -- Actualitzo 
    		v_comptadors := v_comptadors - 1;
	    	IF (V_COMPTADORS = 0) THEN 
			   v_comptadors := 1;
			END IF;
            UPDATE HISTORICANY
               SET CONSUM = CONSUM - V_CONSUM,
                   CONSUMAVG = round((CONSUM-V_CONSUM) / v_comptadors,2)
             WHERE BD = 0
               AND IDANY = TO_CHAR(SysDate,'YYYY')
               AND IDCLAU = outId;
      	   RSP := 'OK UPDATE HISTORIC ANUAL '||TO_CHAR(SysDate,'YYYY'); 
           INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
                VALUES (SYSDATE,'PKG_COMPTADORS.PR_BAIXA_COMPTADORS', v_in, 'RSP: '||RSP);
           FETCH LiniasComptador INTO outId;	  
     END LOOP;
     CLOSE LiniasComptador;
	-- Actualitzo 
    v_total := v_total * 0.50;
    UPDATE HISTORICANY
       SET CONSUMTOTAL = v_total
     WHERE BD = 0;		  
   	 RSP := 'OK UPDATE HISTORIC ANUAL '||TO_CHAR(SysDate,'YYYY'); 
     INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_COMPTADORS.PR_BAIXA_COMPTADORS', v_in, 'RSP: '||RSP);
   END IF;
--
-- CONSUM MIG DEL CLIENT DADES ESTADISTIQUES.
--
   IF (V_CONSUM > 0) THEN
	  -- AGAFO ELS CONSUM DEL CLIENT VELL.
	  SELECT COMPTADORS,CONSUMTOTAL
	    INTO V_COMPTADORS,V_TOTAL
	    FROM CLIENTS
	   WHERE IDCLIENT = V_VELL;
 	  V_COMPTADORS := V_COMPTADORS - 1;
	  V_MIG := 0;
	  IF (V_COMPTADORS > 0) THEN
	     V_MIG := (V_TOTAL-V_CONSUM) / V_COMPTADORS; 
	  END IF;
	  -- FAIG L'UPDATE
	  UPDATE CLIENTS
	     SET COMPTADORS = V_COMPTADORS,
		     CONSUMTOTAL = V_TOTAL - V_CONSUM,
			 CONSUMMIG = V_MIG
       WHERE IDCLIENT = V_VELL;
   END IF;
 	  
--
-- Tots els controls superats.
--
        UPDATE COMPTADORS
           SET ACTIU = 0,
               DATABAIXA = SYSDATE
       WHERE IDCOMPTADOR = INCOMPTADOR;
       RSP := 'OK';
       INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_COMPTADORS.PR_BAIXA_COMPTADORS', v_in, 'RSP: '||RSP);
       COMMIT;
  EXCEPTION
    WHEN V_COMPTADOR_EXISTENT THEN
         RSP := 'ERROR: El Comptador no esta donat d`alta';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_COMPTADORS.PR_BAIXA_COMPTADORS', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_COMPTADOR_BAIXA THEN
         RSP := 'ERROR: El Comptador ja esta esborrat';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_COMPTADORS.PR_BAIXA_COMPTADORS', v_in, 'RSP: '||RSP);
         COMMIT;	  
    WHEN V_CAMP_NUL THEN
         RSP := 'ERROR: No pot haver camps NULL';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_COMPTADORS.PR_BAIXA_COMPTADORS', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN OTHERS THEN
         err_num := SQLCODE;
         err_msg := SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_COMPTADORS.PR_BAIXA_COMPTADORS', v_in, 'RSP: '||RSP);
         COMMIT;
  END;

  PROCEDURE PR_MODIFICACIO_COMPTADORS (inComptador      IN COMPTADORS.IDCOMPTADOR%TYPE,
                                       inContracte      IN COMPTADORS.IDCONTRACTE%TYPE,
                                       inClient         IN COMPTADORS.IDCLIENT%TYPE,
                                       inAdreca         IN COMPTADORS.ADRECA%TYPE,
                                       inPoblacio       IN COMPTADORS.IDPOBLACIO%TYPE,
                                       inPotencia       IN COMPTADORS.POTENCIACONTRACTE%TYPE,
                                       inConsum         IN COMPTADORS.CONSUMACTUAL%TYPE,
                                       inDataLectura    IN COMPTADORS.DATALECTURA%TYPE,
                                       inModel          IN COMPTADORS.IDMODEL%TYPE,
                                       inAny            IN COMPTADORS.ANYFABRICACIO%TYPE,
                                       inDataAlta       IN COMPTADORS.DATAALTA%TYPE,
                                       inDataBaixa      IN COMPTADORS.DATABAIXA%TYPE,
                                       RSP              OUT VARCHAR2)
   IS

   V_CAMP_NUL             EXCEPTION;
   V_COMPTADOR_EXISTENT   EXCEPTION;
   V_COMPTADOR_BAIXA      EXCEPTION;
   V_CONTRACTE_REPETIT    EXCEPTION;
   V_CLIENT_EXISTENT      EXCEPTION;
   V_CLIENT_ACTIU         EXCEPTION;
   V_POBLACIO_EXISTENT    EXCEPTION;
   V_PROVINCIA_EXISTENT   EXCEPTION;
   V_PAIS_EXISTENT        EXCEPTION;
   V_MODEL_EXISTENT       EXCEPTION;

   v_in          VARCHAR2(500);
   hihies        VARCHAR2(1);
   ActiuClient   NUMBER;
   v_Provincia   NUMBER;
   v_Pais        NUMBER;
   v_actiu       NUMBER;
   v_vell        NUMBER;
   v_consum      NUMBER;
   v_comptadors  NUMBER;
   v_total       NUMBER;
   v_mig         NUMBER;
   v_actual      NUMBER;
   err_num       NUMBER;
   err_msg       VARCHAR2(100);

   BEGIN
     v_in := 'Comptador:'||inComptador||', Contracte:'||inContracte||', Client:'||inClient||', Adreca:'||inAdreca||', Provincia:'||inPoblacio||
	     ', Potencia:'||inPotencia||', Consum:'||inConsum||', Data Lectura:'||inDataLectura||', Model'||inModel||
         ', Any Fabricacio:'||inAny||', Data Alta:'||inDataAlta||', Data Baixa:'||inDataBaixa;
--
-- Primera comprovacio que tots els camps obligatoris tinguin valor
--
     IF (TRIM(inComptador)     IS NULL) THEN
         RAISE V_CAMP_NUL;
     END IF;
--
-- Segona comprovacio que el idComptador existeix
--
     BEGIN
       SELECT idPais,idProvincia,ConsumActual,idClient,Actiu
         INTO v_Pais,v_Provincia,v_actual,v_vell,v_actiu
         FROM COMPTADORS
        WHERE IDCOMPTADOR  = inComptador
          AND ROWNUM  = 1;
    EXCEPTION
         WHEN NO_DATA_FOUND THEN RAISE V_COMPTADOR_EXISTENT;
     END;	 	 
--
-- Tercera comprovacio que el Contracte no es trovi amb un altre id
--
     BEGIN
       SELECT 'Y'
         INTO hihies
         FROM COMPTADORS
        WHERE IDCONTRACTE  = inContracte
          AND IDCOMPTADOR <> inComptador
          AND ROWNUM  = 1;
        RAISE V_CONTRACTE_REPETIT;
    EXCEPTION
         WHEN NO_DATA_FOUND THEN NULL;
    END;
--
-- Tercera comprovacio que no estigui de baixa
--	 
     BEGIN 
        IF (V_ACTIU = 0) THEN
          RAISE V_COMPTADOR_BAIXA;
        END IF;     
     END;	
--
-- Quarta comprovacio que el IdPoblacio estigui donat d'alta si s'ha canviat
--
    IF (TRIM(inPoblacio)     IS NOT NULL) THEN
       BEGIN
	     SELECT idProvincia
           INTO v_Provincia
           FROM Poblacio
          WHERE idPoblacio = inPoblacio
            AND ROWNUM = 1;      
      EXCEPTION
           WHEN NO_DATA_FOUND THEN RAISE V_POBLACIO_EXISTENT;
	   END;
	   -- COMPROVACIO PROVINCIA.
	   BEGIN	
          SELECT idPais
            INTO v_Pais
            FROM Provincia
           WHERE idProvincia = v_Provincia
             AND ROWNUM = 1;      
       EXCEPTION
            WHEN NO_DATA_FOUND THEN RAISE V_PROVINCIA_EXISTENT;		  
       END;
	 -- COMPROVACIO PAIS.
       BEGIN
          SELECT 'Y'
            INTO hihies
            FROM Pais
           WHERE idPais = v_Pais
             AND ROWNUM = 1;      
       EXCEPTION
            WHEN NO_DATA_FOUND THEN RAISE V_PAIS_EXISTENT;
       END;	   	   
    END IF;	
--
-- SI S'HA CANVIAT EL CLIENT QUE ESTIGI DONAT D'ALTA.
--
    IF (TRIM(inClient)     IS NOT NULL) THEN
       BEGIN
          SELECT Actiu
            INTO ActiuClient
            FROM CLIENTS
           WHERE IDCLIENT  = inClient
             AND ROWNUM  = 1;			 
       EXCEPTION
            WHEN NO_DATA_FOUND THEN RAISE V_CLIENT_EXISTENT;
       END;
       -- COMPROVAR QUE ESTIGUI ACTIU.
       BEGIN
          IF (ActiuClient = 0) THEN 
             RAISE V_CLIENT_ACTIU;
          END IF;
       END;
    END IF;
--
-- SI S'HA CANVIAT EL MODEL QUE ESTIGI DONAT D'ALTA.
--
    IF (TRIM(inModel)     IS NOT NULL) THEN
      BEGIN
       SELECT 'Y'
         INTO hihies
         FROM MODEL
        WHERE IDMODEL  = inModel
          AND ROWNUM  = 1;
    EXCEPTION
         WHEN NO_DATA_FOUND THEN RAISE V_MODEL_EXISTENT;
    END;
   END IF;
--
-- ESTADISTIQUES.
--      
   IF (TRIM(inClient)     IS NOT NULL) THEN
	  -- AGAFO ELS CONSUM DEL CLIENT VELL.
	  SELECT COMPTADORS,CONSUMTOTAL
	    INTO V_COMPTADORS,V_TOTAL
	    FROM CLIENTS
	   WHERE IDCLIENT = V_VELL;
 	  V_COMPTADORS := V_COMPTADORS - 1;
	  V_MIG := 0;
	  IF (V_COMPTADORS > 0) THEN
	     V_MIG := (V_TOTAL-V_ACTUAL) / V_COMPTADORS; 
	  END IF;
	  -- FAIG L'UPDATE
	  UPDATE CLIENTS
	     SET COMPTADORS = V_COMPTADORS,
		     CONSUMTOTAL = V_TOTAL - V_ACTUAL,
			 CONSUMMIG = V_MIG
       WHERE IDCLIENT = V_VELL;
	   -- CALCUL NOU CLIENT.
	   UPDATE CLIENTS
	      SET COMPTADORS = COMPTADORS + 1,
		      CONSUMTOTAL = CONSUMTOTAL + V_ACTUAL,
			  CONSUMMIG = (CONSUMTOTAL+V_ACTUAL) / (COMPTADORS+1)
		WHERE IDCLIENT = inClient;
   END IF;   
--
-- Tots els controls superats.
--
      UPDATE COMPTADORS SET
	       idContracte = NVL(inContracte, idContracte),
	       idClient = NVL(inClient, idClient),
  	       Adreca = NVL(inAdreca, Adreca),
	       idPais = NVL(v_Pais, idPais),
   		   idProvincia = NVL(v_Provincia, idProvincia), 
           idPoblacio =  NVL(idPoblacio, idPoblacio),
           PotenciaContracte = NVL(inPotencia, PotenciaContracte),
		   ConsumActual = NVL(inConsum, ConsumActual),
           DataLectura =  NVL(inDataLectura, DataLectura),
           idModel =  NVL(inModel, idModel),
           AnyFabricacio =  NVL(inAny, AnyFabricacio),
    	   DataAlta = NVL(inDataAlta, DataAlta),
	       DataBaixa = NVL(inDataBaixa, DataBaixa)
	    WHERE idComptador = inComptador;
      RSP := 'OK';
      INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_COMPTADORS.PR_MODIFICACIO_COMPTADORS', v_in, 'RSP: '||RSP);

      COMMIT;
  EXCEPTION
    WHEN V_CONTRACTE_REPETIT THEN
         RSP := 'ERROR: El Contracte ya esta assignat a un altre comptador';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
              VALUES (SYSDATE,'PKG_COMPTADORS.PR_MODIFICACIO_COMPTADORS', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_COMPTADOR_BAIXA THEN
         RSP := 'ERROR: El Comptador ja esta esborrat';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_COMPTADORS.PR_BAIXA_COMPTADORS', v_in, 'RSP: '||RSP);
         COMMIT;	  
    WHEN V_COMPTADOR_EXISTENT THEN
         RSP := 'ERROR: El comptador no esta donat d`alta';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
              VALUES (SYSDATE,'PKG_COMPTADORS.PR_MODIFICACIO_COMPTADORS', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_CLIENT_EXISTENT THEN
         RSP := 'ERROR: El Client no esta donat d`alta';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
              VALUES (SYSDATE,'PKG_COMPTADORS.PR_MODIFICACIO_COMPTADORS', v_in, 'RSP: '||RSP);
         COMMIT;	  
    WHEN V_CLIENT_ACTIU THEN
         RSP := 'ERROR: El Client esta donat de baixa';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
              VALUES (SYSDATE,'PKG_COMPTADORS.PR_MODIFICACIO_COMPTADORS', v_in, 'RSP: '||RSP);
         COMMIT;	  
    WHEN V_MODEL_EXISTENT THEN
         RSP := 'ERROR: El Model no esta donat d`alta';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
              VALUES (SYSDATE,'PKG_COMPTADORS.PR_MODIFICACIO_COMPTADORS', v_in, 'RSP: '||RSP);
         COMMIT;	  
    WHEN V_POBLACIO_EXISTENT THEN
         RSP := 'ERROR: Poblacio Inexistent';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
              VALUES (SYSDATE,'PKG_COMPTADORS.PR_MODIFICACIO_COMPTADORS', v_in, 'RSP: '||RSP);
         COMMIT;	  
    WHEN V_PROVINCIA_EXISTENT THEN
         RSP := 'ERROR: Provincia Inexistent';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
              VALUES (SYSDATE,'PKG_COMPTADORS.PR_MODIFICACIO_COMPTADORS', v_in, 'RSP: '||RSP);
         COMMIT;	  
    WHEN V_PAIS_EXISTENT THEN
         RSP := 'ERROR: Pais Inexistent';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
              VALUES (SYSDATE,'PKG_COMPTADORS.PR_MODIFICACIO_COMPTADORS', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_CAMP_NUL THEN
         RSP := 'ERROR: No pot haver camps NULL';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
              VALUES (SYSDATE,'PKG_COMPTADORS.PR_MODIFICACIO_COMPTADORS', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN OTHERS THEN
         err_num := SQLCODE;
         err_msg := SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg; 
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
              VALUES (SYSDATE,'PKG_COMPTADORS.PR_MODIFICACIO_COMPTADORS', v_in, 'RSP: '||RSP);
         COMMIT;
  END;
END PKG_COMPTADORS;
--
-- FI DEL PACKAGE
--
spool off;
