spool 9-package_liniacomunicacio.log

CREATE OR REPLACE PACKAGE ELE.PKG_LINIACOMUNICACIO AS

--
-- NOM.........:   PKG_LINIACOMUNICACIO
-- FINALITAT...:   GESTIO DE TOTA LA INFORMACIO REFERENTS A LINIES COMUNICACIO
--
--
--
--
-- NOM.........:   PR_ALTA_LINIACOMUNICACIO
-- FINALITAT...:   DONAR D'ALTA UNA NOVA LINIA DE COMUNICACIO
-- CONTROLS....:   INFORMAR CAMPS OBLIGATORIS
--                 REFERENCIA NO REPETIDA
--                 CAPACITAT MAXIMA
--              
-- PARAMETRES..:   
--         IN..:   idReferencia.....: Codi de la linia de comunicacio (Obligatori)
--	    	       CapacitatMaxima..: Capacitat maxima a transportar (Obliagtoir)
--        OUT..:   RSP
--
-- PRECONDICIO.:   LINIA NO EXISTEIX
-- POSTCONDICIO:   ES DONA D'ALTA UNA NOVA LINEA DE COMUNICACIO
-- RETURN .....:   OK SI S'HA EFECTUAT L'ALTA
--                 ERROR ALGUN CAMP DEL PARAMETRE NO TE VALOR
--                 ERROR REGISTRE DUPLICAT (LINIA COMUNICACIO EXISTENT)
--                 ERROR SI NO TE SUMINISTRAMENT ELECTRIC.
--
--

PROCEDURE PR_ALTA_LINCOMUNICACIO (inReferencia     IN LINIACOMUNICACIO.IDREFERENCIA%TYPE,
                                  inCapacitat      IN LINIACOMUNICACIO.CAPACITATMAXIMA%TYPE,
                                  RSP              OUT VARCHAR2);

--
-- NOM.........:   PR_BAIXA_LINIACOMUNICACIO
-- FINALITAT...:   DONAR DE BAIXA UNA LINIA COMUNICACIO
-- CONTROLS....:   LINIA NO DONADA DE BAIXA
--                 LINIA EXISTEIX EN EL FITXER-
--                 LINIA SENSE CENTRALS DE COMUNICACIO.
--              
-- PARAMETRES..:   
--         IN..:   idReferencia: Codi de la linia de comunicacio (Obligatori)
--        OUT..:   RSP
--
-- PRECONDICIO.:   LINIA COMUNICACIO EXISTEIX
-- POSTCONDICIO:   ES DONA DE BAIXA LA LINIA DE COMUNICACIO
-- RETURN .....:   OK SI S'HA EFECTUAT LA BAIXA
--                 ERROR SI LA LINIA DE COMUNICACIO NO EXISTEIX
--                 ERROR SI LA LINIA DE COMUNICACIO ESTA DE BAIXA              
--                 ERROR SI LA LINIA DE COMUNICACIO TE CENTRALS DE PRODUCCIO ACTIVES.
--
--

  PROCEDURE PR_BAIXA_LINCOMUNICACIO (inLiniaComunicacio      IN LINIACOMUNICACIO.IDLINIACOMUNICACIO%TYPE,
                                     RSP                     OUT VARCHAR2);

--
-- NOM.........:   PR_MODIFICACIO_LINIACOMUNICACIO
-- FINALITAT...:   PODER MODIFICAR QUALSEVOL CAMP D'UNA LINIA DE COMUNICACIO
-- CONTROLS....:   LINIA COMUNICACIO EXISTENT EN EL FITXER
--                 INFORMAR CAMPS OBLIGATORIS
--                 LINIA COMUNICACIO NO REPETIT
--              
-- PARAMETRES..:   
--         IN..:   idliniacomunicacio: codi de la linia de comunicacio (obligatori)
--                 idreferencia .....: referencia de la linia de comunicacio 
--                 capacitatmaxima...: capacitat maxima a transportar per la linia
--                 capacitatproduida.: capacitat produida
--                 DataAlta..........: Canviar la data d'alta de la linia de comunicacio
--                 DataBaixa.........: Canviar la data de baixa de la linia de comunicacio
--        OUT..:   RSP
--
-- PRECONDICIO.:   LINIA DE COMUNICACIO EXISTEIX
-- POSTCONDICIO:   ES REALITZA LA MODIFICACIO DELS CAMPS DE LA LINIA DE COMUNICACIO
-- RETURN .....:   OK SI S'HA EFECTUAT LA MODIFICACIO
--                 ERROR ALGUN CAMP DEL PARAMETRE NO TE VALOR
--                 ERROR REGISTRE DUPLICAT (LINIA COMUNICACIO EXISTENT)
--                 ERROR ALTA NO CONTE VALORS 0 - 1
--

  PROCEDURE PR_MODIFICACIO_LINCOMUNICACIO (inLiniaComunicacio      IN LINIACOMUNICACIO.IDLINIACOMUNICACIO%TYPE,
                                           inReferencia            IN LINIACOMUNICACIO.IDREFERENCIA%TYPE,
                                           inCapacitat             IN LINIACOMUNICACIO.CAPACITATMAXIMA%TYPE,
                                           inProduida              IN LINIACOMUNICACIO.CAPACITATPRODUIDA%TYPE,
                                           inDataAlta              IN LINIACOMUNICACIO.DATAALTA%TYPE,
                                           inDataBaixa             IN LINIACOMUNICACIO.DATABAIXA%TYPE,
                                           RSP                     OUT VARCHAR2);

END PKG_LINIACOMUNICACIO;

--
--
--
CREATE OR REPLACE PACKAGE BODY ELE.PKG_LINIACOMUNICACIO AS

PROCEDURE PR_ALTA_LINCOMUNICACIO (inReferencia     IN LINIACOMUNICACIO.IDREFERENCIA%TYPE,
                                  inCapacitat      IN LINIACOMUNICACIO.CAPACITATMAXIMA%TYPE,
                                  RSP              OUT VARCHAR2)
  IS

   V_CAMP_NUL                  EXCEPTION;
   V_LINCOMUNICACIO_EXISTENT   EXCEPTION;

   v_in          VARCHAR2(500);
   hihies        VARCHAR2(1);
   err_num       NUMBER;
   err_msg       VARCHAR2(100);

   BEGIN
     v_in := 'Referencia:'||inReferencia||', Capacitat Maxima:'||inCapacitat;
--
-- Primera comprovacio que tots els camps obligatoris tinguin valor
--
     IF (TRIM(inReferencia)     IS NULL) THEN
         RAISE V_CAMP_NUL;
     END IF;
     IF (TRIM(inCapacitat)     IS NULL) THEN
         RAISE V_CAMP_NUL;
     END IF;
--
-- Segona comprovacio que la Referenciano es trovi amb un altre id.
--
     BEGIN
       SELECT 'Y'
         INTO hihies
         FROM LINIACOMUNICACIO
        WHERE IDREFERENCIA  = inReferencia
          AND ROWNUM  = 1;
        RAISE V_LINCOMUNICACIO_EXISTENT;
    EXCEPTION
         WHEN NO_DATA_FOUND THEN NULL;
    END;
--
-- Tots els controls superats.
--
      INSERT INTO LINIACOMUNICACIO (IDREFERENCIA, CAPACITATMAXIMA, CAPACITATPRODUIDA, CONSUM, ACTIU, DATAALTA)
           VALUES (inReferencia, inCapacitat, 0, 0, 1, SYSDATE);

      RSP := 'OK';
      INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_LINCOMUNICACIO.PR_ALTA_LINCOMUNICACIO', v_in, 'RSP: '||RSP);
      COMMIT;
  EXCEPTION
    WHEN V_LINCOMUNICACIO_EXISTENT THEN
         RSP := 'ERROR: La linia de comunicacio ya esta donada d`alta';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_LINCOMUNICACIO.PR_ALTA_LINCOMUNICACIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_CAMP_NUL THEN
         RSP := 'ERROR: No pot haver camps NULL';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_LINCOMUNICACIO.PR_ALTA_LINCOMUNICACIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN OTHERS THEN
         err_num := SQLCODE;
         err_msg := SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_LINCOMUNICACIO.PR_ALTA_LINCOMUNICACIO', v_in, 'RSP: '||RSP);
         COMMIT;
  END;

  PROCEDURE PR_BAIXA_LINCOMUNICACIO (inLiniaComunicacio      IN LINIACOMUNICACIO.IDLINIACOMUNICACIO%TYPE,
                                     RSP                     OUT VARCHAR2)
  IS

   V_CAMP_NUL                  EXCEPTION;
   V_LINCOMUNICACIO_EXISTENT   EXCEPTION;
   V_LINCOMUNICACIO_BAIXA      EXCEPTION;
   V_LINCOMUNICACIO_DISTRI     EXCEPTION;
   V_CENTRAL_ACTIVA            EXCEPTION;

   v_in           VARCHAR2(500);
   v_consum       NUMBER;
   v_total        NUMBER;
   hihies         VARCHAR2(1);
   v_actiu        NUMBER;
   err_num        NUMBER;
   err_msg        VARCHAR2(100);

   BEGIN
     v_in := 'Linia comunicacio:'||inLiniaComunicacio;
--
-- Primera comprovacio que tots els camps obligatoris tinguin valor.
--
     IF (TRIM(inLiniaComunicacio)     IS NULL) THEN
         RAISE V_CAMP_NUL;
     END IF;
--
-- Segona comprovacio que el idLiniaComunicacio existeix
--
     BEGIN
       SELECT Actiu,Consum
         INTO v_actiu,v_consum
         FROM LINIACOMUNICACIO
        WHERE IDLINIACOMUNICACIO  = inLiniaComunicacio
          AND ROWNUM  = 1;        
    EXCEPTION
         WHEN NO_DATA_FOUND THEN RAISE V_LINCOMUNICACIO_EXISTENT;
    END;	 
--
-- Tercera comprovacio que no estigui de baixa
--	 
     BEGIN	 
        IF (V_ACTIU = 0) THEN
          RAISE V_LINCOMUNICACIO_BAIXA;
        END IF;     
     END;
-- Quarta comprovaci que no tingui centrals de distribucio actives.
--
    BEGIN
      SELECT 'Y'
	    INTO hihies
	   FROM DISTRIBUCIO
	  WHERE ACTIU = 1
	    AND IDDISTRIBUCIO IN (SELECT DISTINCT(IDDISTRIBUCIO) FROM COMUNICACIOPRODUCCIO WHERE ACTIU = 1 AND IDLINIACOMUNICACIO = inLiniaComunicacio)
		AND ROWNUM = 1;
	  RAISE V_LINCOMUNICACIO_DISTRI;
	EXCEPTION
	   WHEN NO_DATA_FOUND THEN NULL;	   	   
	END;	 
--	
-- Tots els controls superats.
--
      UPDATE LINIACOMUNICACIO
         SET ACTIU = 0,
             DATABAIXA = SYSDATE
       WHERE IDLINIACOMUNICACIO = INLINIACOMUNICACIO;
      RSP := 'OK';
      INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_LINCOMUNICACIO.PR_BAIXA_LINCOMUNICACIO', v_in, 'RSP: '||RSP);
      COMMIT;
  EXCEPTION
    WHEN V_LINCOMUNICACIO_DISTRI THEN
         RSP := 'ERROR: La linia comunicacio te centrals de distirbucio actives';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_LINCOMUNICACIO.PR_BAIXA_LINCOMUNICACIO', v_in, 'RSP: '||RSP);
         COMMIT;	
    WHEN V_LINCOMUNICACIO_EXISTENT THEN
         RSP := 'ERROR: La linia comunicacio no esta donat d`alta';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_LINCOMUNICACIO.PR_BAIXA_LINCOMUNICACIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_LINCOMUNICACIO_BAIXA THEN
         RSP := 'ERROR: La linia comunicacio ja esta esborrada';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_LINCOMUNICACIO.PR_BAIXA_LINCOMUNICACIO', v_in, 'RSP: '||RSP);
         COMMIT;	  
    WHEN V_CENTRAL_ACTIVA THEN
         RSP := 'ERROR: Hi ha centrals de produccio en la linia';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_LINCOMUNICACIO.PR_BAIXA_LINCOMUNICACIO', v_in, 'RSP: '||RSP);
         COMMIT;	  
    WHEN V_CAMP_NUL THEN
         RSP := 'ERROR: No pot haver camps NULL';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_LINCOMUNICACIO.PR_BAIXA_LINCOMUNICACIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN OTHERS THEN
         err_num := SQLCODE;
         err_msg := SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_LINCOMUNICACIO.PR_BAIXA_LINCOMUNICACIO', v_in, 'RSP: '||RSP);
         COMMIT;
  END;

  PROCEDURE PR_MODIFICACIO_LINCOMUNICACIO (inLiniaComunicacio      IN LINIACOMUNICACIO.IDLINIACOMUNICACIO%TYPE,
                                           inReferencia            IN LINIACOMUNICACIO.IDREFERENCIA%TYPE,
                                           inCapacitat             IN LINIACOMUNICACIO.CAPACITATMAXIMA%TYPE,
                                           inProduida              IN LINIACOMUNICACIO.CAPACITATPRODUIDA%TYPE,
                                           inDataAlta              IN LINIACOMUNICACIO.DATAALTA%TYPE,
                                           inDataBaixa             IN LINIACOMUNICACIO.DATABAIXA%TYPE,
                                           RSP                     OUT VARCHAR2)
   IS

   V_CAMP_NUL                  EXCEPTION;
   V_LINCOMUNICACIO_EXISTENT   EXCEPTION;
   V_LINCOMUNICACIO_REPETIT    EXCEPTION;

   v_in          VARCHAR2(500);
   hihies        VARCHAR2(1);
   err_num       NUMBER;
   err_msg       VARCHAR2(100);


   BEGIN
     v_in := 'Linia comunicacio:'||inLIniaComunicacio||', Referencia:'||inReferencia||', Capacitat:'||inCapacitat||', Produccio:'||inProduida||
             ', Data Alta:'||inDataAlta||', Data Baixa:'||inDataBaixa;
--
-- Primera comprovacio que tots els camps obligatoris tinguin valor
--
     IF (TRIM(inLiniaComunicacio)     IS NULL) THEN
         RAISE V_CAMP_NUL;
     END IF;
--
-- Segona comprovacio que el idLiniacomunicacio existeix
--
     BEGIN
       SELECT 'Y'
         INTO hihies
         FROM LINIACOMUNICACIO
        WHERE IDLINIACOMUNICACIO  = inLiniaComunicacio
          AND ROWNUM  = 1;        
    EXCEPTION
         WHEN NO_DATA_FOUND THEN RAISE V_LINCOMUNICACIO_EXISTENT;
    END;	 	 
--
-- Tercera comprovacio
--	 
     BEGIN
       SELECT 'Y'
         INTO hihies
         FROM LINIACOMUNICACIO
        WHERE IDREFERENCIA  = inReferencia
          AND IDLINIACOMUNICACIO <> inLiniaComunicacio
          AND ROWNUM  = 1;
        RAISE V_LINCOMUNICACIO_REPETIT;
    EXCEPTION
         WHEN NO_DATA_FOUND THEN NULL;
    END;
--
-- Tots els controls superats.
--
      UPDATE LINIACOMUNICACIO SET
             idReferencia = NVL(inReferencia, idReferencia),
	     	 CapacitatMaxima = NVL(inCapacitat, CapacitatMaxima),
     		 CapacitatProduida = NVL(inProduida, CapacitatProduida),
    		 DataAlta = NVL(inDataAlta, DataAlta),
	    	 DataBaixa = NVL(inDataBaixa, DataBaixa)
	    WHERE idLiniaComunicacio = inLiniaComunicacio;
      RSP := 'OK';
      INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_LINCOMUNICACIO.PR_MODIFICACIO_LINCOMUNICACIO', v_in, 'RSP: '||RSP);
      COMMIT;
  EXCEPTION
    WHEN V_LINCOMUNICACIO_REPETIT THEN
         RSP := 'ERROR: La Referencia ya esta assignada a un altre linia comunicacio';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_LINCOMUNICACIO.PR_MODIFICACIO_LINCOMUNICACIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_LINCOMUNICACIO_EXISTENT THEN
         RSP := 'ERROR: La linia comunicacio no esta donada d`alta';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_LINCOMUNICACIO.PR_MODIFICACIO_LINCOMUNICACIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN V_CAMP_NUL THEN
         RSP := 'ERROR: No pot haver camps NULL';
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_LINCOMUNICACIO.PR_MODIFICACIO_LINCOMUNICACIO', v_in, 'RSP: '||RSP);
         COMMIT;
    WHEN OTHERS THEN
         err_num := SQLCODE;
         err_msg := SUBSTR(SQLERRM, 1, 200);
         RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg; 
         INSERT INTO LOGS (DATA, PROCEDIMENT, PARAMSIN, PARAMSOUT)
           VALUES (SYSDATE,'PKG_LINCOMUNICACIO.PR_MODIFICACIO_LINCOMUNICACIO', v_in, 'RSP: '||RSP);
         COMMIT;
  END;
END PKG_LINIACOMUNICACIO;
--
-- FI DEL PACKAGE
--
spool off;
